{--------------------------------------------------------------------------------
* Description : Examples how to use the TscExcelExport component
* Author : Stefan Cruysberghs
* Email : stefancr@yucom.be
* Website : http://users.pandora.be/stefancr (http://www.stefancr.yucom.be)
--------------------------------------------------------------------------------}

unit MainExcelExport;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Db, DBTables, scExcelExport, Grids, DBGrids, ExtCtrls, Buttons,
  ComCtrls, OleServer, Excel2000;

  // Use the same Excel unit as you use in the scExcelExport component !

type
  TFormExcelExport = class(TForm)
    TableOrders: TTable;
    DBGrid1: TDBGrid;
    DataSource1: TDataSource;
    TableAnimals: TTable;
    TableVendors: TTable;
    LabelHeaderFont: TLabel;
    LabelTitleFont: TLabel;
    PanelSubTitle: TPanel;
    BitBtnExportDataset: TBitBtn;
    StatusBar: TStatusBar;
    BitBtnSaveXLS: TBitBtn;
    BitBtnPrintPreview: TBitBtn;
    BitBtnExportPropeties: TBitBtn;
    BitBtnExportConnectTo: TBitBtn;
    PanelTitle: TPanel;
    scExcelExport1: TscExcelExport;
    BitBtnExportDatasetColors: TBitBtn;
    scExcelExport2: TscExcelExport;
    BitBtnExportEvents: TBitBtn;
    BitBtnExportExistingFile: TBitBtn;
    BitBtnCloseAll: TBitBtn;
    BitBtnExportGroups: TBitBtn;
    QuerySortShipVia: TQuery;
    BitBtnAccessingExcel: TBitBtn;
    TableOrdersOrderNo: TFloatField;
    TableOrdersCustNo: TFloatField;
    TableOrdersSaleDate: TDateTimeField;
    TableOrdersShipDate: TDateTimeField;
    TableOrdersEmpNo: TIntegerField;
    TableOrdersShipToContact: TStringField;
    TableOrdersShipToAddr1: TStringField;
    TableOrdersShipToAddr2: TStringField;
    TableOrdersShipToCity: TStringField;
    TableOrdersShipToState: TStringField;
    TableOrdersShipToZip: TStringField;
    TableOrdersShipToCountry: TStringField;
    TableOrdersShipToPhone: TStringField;
    TableOrdersShipVIA: TStringField;
    TableOrdersPO: TStringField;
    TableOrdersTerms: TStringField;
    TableOrdersPaymentMethod: TStringField;
    TableOrdersItemsTotal: TCurrencyField;
    TableOrdersTaxRate: TFloatField;
    TableOrdersFreight: TCurrencyField;
    TableOrdersAmountPaid: TCurrencyField;
    procedure scExcelExport1ExportRecords(Sender: TObject; IntRecordNumber: Integer);
    procedure BitBtnExportDatasetClick(Sender: TObject);
    procedure BitBtnSaveXLSClick(Sender: TObject);
    procedure BitBtnPrintPreviewClick(Sender: TObject);
    procedure BitBtnExportPropetiesClick(Sender: TObject);
    procedure BitBtnExportConnectToClick(Sender: TObject);
    procedure BitBtnExportDatasetColorsClick(Sender: TObject);
    procedure BitBtnExportEventsClick(Sender: TObject);
    procedure scExcelExport2GetEOF(Sender: TObject; var BlnEOF: Boolean);
    procedure scExcelExport2GetFieldCount(Sender: TObject;
      var IntFieldCount: Integer);
    procedure scExcelExport2GetFieldDataSize(Sender: TObject;
      const FieldIndex: Integer; var IntFieldDataSize: Integer);
    procedure scExcelExport2GetFieldDataType(Sender: TObject;
      const FieldIndex: Integer; var FieldDataType: TFieldType);
    procedure scExcelExport2GetFieldDisplayName(Sender: TObject;
      const FieldIndex: Integer; var StrFieldDisplayName: String);
    procedure scExcelExport2GetFieldName(Sender: TObject;
      const FieldIndex: Integer; var StrFieldName: String);
    procedure scExcelExport2GetFieldValue(Sender: TObject;
      const FieldIndex: Integer; var VarValue: Variant);
    procedure scExcelExport2GetFieldVisible(Sender: TObject;
      const FieldIndex: Integer; var BlnFieldVisible: Boolean);
    procedure scExcelExport2ExportRecords(Sender: TObject;
      IntRecordNumber: Integer);
    procedure BitBtnExportExistingFileClick(Sender: TObject);
    procedure BitBtnCloseAllClick(Sender: TObject);
    procedure BitBtnExportGroupsClick(Sender: TObject);
    procedure scExcelExport2GotoFirstRecord(Sender: TObject);
    procedure scExcelExport2GotoNextRecord(Sender: TObject);
    procedure BitBtnAccessingExcelClick(Sender: TObject);
  private
    procedure ChangeCellColors(Sender: TObject; Field: TField;
      var ColorBackground: TColor);
  public
  end;

var
  FormExcelExport: TFormExcelExport;

implementation

{$R *.DFM}

//-----------------------------------------------------------------------------
procedure TFormExcelExport.scExcelExport1ExportRecords(Sender: TObject;
  IntRecordNumber: Integer);
begin
  StatusBar.Panels[0].Text := 'Records : '+IntToStr(IntRecordNumber);
end;

//-----------------------------------------------------------------------------
// Export dataset to Excel and show duration
//-----------------------------------------------------------------------------
procedure TFormExcelExport.BitBtnExportDatasetClick(Sender: TObject);
var
  Duration : TDateTime;
begin
  try
    scExcelExport1.ExcelVisible:=True;
    //scExcelExport1.LoadDefaultProperties;
    scExcelExport1.Dataset:=TableOrders;
    scExcelExport1.WorksheetName:='TscExcelExport example';
    scExcelExport1.ConnectTo := ctNewExcel;
    Duration := Now();
    scExcelExport1.ExportDataset;
    StatusBar.Panels[1].Text := 'Duration : '+TimeToStr(Now() - Duration);
  finally
    scExcelExport1.Disconnect;
  end;
end;

//-----------------------------------------------------------------------------
// Export dataset to Excel, adding summaries and using all the layout properties
//-----------------------------------------------------------------------------
procedure TFormExcelExport.BitBtnExportPropetiesClick(Sender: TObject);
begin
  try
    scExcelExport1.LoadDefaultProperties;
    scExcelExport1.WorksheetName := 'TscExcelExport layout example';
    scExcelExport1.Dataset:=TableOrders;
    scExcelExport1.StyleColumnWidth:=cwOwnerWidth;
    scExcelExport1.ColumnWidth := 20;

    scExcelExport1.HeaderText.Text:= 'Header';
    scExcelExport1.HeaderText.Add('Header - Line2');
    scExcelExport1.HeaderText.Add('Header - Line3');
    scExcelExport1.FooterText.Add('Footer - Line1');
    scExcelExport1.FooterText.Add('Footer - Line2');
    scExcelExport1.BeginRowHeader := 5;
    scExcelExport1.BorderHeader.BackColor := clGreen;
    scExcelExport1.FontHeader := TxlFont(LabelHeaderFont.Font);

    scExcelExport1.BeginRowTitles := 5;
    scExcelExport1.FontTitles := TxlFont(LabelTitleFont.Font);
    scExcelExport1.FontTitles.Orientation := 45;
    scExcelExport1.BorderTitles.BackColor := clYellow;
    scExcelExport1.BorderTitles.BorderColor := clRed;
    scExcelExport1.BorderTitles.LineStyle := blDouble;
    scExcelExport1.BorderTitles.Weight := bwThick;

    scExcelExport1.BeginRowData := 8;
    scExcelExport1.SummarySelection := ssValues;
    scExcelExport1.SummaryCalculation := scMAX;
    scExcelExport1.BorderSummary.BackColor := clYellow;
    scExcelExport1.BorderSummary.BorderColor := clRed;
    scExcelExport1.BorderSummary.LineStyle := blLine;
    scExcelExport1.BorderSummary.Weight := bwThick;

    scExcelExport1.BorderData.LineStyle := blDouble;    

    scExcelExport1.ExcelVisible:=True;
    scExcelExport1.ExportDataset;
  finally
    scExcelExport1.Disconnect;
  end;
end;

//-----------------------------------------------------------------------------
// Export dataset to Excel without showing Excel and save worksheet to file
//-----------------------------------------------------------------------------
procedure TFormExcelExport.BitBtnSaveXLSClick(Sender: TObject);
begin
  try
    scExcelExport1.LoadDefaultProperties;
    scExcelExport1.ExcelVisible:=False;
    scExcelExport1.WorksheetName := 'TscExcelExport example';
    scExcelExport1.Dataset:=TableOrders;
    scExcelExport1.ExportDataset;
    scExcelExport1.SaveAs('c:\ExcelExport.xls',ffXLS);
    //scExcelExport1.SaveAs('c:\ExcelExport.htm',ffHTM);
    //scExcelExport1.SaveAs('c:\ExcelExport.csv',ffCSV);
  finally
    scExcelExport1.Disconnect;
  end;
end;

//-----------------------------------------------------------------------------
// Export dataset to Excel and show print preview
//-----------------------------------------------------------------------------
procedure TFormExcelExport.BitBtnPrintPreviewClick(Sender: TObject);
begin
  try
    scExcelExport1.LoadDefaultProperties;
    scExcelExport1.ExcelVisible:=False;
    scExcelExport1.WorksheetName := 'TscExcelExport example';
    scExcelExport1.Dataset:=TableOrders;
    scExcelExport1.ExportDataset;
    scExcelExport1.PrintPreview(True);
  finally
    scExcelExport1.Disconnect;
  end;
end;

//-----------------------------------------------------------------------------
// Export datasets to Excel, using the different connect properties
//-----------------------------------------------------------------------------
procedure TFormExcelExport.BitBtnExportConnectToClick(Sender: TObject);
begin
  try
    scExcelExport1.ExcelVisible:=True;
    scExcelExport1.LoadDefaultProperties;

    // Start excel and create new workbook and worksheet Orders
    scExcelExport1.Dataset:=TableOrders;
    scExcelExport1.WorksheetName:='Orders';
    scExcelExport1.ConnectTo := ctNewExcel;
    scExcelExport1.ShowTitles := False;
    scExcelExport1.ExportDataset;

    // Create new workbook and new worksheet Animals in active excel
    scExcelExport1.Disconnect;
    scExcelExport1.Dataset:=TableAnimals;
    scExcelExport1.WorksheetName:='Animals';
    scExcelExport1.ConnectTo := ctNewWorkbook;
    scExcelExport1.ShowTitles := False;
    scExcelExport1.ExportDataset;

    // Create new worksheet Vendors in active workbook in active excel
    scExcelExport1.Disconnect;
    scExcelExport1.Dataset:=TableVendors;
    scExcelExport1.WorksheetName:='Vendors';
    scExcelExport1.ConnectTo := ctNewWorksheet;
    scExcelExport1.ShowTitles := True;
    scExcelExport1.ExportDataset;

    // Add data (of vendors) in existing worksheet Animals
    scExcelExport1.Disconnect;
    scExcelExport1.Dataset:=TableVendors;
    scExcelExport1.WorksheetName:='Animals';
    scExcelExport1.ConnectTo := ctNewWorksheet;
    scExcelExport1.BeginColumnData := 10;
    scExcelExport1.ShowTitles := True;
    scExcelExport1.ExportDataset;
  finally
    scExcelExport1.Disconnect;
  end;
end;

//-----------------------------------------------------------------------------
procedure TFormExcelExport.ChangeCellColors(Sender: TObject; Field: TField; var ColorBackground: TColor);
begin
  if Field.FieldName = 'CustNo' then
  begin
    if Field.Value > 2000 then
      ColorBackground := clRed;
    if Field.Value > 3000 then
      ColorBackground := clGreen;
  end;

  if Field.FieldName = 'EmpNo' then
  begin
    if Field.Value > 100 then
      ColorBackground := clYellow;
  end;
end;

//-----------------------------------------------------------------------------
// Export datasets to Excel and change background colors of cells
// the depending values of the fields (see procedure ChangeCellColors)
//-----------------------------------------------------------------------------
procedure TFormExcelExport.BitBtnExportDatasetColorsClick(Sender: TObject);
begin
  try
    scExcelExport1.ExcelVisible:=True;
    scExcelExport1.LoadDefaultProperties;
    scExcelExport1.Dataset:=TableOrders;
    scExcelExport1.WorksheetName:='TscExcelExport colors example';
    scExcelExport1.ConnectTo := ctNewExcel;
    scExcelExport1.OnGetCellBackgroundColorEvent := ChangeCellColors;
    scExcelExport1.ExportDataset;
  finally
    scExcelExport1.Disconnect;
    scExcelExport1.OnGetCellBackgroundColorEvent := nil;
  end;
end;

//-----------------------------------------------------------------------------
// Export data to Excel by using the export events
// No dataset is connected to the TscExcelExport component
// Fieldnames, FieldValues, EOF, ... will be retrieved in the triggered events
//-----------------------------------------------------------------------------
procedure TFormExcelExport.BitBtnExportEventsClick(Sender: TObject);
var
  Duration : TDateTime;
begin
  try
    TableOrders.DisableControls;
    TableOrders.First;
    scExcelExport2.ExcelVisible:=True;
    scExcelExport2.LoadDefaultProperties;
    scExcelExport2.ConnectTo := ctNewExcel;
    scExcelExport2.DataPipe := dpCustom;
    scExcelExport2.GroupFields.Clear;
    scExcelExport2.GroupFields.Add('ShipVia');
    Duration := Now();
    scExcelExport2.ExportDataset;
    StatusBar.Panels[1].Text := 'Duration : '+TimeToStr(Now() - Duration);
  finally
    scExcelExport2.Disconnect;
    TableOrders.EnableControls;
  end;
end;

//-----------------------------------------------------------------------------
procedure TFormExcelExport.scExcelExport2GetEOF(Sender: TObject; var BlnEOF: Boolean);
begin
  BlnEOF := QuerySortShipVia.Eof;
end;

//-----------------------------------------------------------------------------
procedure TFormExcelExport.scExcelExport2GetFieldCount(Sender: TObject;
  var IntFieldCount: Integer);
begin
  IntFieldCount := QuerySortShipVia.FieldCount;
end;

//-----------------------------------------------------------------------------
procedure TFormExcelExport.scExcelExport2GetFieldDataSize(Sender: TObject;
  const FieldIndex: Integer; var IntFieldDataSize: Integer);
begin
  IntFieldDataSize := QuerySortShipVia.Fields[FieldIndex].Size;
end;

//-----------------------------------------------------------------------------
procedure TFormExcelExport.scExcelExport2GetFieldDataType(Sender: TObject;
  const FieldIndex: Integer; var FieldDataType: TFieldType);
begin
  FieldDataType := QuerySortShipVia.Fields[FieldIndex].DataType;
end;

//-----------------------------------------------------------------------------
procedure TFormExcelExport.scExcelExport2GetFieldDisplayName(
  Sender: TObject; const FieldIndex: Integer;
  var StrFieldDisplayName: String);
begin
  StrFieldDisplayName := '* '+QuerySortShipVia.Fields[FieldIndex].FieldName+' *';
end;

//-----------------------------------------------------------------------------
procedure TFormExcelExport.scExcelExport2GetFieldName(Sender: TObject;
  const FieldIndex: Integer; var StrFieldName: String);
begin
  StrFieldName := QuerySortShipVia.Fields[FieldIndex].FieldName;
end;

//-----------------------------------------------------------------------------
procedure TFormExcelExport.scExcelExport2GetFieldValue(Sender: TObject;
  const FieldIndex: Integer; var VarValue: Variant);
begin
  VarValue := QuerySortShipVia.Fields[FieldIndex].Value;
end;

//------------------------------------------------------------------------------
procedure TFormExcelExport.scExcelExport2GotoFirstRecord(Sender: TObject);
begin
  QuerySortShipVia.First;
end;

//------------------------------------------------------------------------------
procedure TFormExcelExport.scExcelExport2GotoNextRecord(Sender: TObject);
begin
  QuerySortShipVia.Next;
end;

//-----------------------------------------------------------------------------
procedure TFormExcelExport.scExcelExport2GetFieldVisible(Sender: TObject;
  const FieldIndex: Integer; var BlnFieldVisible: Boolean);
begin
  BlnFieldVisible := True;
end;

//-----------------------------------------------------------------------------
procedure TFormExcelExport.scExcelExport2ExportRecords(Sender: TObject;
  IntRecordNumber: Integer);
begin
  StatusBar.Panels[0].Text := 'Records : '+IntToStr(IntRecordNumber);
end;


//------------------------------------------------------------------------------
procedure TFormExcelExport.BitBtnExportExistingFileClick(Sender: TObject);
begin
  try
    scExcelExport1.LoadDefaultProperties;
    scExcelExport1.ExcelVisible:=True;
    scExcelExport1.ConnectTo:=ctNewExcel;
    scExcelExport1.Dataset:=TableAnimals;
    // Open this file
    scExcelExport1.Filename:='c:\ExcelExport.xls';
    // Add data to existing worksheet
    scExcelExport1.WorksheetName:='TscExcelExport example';
    scExcelExport1.BeginColumnData := 30;
    scExcelExport1.ExportDataset;
  finally
    scExcelExport1.Disconnect;
  end;
end;

//------------------------------------------------------------------------------
procedure TFormExcelExport.BitBtnCloseAllClick(Sender: TObject);
begin
  scExcelExport1.CloseAllExcelApps;
end;

//------------------------------------------------------------------------------
procedure TFormExcelExport.BitBtnExportGroupsClick(Sender: TObject);
begin
  try
    scExcelExport1.ExcelVisible:=True;
    scExcelExport1.LoadDefaultProperties;
    scExcelExport1.Dataset:=QuerySortShipVia;
    
    scExcelExport1.SummarySelection := ssValues;
    scExcelExport1.SummaryCalculation := scSUM;
    scExcelExport1.BorderSummary.BackColor := clGreen;
    scExcelExport1.BorderSummary.BorderColor := clRed;
    scExcelExport1.BorderSummary.LineStyle := blLine;
    scExcelExport1.BorderSummary.Weight := bwThick;

    scExcelExport1.FontGroup := TxlFont(LabelTitleFont.Font);
    scExcelExport1.BorderGroup.BackColor := clYellow;
    scExcelExport1.WorksheetName:='TscExcelExport grouping example';
    scExcelExport1.ConnectTo := ctNewExcel;
    scExcelExport1.StyleColumnWidth := cwEnhAutoFit;
    scExcelExport1.GroupFields.Clear;
    scExcelExport1.GroupFields.Add('ShipVia');
    scExcelExport1.GroupFields.Add('Terms');
    //scExcelExport1.GroupFields.Add('OrderNo');

    scExcelExport1.BorderHeader.BackColor := clGreen;
    scExcelExport1.FontHeader := TxlFont(LabelHeaderFont.Font);
    scExcelExport1.HeaderText.Text:= 'Header';
    scExcelExport1.HeaderText.Add('Header - Line2');
    scExcelExport1.HeaderText.Add('Header - Line3');
    scExcelExport1.FooterText.Add('Footer - Line1');
    scExcelExport1.FooterText.Add('Footer - Line2');
    scExcelExport1.BeginRowHeader := 3;

    scExcelExport1.ExportDataset;
  finally
    scExcelExport1.Disconnect;
  end;
end;

//------------------------------------------------------------------------------
procedure TFormExcelExport.BitBtnAccessingExcelClick(Sender: TObject);
begin
  try
    scExcelExport1.ExcelVisible:=True;
    scExcelExport1.LoadDefaultProperties;
    scExcelExport1.Dataset:=TableOrders;
    scExcelExport1.WorksheetName:='TscExcelExport example';
    scExcelExport1.ConnectTo := ctNewExcel;
    scExcelExport1.Connect;
    scExcelExport1.ExcelWorkSheet.Range['A1','C10'].Borders.Color := clRed;

    scExcelExport1.ExportDataset;

    scExcelExport1.ExcelWorkSheet.Range['B5','E7'].Cells.Clear;

    scExcelExport1.ExcelWorkSheet.Range[Format('A%d',[scExcelExport1.EndRowData+3]),
      Format('A%d',[scExcelExport1.EndRowData+3])].Font.Size := 16;
    scExcelExport1.ExcelWorkSheet.Range[Format('A%d',[scExcelExport1.EndRowData+3]),
      Format('A%d',[scExcelExport1.EndRowData+3])].Value := 'Adding extra information to Excel worksheet';
  finally
    scExcelExport1.Disconnect;
  end;
end;

end.
