unit NeuralNet;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, TedRBFnet, ExtCtrls;

type
  TrbfDemoForm = class(TForm)
    initialisingGroupBox: TGroupBox;
    chooseTrainingButton: TButton;
    centerMethodLabel: TLabel;
    centerMethodComboBox: TComboBox;
    topologyLabel: TLabel;
    learningGroupBox: TGroupBox;
    learningButton: TButton;
    errorCheckBox: TCheckBox;
    verifyingGroupBox: TGroupBox;
    trainingOutputButton: TButton;
    testButton: TButton;
    rbfnet: TTedRBFNet;
    opendialog: TOpenDialog;
    numbOfInputsEdit: TEdit;
    numberOfInputsLabel: TLabel;
    numbOfHiddensEdit: TEdit;
    numberOfHiddensLabel: TLabel;
    numbOfOutputsEdit: TEdit;
    NumberOfOutputsLabel: TLabel;
    numbOfCyclesLabel: TLabel;
    numbOfCyclesEdit: TEdit;
    ErrorBoundLabel: TLabel;
    ErrorBoundEdit: TEdit;
    // let the net learn, put errors into file
    procedure learningButtonClick(Sender: TObject);

    //compare training data with net behaviour
    procedure trainingOutputButtonClick(Sender: TObject);

    //get training data and initialise the net
    procedure chooseTrainingButtonClick(Sender: TObject);

    //check net with test data
    procedure testButtonClick(Sender: TObject);
    procedure centerMethodComboBoxChange(Sender: TObject);
    procedure numbOfHiddensEditOnExit(Sender: TObject);
    procedure numbOfCyclesEditOnExit(Sender: TObject);
    procedure rbfDemoFormCreate(Sender: TObject);
    procedure rbfDemoFromClose(Sender: TObject; var Action: TCloseAction);
    procedure numbOfInputsEditOnExit(Sender: TObject);
    procedure numbOfOutputsEditOnExit(Sender: TObject);
    procedure errorBoundEditOnExit(Sender: TObject);


  private
    FNumberOfLearnCycles: integer;
    FErrorBound: double;
    //for training, test data
    procedure ParseTextFileIntoDoubles(const fileName: string;
                        var doubles: TDoubleArray);

    //lets the net learn
    procedure learning();

    //gives outputs of the net with training values
    procedure trainingOutput();

    //initialises and trains the RBFnet
    procedure trainingAndInitNet();

    //tests the RBFnet
    procedure test();
  end;

var
  rbfDemoForm: TrbfDemoForm;

implementation

{$R *.dfm}


procedure TrbfDemoForm.learning();
var
  dest: TextFile;
  error: double;
  i, cycles: integer;
  cycleStr: String;
begin
  AssignFile (dest, 'errorStats.txt');
  Rewrite (dest);
  error := 0.0;
  cycles:=1;

  //no errorbound set?
  if not errorCheckBox.checked then
  begin
    for i:=1 to FNumberOfLearnCycles do
    begin
      rbfNet.learning(cycles);
      error := rbfNet.errorOfTrainingData();
      writeln(dest, FloatToStr(error));
    end;
    cycles:=FNumberOfLearnCycles;
  end
  else // error bounded
  begin
    error := rbfnet.ErrorOfTrainingData();
    i:=0;
    while (i < FNumberOfLearnCycles) and (error > FErrorBound) do
    begin
      rbfNet.learning(cycles,FErrorBound);
      i:=i+1;
      error := rbfnet.errorOfTrainingData();
      writeln(dest, FloatToStr(error));
    end;
    cycles:=i;
  end;
  closeFile(dest);
  if cycles = 1 then
    cycleStr:='cycle'
  else
    cycleStr:='cycles';
  ShowMessage('Learning finished after '+FloatToStr(cycles)+' '+
                        cycleStr+' with error '
                +FloatToStr(error));
end;


procedure TrbfDemoForm.learningButtonClick(Sender: TObject);

begin
   if not rbfnet.isInit() then
   begin
     showMessage('TrbfDemoForm.learningButtonClick: net not initialised');
     exit;
   end;
   learning();
end;

procedure TrbfDemoForm.trainingOutput();
var
  error: double;
  i,j: integer;
  outputNet: TDoubleArray;
  trainingPattern: TTrainingPattern;
  inputString, outputString, expectedOutputString: String;
begin
  rbfnet.getTrainingPattern(trainingPattern);
  setLength(outputNet,rbfnet.Topology.NumberOfOutputNeurons);
  for i := 0 to High(trainingPattern) do
  begin
    rbfnet.getOutput(trainingPattern[i].input,outputNet);
    inputString:='';
    for j:=0 to rbfnet.topology.numberOfInputNeurons-1 do
    begin
      inputString:=inputString+'   '+
                        format('%.5f',[trainingPattern[i].input[j]]);
    end;

    expectedOutputString:='';
    outputString:='';
    for j:=0 to rbfnet.topology.numberOfOutputNeurons-1 do
    begin
      expectedOutputString:=expectedOutputString+'   '+
        format('%.5f',[trainingPattern[i].output[j]]);
      error:=abs(trainingPattern[i].output[j]-outputNet[j]);
      outputString:=outputString+'   '+format('%.5f',[outputNet[j]])+
        ' '+'('+format('%.5f',[error])+')';
    end;

    ShowMessage('Training vector no. '+FloatToStr(i+1)+': '+ #13#10 +
                'Input: '+inputString+ #13#10
                +'Expected Output: '+expectedOutputString+#13#10
                +'Real Output (error): '+outputString);
  end;
end;

procedure TrbfDemoForm.trainingOutputButtonClick(Sender: TObject);

begin
  if not rbfnet.isInit then
  begin
    ShowMessage('TrbfDemoForm.trainingOutputButtonClick:'+
      ' net is not initialised');
    exit;
  end;
  trainingOutput();
end;

procedure TrbfDemoForm.trainingAndInitNet();
var
  numberOfTrainingValues: integer;
  doubles: TDoubleArray;
  messagStr: String;
begin
  ParseTextFileIntoDoubles(openDialog.FileName, doubles);
  //check for invalid datafile
  if (length(doubles) mod (rbfnet.Topology.NumberOfInputNeurons+
                  rbfnet.Topology.NumberOfOutputNeurons)> 0) then
  begin
    ShowMessage('TrbfDemoForm.trainingButtonClick: invalid test data');
    exit;
  end;
  rbfnet.SetTrainingPattern(doubles,numberOfTrainingValues);
  rbfnet.init();
  if rbfnet.isInit() then
  begin
    messagStr:='Trainingspattern parsed and RBF net initialised, ';
    messagStr:=messagStr+'number of training values: '
          +FloatToStr(numberOfTrainingValues);
    ShowMessage(messagStr);
  end;
end;

procedure TrbfDemoForm.chooseTrainingButtonClick(Sender: TObject);

begin
  if not (centerMethodComboBox.ItemIndex in [0..2]) then
  begin
    showMessage('TrbfDemoForm.chooseTrainingButtonClick: '+
      'must choose center method');
    exit;
  end;
  if openDialog.Execute then
  begin
    trainingAndInitNet();
  end;
end;

procedure TrbfDemoForm.test();
var
  doubles, testInput, outputNet: TDoubleArray;
  numberOfTestValues,counter,i,j: integer;
  inputString, outputString, expectedOutputString: String;
  error: double;
begin
  ParseTextFileIntoDoubles(openDialog.FileName, doubles);
  //check for invalid datafile
  if (length(doubles) mod (rbfnet.Topology.NumberOfInputNeurons+
                  rbfnet.Topology.NumberOfOutputNeurons)> 0) then
  begin
    ShowMessage('TrbfDemoForm.test: invalid test data');
    exit;
  end;
  numberOfTestValues:=length(doubles) div
        (rbfnet.Topology.NumberOfInputNeurons+
                rbfnet.Topology.NumberOfOutputNeurons);
  counter:=0;
  setLength(testInput,rbfnet.Topology.NumberOfInputNeurons);
  setLength(outputNet,rbfnet.Topology.NumberOfOutputNeurons);
  for i:=0 to numberOfTestValues-1 do
  begin
    inputString:='';
    for j:=0 to rbfnet.Topology.NumberOfInputNeurons-1 do
    begin
      //get Input for the net for ith testvalue
      testInput[j]:=doubles[counter];
      inputString:=inputString+'   '+
                        format('%.5f',[testInput[j]]);
      counter:=counter+1;
    end;
    rbfnet.getOutput(testInput,outputNet);
    expectedOutputString:='';
    outputString:='';
    for j:=0 to rbfnet.Topology.NumberOfOutputNeurons-1 do
    begin
      expectedOutputString:=expectedOutputString+'   '+
        format('%.5f',[doubles[counter]]);
      error:=abs(doubles[counter]-outputNet[j]);
      outputString:=outputString+'   '+format('%.5f',[outputNet[j]])+
        ' '+'('+format('%.5f',[error])+')';
      counter:=counter+1;
    end;
    ShowMessage('Test vector no. '+FloatToStr(i+1)+': '+ #13#10 +
                'Input: '+inputString+ #13#10
                +'Expected Output: '+expectedOutputString+#13#10
                +'Real Output (error): '+outputString);
  end;
end;

procedure TrbfDemoForm.testButtonClick(Sender: TObject);

begin
 if not rbfnet.isInit() then
 begin
   showMessage('TrbfDemoForm.testButtonClick: net is not initialised.');
   exit;
 end;
 if openDialog.Execute then
 begin
   test();
 end;
end;


procedure TrbfDemoForm.ParseTextFileIntoDoubles(const fileName: string;
                        var doubles: TDoubleArray);
var
  posInLine, counterOfNumbers: integer;
  f: textfile;
  line, numberInString: string;
begin
  DecimalSeparator:='.';
  assignFile(f,fileName);
  reset(f);
  counterOfNumbers:=0;
  setLength(numberInString,20); //numbers must not be longer than 20 characters
  while (not eof(f)) do
  begin
    readln(f,line);
    //its not a comment, isn't it?
    if not (line[1]='#') then
    begin
      numberInString:='';
      for posInLine:=1 to length(line) do
      begin
        if  (line[posInLine]=' ') then
        begin
          // is there a number to convert?
          if not (numberInString='') then
          begin
            counterOfNumbers:=counterOfNumbers+1;
            //get memory for new double
            setLength(doubles, counterOfNumbers);
            doubles[counterOfNumbers-1]:=StrToFloat(numberInString);
            numberInString:='';
          end;
        end
        else
        begin
          //scan the number
          numberInString:=numberInString+line[posInLine];
        end;
      end;
      // last number is probably not yet converted
      if not (numberInString='') then
      begin
        counterOfNumbers:=counterOfNumbers+1;
        setLength(doubles, counterOfNumbers);
        doubles[counterOfNumbers-1]:=StrToFloat(numberInString);
        numberInString:='';
      end;
    end;
  end;
  closeFile(f);
end;

procedure TrbfDemoForm.centerMethodComboBoxChange(Sender: TObject);
begin
  if (centerMethodComboBox.ItemIndex in [0..2]) then
    rbfnet.CenterMethod:=TCenterMethod(centerMethodComboBox.ItemIndex);

end;

procedure TrbfDemoForm.rbfDemoFormCreate(Sender: TObject);
begin
  //initialisation
  DecimalSeparator:='.';
  RBFNet:=TTedRBFnet.create(nil);
  numbOfInputsEdit.Text:=IntToStr(rbfNet.topology.numberOfInputNeurons);
  numbOfHiddensEdit.Text:=IntToStr(rbfNet.topology.numberOfHiddenNeurons);
  numbOfOutputsEdit.Text:=IntToStr(rbfNet.topology.numberOfOutputNeurons);
  FNumberOfLearnCycles:=1000;
  FErrorBound:=0.1;
  numbOfCyclesEdit.Text:=IntToStr(FNumberOfLearnCycles);
  errorBoundEdit.Text:=FloatToStr(FErrorBound);
end;

procedure TrbfDemoForm.rbfDemoFromClose(Sender: TObject;
  var Action: TCloseAction);
begin
  rbfNet.free();
end;

procedure TrbfDemoForm.numbOfInputsEditOnExit(Sender: TObject);
var
  int: integer;
begin
  try
    int:=StrToInt(numbOfInputsEdit.Text);
  except on EConvertError do
  begin
    showMessage('TrbfDemoForm.numbOfInputsEditOnExit: invalid topology');
    exit;
  end;
  end;
  rbfnet.topology.numberOfInputNeurons:=int;
end;

procedure TrbfDemoForm.numbOfHiddensEditOnExit(Sender: TObject);
var
  int: integer;
begin
  try
    int:=StrToInt(numbOfHiddensEdit.Text);
  except on EConvertError do
  begin
    showMessage('TrbfDemoForm.numbOfHiddensEditOnExit: invalid topology');
    exit;
  end;
  end;
  rbfnet.topology.numberOfHiddenNeurons:=int;
end;

procedure TrbfDemoForm.numbOfOutputsEditOnExit(Sender: TObject);
var
  int: integer;
begin
  try
    int:=StrToInt(numbOfOutputsEdit.Text);
  except on EConvertError do
  begin
    showMessage('TrbfDemoForm.numberOfOutputsEditOnExit: invalid topology');
    exit;
  end;
  end;
  rbfnet.topology.numberOfOutputNeurons:=int;
end;

procedure TrbfDemoForm.numbOfCyclesEditOnExit(Sender: TObject);
var
  i: integer;
begin
  try
    i:=StrToInt(numbOfCyclesEdit.Text);
  except on EConvertError do
  begin
    showMessage('TrbfDemoForm.numbOfCyclesOnExit: '+
                  'invalid number of learning cycles');
    exit;
  end;
  end;
  FNumberOfLearnCycles:=i;
end;

procedure TrbfDemoForm.errorBoundEditOnExit(Sender: TObject);
var
  doub: double;
begin
  try
    doub:=StrToFloat(errorBoundEdit.Text);
  except on EConvertError do
  begin
    showMessage('TrbfDemoForm.errorBoundEditOnExit: '
                  +'invalid error bound');
    exit;
  end;
  end;
  FErrorBound:=doub;
end;

end.

