///////////////////////////////////////////////////////
//      Component: TOXPComboBox v1.0 pre;
//         Author: Eugene "Jek" Efimochkin from SteamworkS Computer Lab;
//    Description: This component is a combobox painted as combos 
//                 in MS OfficeXP.;
//     Native VCL: Yes;
//      Copyright: 2002 SteamworkS Computer Lab Of MAEMAE;
//        License: GNU GPL, see license.txt (or http://www.gnu.org/licenses/gpl.txt);
//     Known bugs: When disabled, combo button frequently flickers.
//            
unit OXPComboBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, StdCtrls, Graphics, math;

type
  TOXPComboBox = Class( TCustomComboBox )
     Private

          FActivate: Boolean;
          FButtonWidth: Integer;
          FMouseInControl: boolean;


          FListInstance: Pointer;
          FDefListProc: Pointer;
          FListHandle: HWND;
          FParentHandle: HWND;

          Procedure PaintButton;
          Procedure PaintBorder;

          Procedure WMPaint( Var Message: TWMPaint ); Message WM_PAINT;
          Procedure WMNCPaint( Var Message: TMessage ); Message WM_NCPAINT;
          Procedure WMSetFocus( Var Message: TMessage ); Message WM_SETFOCUS;
          Procedure WMKillFocus( Var Message: TMessage ); Message WM_KILLFOCUS;
          Procedure CMEnabledChanged( Var Message: TMessage ); Message CM_ENABLEDCHANGED;
          Procedure CMFontChanged( Var Message: TMessage ); Message CM_FONTCHANGED;
          Procedure CMMouseEnter( Var Message: TMessage ); Message CM_MOUSEENTER;
          Procedure CMMouseLeave( Var Message: TMessage ); Message CM_MOUSELEAVE;

          Procedure ListWndProc( Var Message: TMessage );
          Function GetButtonRect: TRect;

     Protected

          Procedure ComboWndProc( Var Message: TMessage; ComboWnd: HWnd; ComboProc: Pointer ); Override;
          Procedure WndProc( Var Message: TMessage ); Override;

     Public

          Constructor Create( AOwner: TComponent ); Override;
          Destructor Destroy; Override;

     published

          Property Style; {Must be published before Items}
          Property Anchors;
          Property BiDiMode;
          Property Color;
          Property Constraints;
          Property Ctl3D;
          Property DragCursor;
          Property DragKind;
          Property DragMode;
          Property DropDownCount;
          Property Enabled;
          Property Font;
          Property ImeMode;
          Property ImeName;
          Property ItemHeight;
          Property MaxLength;
          Property ParentBiDiMode;
          Property ParentColor;
          Property ParentCtl3D;
          Property ParentFont;
          Property ParentShowHint;
          Property PopupMenu;
          Property ShowHint;
          Property Sorted;
          Property TabOrder;
          Property TabStop;
          Property Text;
          Property Visible;
          Property OnChange;
          Property OnClick;
          Property OnContextPopup;
          Property OnDblClick;
          Property OnDragDrop;
          Property OnDragOver;
          Property OnDrawItem;
          Property OnDropDown;
          Property OnEndDock;
          Property OnEndDrag;
          Property OnEnter;
          Property OnExit;
          Property OnKeyDown;
          Property OnKeyPress;
          Property OnKeyUp;
          Property OnMeasureItem;
          Property OnStartDock;
          Property OnStartDrag;
          Property Items; { Must be published after OnMeasureItem }
     End;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('SteamworkS', [TOXPComboBox]);
end;

function BlendColors(const Color1, Color2: TColor; Amount: Extended): TColor;
Var
  R,R2,G,G2,B,B2: Integer;
  win1, win2: Integer;
begin
  win1 := ColorToRGB(color1);
  win2 := ColorToRGB(color2);

  // Convert first
  R := GetRValue(win1);
  G := GetGValue(win1);
  B := GetBValue(win1);
  // Convert second
  R2 := GetRValue(win2);
  G2 := GetGValue(win2);
  B2 := GetBValue(win2);

  // Mix them into color1

  b2:=round((1-amount)*b+amount*b2);
  g2:=round((1-amount)*g+amount*g2);
  r2:=round((1-amount)*r+amount*r2);

  // Just to make sure...
  if R2 < 0 then R2 := 0;
  if G2 < 0 then G2 := 0;
  if B2 < 0 then B2 := 0;

  // If we ever reach white we should revert to first color
  if R2 > 255 then R2 := r;
  if G2 > 255 then G2 := g;
  if B2 > 255 then B2 := b;

  Result := TColor(RGB(R2, G2, B2));
end;

Constructor TOXPComboBox.Create( AOwner: TComponent );
Begin
     Inherited Create( AOwner );

     ControlStyle := ControlStyle - [ csFixedHeight ] + [ csOpaque ];
     TControlCanvas( Canvas ).Control := Self;

     FListInstance := MakeObjectInstance( ListWndProc );
     FDefListProc := Nil;

     FActivate := TRUE;
     FButtonWidth := GetSystemMetrics( SM_CXVSCROLL );
     FMouseInControl := False;
End;

Destructor TOXPComboBox.Destroy;
Begin
     FreeObjectInstance( FListInstance );
     Inherited Destroy;
End;

Function TOXPComboBox.GetButtonRect: TRect;
Begin
     GetWindowRect( Handle, Result );
     OffsetRect( Result, -Result.Left, -Result.Top );
     Inc( Result.Left, ClientWidth - FButtonWidth - 2 );
     //Dec( Result.Right, 2 );
     //InflateRect( Result, 0, -2 );
     {If you wanna see some variation in button style, uncomment two lines above}
End;
     
Procedure TOXPComboBox.CMMouseEnter( Var Message: TMessage );
Begin
     Inherited;
     FMouseInControl := True;
     Invalidate;
End;

Procedure TOXPComboBox.CMMouseLeave( Var Message: TMessage );
Begin
     Inherited;
     FMouseInControl := False;
     Invalidate;
End;

Procedure TOXPComboBox.CMEnabledChanged( Var Message: TMessage );
Begin
     Inherited;
     Invalidate;
End;

Procedure TOXPComboBox.WMNCPaint( Var Message: TMessage );
Begin
     //Inherited;
     PaintBorder;
     PaintButton;
End;

Procedure TOXPComboBox.WMPaint( Var Message: TWMPaint );
Var
     DC: HDC;
     PS: TPaintStruct;
Begin
     If Enabled then begin
        DC := BeginPaint( Handle, PS );
        Try
          PaintWindow( DC );
        Finally
          EndPaint( Handle, PS );
        End;
        PaintBorder;
     end
     else with Canvas do Begin
          Pen.Color:=clBtnShadow;
          Brush.Color:=BlendColors(clBtnShadow, clWindow, 0.231);
          Rectangle(ClipRect);
     end;
     PaintButton;

     Message.Result := 0;
End;

Procedure TOXPComboBox.PaintButton;
Var
     R: TRect;
     X, Y: Integer;
     DDArrColor: TColor;
Begin
     R := GetButtonRect;

     If Style <> csSimple Then
     Begin

          With Canvas do begin
               If Not Enabled then begin
                  //   ,  
                  InflateRect(R,-1,-1);
                  Pen.Style:=psClear;
                  Brush.Color:= BlendColors(clBtnShadow, clWindow, 0.231);
                  DDArrColor:=clBtnShadow;
               end
               else if DroppedDown then begin
                       //  ,  Blend Color,   
                       Pen.Style:=psSolid;
                       Pen.Color:=clHighlight;
                       Brush.Color:=BlendColors(clHighlight, clWindow, 0.60);
                       DDArrColor:=clHighlightText;
                    end
                    else if FMouseInControl then begin
                            //  ,  Blend Color,   
                            Pen.Style:=psSolid;
                            Pen.Color:=clHighlight;
                            Brush.Color:=BlendColors(clHighlight, clWindow, 0.60);
                            DDArrColor:=clMenuText;
                         end
                         else begin
                              //  , Rect  1    ,   
                              Pen.Style:=psSolid;
                              Pen.Color:=Color;
                              Brush.Color:=BlendColors(clBtnFace, clWindow, 0.16);
                              DDArrColor:=clMenuText;
                         end;
               Rectangle( R );

               X := ( R.Right - R.Left ) Div 2 - 6 + R.Left;
               Y := ( R.Bottom - R.Top ) Div 2 - 1 + R.Top;
               Brush.Color := DDArrColor;
               Pen.Color := DDArrColor;
               Pen.Style := psSolid;
               Polygon( [ Point( X + 4, Y ), Point( X + 8, Y ), Point( X + 6, Y + 2 ) ] );
          end;


     End;
     //ExcludeClipRect( Canvas.Handle, ClientWidth - GetSystemMetrics( SM_CXVSCROLL ), 0, ClientWidth, ClientHeight );
End;

Procedure TOXPComboBox.PaintBorder;
Var
     R: TRect;
Begin
     GetWindowRect( Handle, R );

     OffsetRect( R, -R.Left, -R.Top );
     If not Enabled then
        Canvas.Brush.Color:=clBtnShadow
     else
     If ((FMouseInControl) Or (DroppedDown)) then
          Canvas.Brush.Color:=clHighlight
     else
         Canvas.Brush.Color := Color;
     Canvas.FrameRect( R );
     If Enabled Then begin
        InflateRect( R, -1, -1 );
        Canvas.Brush.Color := Color;
        Canvas.FrameRect( R );
     end;
     
End;

Procedure TOXPComboBox.WndProc( Var Message: TMessage );
Begin
     If Message.Msg = WM_PARENTNOTIFY Then
     Begin
          Case LoWord( Message.wParam ) Of
               WM_CREATE:
                    If FDefListProc <> Nil Then
                    Begin
                         SetWindowLong( FListHandle, GWL_WNDPROC, Longint( FDefListProc ) );
                         FDefListProc := Nil;
                         FParentHandle := Message.lParam;
                    End
                    Else
                         If FParentHandle = 0 Then
                              FParentHandle := Message.lParam Else
                              FListHandle := Message.lParam;
          End;
     End;

     Inherited;
End;

Procedure TOXPComboBox.ListWndProc( Var Message: TMessage );
Var
     FontSize: Integer;
     DC: HDC;
     TempFont: HFont;
     Metric: TTextMetric;
Begin
     Case Message.Msg Of
          WM_WINDOWPOSCHANGING:
               Begin
                    With TWMWindowPosMsg( Message ).WindowPos^ Do
                    Begin
                         If Style In [ csDropDown, csDropDownList ] Then
                         Begin
                              DC := GetDC( 0 );
                              TempFont := SelectObject( DC, Font.Handle );
                              GetTextMetrics( DC, Metric );
                              FontSize := Round( Metric.tmHeight + Metric.tmHeight / 8 );
                              cy := ( FontSize - 2 ) * Min( DropDownCount, Items.Count ) + 4;
                              SelectObject( DC, TempFont );
                              ReleaseDC( 0, DC );
                         End
                         Else CY := ( ItemHeight ) * Min( DropDownCount, Items.Count ) + 4;

                         If CY <= 4 Then CY := 10;
                    End;
               End;
     Else
          Message.Result := CallWindowProc( FDefListProc, FListHandle, Message.Msg, Message.WParam, Message.LParam );
     End;
End;

Procedure TOXPComboBox.ComboWndProc( Var Message: TMessage; ComboWnd: HWnd; ComboProc: Pointer );
Begin
     Inherited;

     PaintBorder;
     PaintButton;
End;

Procedure TOXPComboBox.WMSetFocus( Var Message: TMessage );
Begin
     Inherited;
     If Not ( csDesigning In ComponentState ) Then
     Begin
          PaintBorder;
          PaintButton;
     End;
End;

Procedure TOXPComboBox.WMKillFocus( Var Message: TMessage );
Begin
     Inherited;
     If Not ( csDesigning In ComponentState ) Then
     Begin
          PaintBorder;
          PaintButton;
     End;
End;

Procedure TOXPComboBox.CMFontChanged( Var Message: TMessage );
Begin
     Inherited;
     RecreateWnd;
End;

end.
