unit IEInstVer;

{
  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------

  IEInstVer
  Copyright  2001, 2002 Ch. Bergmann
  All rights reserved.

  ------------------------------------------------------------------------------

  All contents of this file and all other files included in the IEInstVer
  archive are Copyright  2001, 2002 Ch. Bergmann - AS NOT OTHERWISE MARKED.

  Use and/or distribution of them requires acceptance of the following
  License Agreement.

  ------------------------------------------------------------------------------

  LICENSE AGREEMENT 1.2:

  "Author" herein refers Ch. Bergmann (the creator of the IEInstVer component).
  "Software" refers to all files bearing this notice, as well as any other
  files and source code included with IEInstVer (typically part of a .zip
  archive), and all content in them, regardless of whether any modifications
  have been made.

  1) This notice may not be removed or altered from any source distribution.

  2) The origin of this software must not be misrepresented, you must not
     claim that you wrote the original software.

  3) The Software may NOT be redistributed in ANY FORM (including but not
     limited to .DPK, .DCR, .PAS, .DCU) after you have obtained it (typically
     by downloading from the Authors site) without the express written
     permission of the Author.

  4) You are permitted to compile the Software into any kind of applications
     (Freeware, Shareware, Commercial).
     If you use this software in a product, an acknowledgment in the product
     documentation or the product itself would be appreciated but is not
     required.

  5) This software is provided "AS-IS", without any express or implied
     warranty. In no event shall the Author be held liable for any damages
     arising from the use of/inability to use this software.

  6) Full backward compatibility in future versions of the Software is not
     guaranteed. In no event shall the Author be held liable for any
     inconvenience or damages arising from lack of backward compatibility.

  7) The Author may terminate your rights to use the Software for any reason
     at his sole discretion.

  8) If you do not agree to any of these terms, you are not permitted to use
     the Software in any way, and all copies of it must be deleted from your
     system(s).

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
}

interface

uses
  Windows, Messages, Classes, Forms, Dialogs, Registry, SysUtils;

type
  TIEInstVer = class(TComponent)
  private
    FAbout: string;
    FTriggerCr: Boolean;
    FTriggerUp: Boolean;
    FUpd: Boolean;
    FIE1: Boolean;
    FIE2: Boolean;
    FIE3: Boolean;
    FIE4: Boolean;
    FIE5: Boolean;
    FIE55: Boolean;
    FIE6: Boolean;
    //-chb FIE6x: Boolean;
    FInstalled: Boolean;
    FShVersionNumber: string;
    FVersionNumber: string;
    FBuild: string;
    FCustom: string;
    FUpdateVersion: string;
    FVersionDescription: string;
    FEncryption: string;
    FWindowTitle: string;
    FPreviousVersionNumber: string;
    FPreviousVersionDescription: string;
    FCookies: string;
    FFavorites: string;
    FHistory: string;
    FCache: string;
    FDownload: string;
    FLinks: string;
    FSave: string;
    FStartPage: string;
    FSearchPage: string;
    FLocalPage: string;
    FUserAgent: string;

    FOnIE1: TNotifyEvent;
    FOnIE2: TNotifyEvent;
    FOnIE3: TNotifyEvent;
    FOnIE4: TNotifyEvent;
    FOnIE5: TNotifyEvent;
    FOnIE55: TNotifyEvent;
    FOnIE6: TNotifyEvent;
    //-chb FOnIE6x: TNotifyEvent;
    FOnNotIE1: TNotifyEvent;
    FOnNotIE2: TNotifyEvent;
    FOnNotIE3: TNotifyEvent;
    FOnNotIE4: TNotifyEvent;
    FOnNotIE5: TNotifyEvent;
    FOnNotIE55: TNotifyEvent;
    FOnNotIE6: TNotifyEvent;
    //-chb FOnNotIE6x: TNotifyEvent;
    FOnInstalled: TNotifyEvent;
    FOnNotInstalled: TNotifyEvent;
    FOnUpd: TNotifyEvent;

    function GetSystemDir: string;
    function GetDllVersion(Dll, IType: string): string;
    function GetIEVersion: string;
    function GetPreviousVersion: string;
    function GetDescription(VerNr: string; UpDate: Boolean): string;
    function GetEncryption: string;
    function GetUpdateVersion: string;
    function GetUserAgent: string;
    function GetShellPath(FolderName: string): string;
    function GetDLDir: string;
    function GetLinksDir: string;
    function GetMisc(Key: string): string;
    procedure GetNoneS(Value: string);
    procedure GetData;

  protected
    procedure Loaded; override;

  public
    constructor Create(AOwner: TComponent); override;
    procedure TriggerEvents;
    procedure UpdateValues;

    property IEInstalled: Boolean read FInstalled;
    property IE1Installed: Boolean read FIE1;
    property IE2Installed: Boolean read FIE2;
    property IE3Installed: Boolean read FIE3;
    property IE4Installed: Boolean read FIE4;
    property IE5Installed: Boolean read FIE5;
    property IE5_5Installed: Boolean read FIE55;
    property IE6Installed: Boolean read FIE6;
    //-chb property IE6_xInstalled: Boolean read FIE6x;
    property VersionNumberRegistry: string read FVersionNumber;
    property BuildNumberRegistry: string read FBuild;
    property CustomizedVersion: string read FCustom;
    property UpdateVersion: string read FUpdateVersion;
    property VersionNumberShDocVwDll: string read FShVersionNumber;
    property VersionDescription: string read FVersionDescription;
    property EncryptionStrength: string read FEncryption;
    property WindowTitle: string read FWindowTitle;
    property UserAgent: string read FUserAgent;
    property PreviousVersionNumber: string read FPreviousVersionNumber;
    property PreviousVersionDescription: string read FPreviousVersionDescription;
    property CookiesFolderPath: string read FCookies;
    property FavoritesFolderPath: string read FFavorites;
    property HistoryFolderPath: string read FHistory;
    property InternetCacheFolderPath: string read FCache;
    property DownloadFolderPath: string read FDownload;
    property LinksFolderPath: string read FLinks;
    property SaveFolderPath: string read FSave;
    property LocalPage: string read FLocalPage;
    property SearchPage: string read FSearchPage;
    property StartPage: string read FStartPage;

  published
    property About: string read FAbout write GetNoneS stored False;
    property TriggerEventsOnLoaded: Boolean read FTriggerCr write FTriggerCr default true;
    property TriggerEventsOnUpdate: Boolean read FTriggerUp write FTriggerUp default true;

    property OnIEInstalled: TNotifyEvent read FOnInstalled write FOnInstalled;
    property OnIE1Installed: TNotifyEvent read FOnIE1 write FOnIE1;
    property OnIE2Installed: TNotifyEvent read FOnIE2 write FOnIE2;
    property OnIE3Installed: TNotifyEvent read FOnIE3 write FOnIE3;
    property OnIE4Installed: TNotifyEvent read FOnIE4 write FOnIE4;
    property OnIE5Installed: TNotifyEvent read FOnIE5 write FOnIE5;
    property OnIE5_5Installed: TNotifyEvent read FOnIE55 write FOnIE55;
    property OnIE6Installed: TNotifyEvent read FOnIE6 write FOnIE6;
    //-chb property OnIE6_xInstalled: TNotifyEvent read FOnIE6x write FOnIE6x;
    property OnNotIEInstalled: TNotifyEvent read FOnNotInstalled write FOnNotInstalled;
    property OnNotIE1Installed: TNotifyEvent read FOnNotIE1 write FOnNotIE1;
    property OnNotIE2Installed: TNotifyEvent read FOnNotIE2 write FOnNotIE2;
    property OnNotIE3Installed: TNotifyEvent read FOnNotIE3 write FOnNotIE3;
    property OnNotIE4Installed: TNotifyEvent read FOnNotIE4 write FOnNotIE4;
    property OnNotIE5Installed: TNotifyEvent read FOnNotIE5 write FOnNotIE5;
    property OnNotIE5_5Installed: TNotifyEvent read FOnNotIE55 write FOnNotIE55;
    property OnNotIE6Installed: TNotifyEvent read FOnNotIE6 write FOnNotIE6;
    //-chb property OnNotIE6_xInstalled: TNotifyEvent read FOnNotIE6x write FOnNotIE6x;
    property OnUpdateValues: TNotifyEvent read FOnUpd write FOnUpd;
  end;

implementation

uses IEInstVerConst;

type
  TDoubleInst = class(Exception);

function TIEInstVer.GetSystemDir: string;
var
  Len: DWORD;
  SysDir: PChar;
  S: string;
begin
  S := 'n/a';
  try
    Len := MAX_PATH + 1;
    GetMem(SysDir, Len);
    try
      if Windows.GetSystemDirectory(SysDir, Len) <> 0 then S := SysDir
    finally
      FreeMem(SysDir)
    end;
    Result := S
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetDllVersion(Dll, IType: string): string;
var
  Info, InfoData, LangPtr: Pointer;
  InfoSize: LongInt;
  Len: DWORD;
  FName: Pchar;
  Infotype, S: string;
begin
  S := 'n/a';
  try
    Len := MAX_PATH + 1;
    FName := Pchar(GetSystemDir + '\' + Dll);
    InfoType := IType;
    InfoSize := GetFileVersionInfoSize(Fname, Len);
    if (InfoSize > 0) then
    begin
      GetMem(Info, InfoSize);
      try
        if GetFileVersionInfo(FName, Len, InfoSize, Info) then
        begin
          Len := 255;
          if VerQueryValue(Info, '\VarFileInfo\Translation', LangPtr, Len)
            then InfoType := Format('\StringFileInfo\%0.4x%0.4x\%s'#0, [LoWord(LongInt(LangPtr^)), HiWord(LongInt(LangPtr^)), InfoType]);
          if VerQueryValue(Info, Pchar(InfoType), InfoData, len) then S := strPas(InfoData)
        end
      finally
        FreeMem(Info, InfoSize)
      end
    end;
    Result := S
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetIEVersion: string;
{
   Q164539 - How to Determine Which Version of Internet Explorer is Installed
   http://support.microsoft.com/support/kb/articles/q164/5/39.asp
   or
   http://support.microsoft.com/default.aspx?scid=kb;EN-US;q164539
}
var
  Reg: TRegistry;
  Ver, S: string;
begin
  S := 'n/a';
  Ver := FShVersionNumber;
  try
    Reg := TRegistry.Create;
    with Reg do
    try
      RootKey := HKEY_LOCAL_MACHINE;
      OpenKey('Software\Microsoft\Internet Explorer', False);
        //-chb IE4/5/6
      if ValueExists('Version') and not (Ver = 'n/a') and not (copy(Ver, 0, 4) = '4.70')
        then S := ReadString('Version')
      else
          //-chb IE1/2/3
      begin
        if ValueExists('IVer') then
        begin
                //-chb IE1
          if ReadString('IVer') = '100.' then S := '4.40.210/308';
                //-chb IE2 WinNT
          if (ReadString('IVer') = '101.') then S := '4.40.520-NT';
                //-chb IE2 Win95
          if (ValueExists('Build')) and (ReadString('IVer') = '102.') and (ReadString('Build') = '520')
            then S := '4.40.520-95';
                //-chb IE3
          if (ValueExists('Build')) and (ReadString('IVer') = '103.')
            then S := '4.70.' + ReadString('Build')
        end
      end;
        //-chb Build#
      if (ValueExists('Build')) and not (length(trim(ReadString('Build'))) = 0)
        then FBuild := ReadString('Build')
      else FBuild := 'n/a';
        //-chb Customized Version
      if (ValueExists('CustomizedVersion')) and not (length(trim(ReadString('CustomizedVersion'))) = 0)
        then FCustom := ReadString('CustomizedVersion')
      else FCustom := 'n/a'
    finally
      Reg.CloseKey;
      Reg.Free
    end;
    Result := S
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetPreviousVersion: string;
{
   Q165871 - How to Determine Your Previous Version of Internet Explorer
   http://support.microsoft.com/support/kb/articles/Q165/8/71.asp
   or
   http://support.microsoft.com/default.aspx?scid=kb;EN-US;q165871
}
var
  Reg: TRegistry;
  Key, S: string;
begin
  S := 'n/a';
  try
    if (FIE4) or (FIE5) or (FIE6) then
    begin
      if FIE4 then key := 'Software\Microsoft\IE4\Setup';
      if (FIE5) or (FIE6) then key := 'Software\Microsoft\IE Setup\Setup';
      Reg := TRegistry.Create;
      with reg do
      try
        RootKey := HKEY_LOCAL_MACHINE;
        OpenKey(Key, false);
        if (ValueExists('PreviousIESysFile')) and not (ReadString('PreviousIESysFile') = '0.0.0.0')
          then S := ReadString('PreviousIESysFile')
      finally
        CloseKey;
        Free
      end
    end;
    Result := S
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetDescription(VerNr: string; UpDate: Boolean): string;
var
  S: string;
begin
  S := 'n/a';
  try
    if not (VerNr = 'n/a') then
    begin
      if UpDate then FInstalled := true;

      //-chb IE1
      if VerNr = '4.40.210/308' then
      begin
        if UpDate then FIE1 := true;
        S := IE1_01
      end;

      //-chb IE2
      if copy(VerNr, 0, 8) = '4.40.520' then
      begin
        if UpDate then FIE2 := true;
        if VerNr = '4.40.520-95' then S := IE2_01;
        if VerNr = '4.40.520-NT' then S := IE2_02;
        if UpDate then FVersionNumber := '4.40.520'
      end;

      //-chb IE3
      if copy(VerNr, 0, 4) = '4.70' then
      begin
        if UpDate then FIE3 := true;
        S := IE3_00;
        if VerNr = '4.70.1155' then S := IE3_01;
        if VerNr = '4.70.1158' then S := IE3_02;
        if VerNr = '4.70.1215' then S := IE3_03;
        if VerNr = '4.70.1300' then S := IE3_04
      end;

      //-chb IE4
      if (copy(VerNr, 0, 4) = '4.71') or (copy(VerNr, 0, 4) = '4.72') then
      begin
        if UpDate then FIE4 := true;
        S := IE4_00;
        if VerNr = '4.71.1008.0003' then S := IE4_01;
        if VerNr = '4.71.1008.3' then S := IE4_01;
        if VerNr = '4.71.1712.0005' then S := IE4_02;
        if VerNr = '4.71.1712.5' then S := IE4_02;
        if VerNr = '4.72.2106.0007' then S := IE4_03;
        if VerNr = '4.72.2106.7' then S := IE4_03;
        if VerNr = '4.72.3110.0003' then S := IE4_04;
        if VerNr = '4.72.3110.3' then S := IE4_04;
        if VerNr = '4.72.3612.1707' then S := IE4_05;
        if VerNr = '4.72.3711.2900' then S := IE4_06
      end;

      //-chb IE5
      if VerNr[1] = '5' then
      begin
        if UpDate then FIE5 := true;
        S := IE5_00;
        if VerNr[3] = '0' then
        begin
          if copy(VerNr, 0, 4) = '5.00' then VerNr := copy(VerNr, 6, length(VerNr) - 5)
          else
            if copy(VerNr, 0, 4) = '5.0.' then VerNr := copy(VerNr, 5, length(VerNr) - 4);
          if VerNr = '0518.0005' then S := IE5_01;
          if VerNr = '0518.5' then S := IE5_01;
          if VerNr = '0910.1308' then S := IE5_02;
          if VerNr = '2014.0213' then S := IE5_03;
          if VerNr = '2014.213' then S := IE5_03;
          if VerNr = '2014.0216' then S := IE5_03a; //-chb undocumented
          if VerNr = '2014.216' then S := IE5_03a; //-chb undocumented
          if VerNr = '2314.1000' then S := IE5_04;
          if VerNr = '2516.1900' then S := IE5_05;

          if VerNr = '2721.1400' then S := IE5_08;
          if VerNr = '2723.2900' then S := IE5_09;
          if VerNr = '2919.0800' then S := IE5_10;
          if VerNr = '2919.800' then S := IE5_10;
          if VerNr = '2919.3800' then S := IE5_11;
          if VerNr = '2919.6307' then S := IE5_12;
          if VerNr = '2919.6400' then S := IE5_13;
          if VerNr = '2920.0000' then S := IE5_14;
          if VerNr = '2920.0' then S := IE5_14;
          if VerNr = '3103.1000' then S := IE5_15;
          if VerNr = '3105.0106' then S := IE5_16;
          if VerNr = '3105.106' then S := IE5_16;
          if VerNr = '3314.2100' then S := IE5_17;
          if VerNr = '3315.2879' then S := IE5_18
        end
        else
        begin
          //-chb IE5.5
          if UpDate then FIE55 := true;
          S := IE55_00;
          if VerNr = '5.50.3825.1300' then S := IE55_01;
          if VerNr = '5.50.4030.2400' then S := IE55_02;
          //-chb if VerNr = '5.50.4133.2400' then S := IE55_xx; //-chb undocumented
          if VerNr = '5.50.4134.0100' then S := IE55_03;
          if VerNr = '5.50.4134.100' then S := IE55_03;
          if VerNr = '5.50.4134.0600' then S := IE55_04;
          if VerNr = '5.50.4134.600' then S := IE55_04;
          if VerNr = '5.50.4308.2900' then S := IE55_05;
          if VerNr = '5.50.4522.1800' then S := IE55_06;
          if VerNr = '5.50.4807.2300' then S := IE55_07;
        end
      end;

      //-chb IE6
      if VerNr[1] = '6' then
      begin
        if UpDate then FIE6 := true;
        S := IE6_00;
        if VerNr[3] = '0' then
        begin
          if copy(VerNr, 0, 4) = '6.00' then VerNr := copy(VerNr, 6, length(VerNr) - 5)
          else
            if copy(VerNr, 0, 4) = '6.0.' then VerNr := copy(VerNr, 5, length(VerNr) - 4);
          if VerNr = '2462.0000' then S := IE6_01;
          if VerNr = '2462.0' then S := IE6_01;
          if VerNr = '2465.0001' then S := IE6_0U1; //-chb undocumented
          if VerNr = '2465.1' then S := IE6_0U1; //-chb undocumented
          if VerNr = '2469.0001' then S := IE6_0U1; //-chb undocumented
          if VerNr = '2469.1' then S := IE6_0U1; //-chb undocumented
          if VerNr = '2475.0001' then S := IE6_0U1; //-chb undocumented
          if VerNr = '2475.1' then S := IE6_0U1; //-chb undocumented
          if VerNr = '2479.0001' then S := IE6_0U1; //-chb undocumented
          if VerNr = '2479.1' then S := IE6_0U1; //-chb undocumented
          if VerNr = '2479.0006' then S := IE6_02;
          if VerNr = '2479.6' then S := IE6_02;
          if VerNr = '2488.0001' then S := IE6_0U1; //-chb undocumented
          if VerNr = '2488.1' then S := IE6_0U1; //-chb undocumented
          if VerNr = '2501.0001' then S := IE6_0U2; //-chb undocumented
          if VerNr = '2501.1' then S := IE6_0U2; //-chb undocumented
          if VerNr = '2512.0001' then S := IE6_0U3; //-chb undocumented
          if VerNr = '2512.1' then S := IE6_0U3; //-chb undocumented
          if VerNr = '2530.0001' then S := IE6_0U4; //-chb undocumented
          if VerNr = '2530.1' then S := IE6_0U4; //-chb undocumented
          if VerNr = '2600.0000' then S := IE6_03;
          if VerNr = '2600.0' then S := IE6_03;
          if VerNr = '2811.1400' then S := IE6_0U5; //-chb undocumented
          if VerNr = '2813.3000' then S := IE6_0U6; //-chb undocumented
          if VerNr = '2813.3' then S := IE6_0U6; //-chb undocumented
          if VerNr = '2800.1005' then S := IE6_0U7; //-chb undocumented
          if VerNr = '2800.1041' then S := IE6_0U8; //-chb undocumented
          if VerNr = '2800.1067' then S := IE6_0U9; //-chb undocumented
          if VerNr = '2800.1082' then S := IE6_0U0; //-chb undocumented
          if VerNr = '2800.1106' then S := IE6_04
        end
        else
        //-chb IE6.x
        begin
          //-chb if UpDate then FIE6x := true;
          S := IE6x_00;
          //-chb if VerNr = '6.xx.xxxx.xxxx' then S := IE6x_xx;
        end
      end
    end;
    Result := S
  except
    Result := 'n/a';
    FInstalled := false;
    FIE1 := false;
    FIE2 := false;
    FIE3 := false;
    FIE4 := false;
    FIE5 := false;
    FIE55 := false;
    FIE6 := false;
    //-chb FIE6x := false
  end
end;

function TIEInstVer.GetEncryption: string;
{
   Q164539 - How to Determine Which Version of Internet Explorer is Installed
   http://support.microsoft.com/support/kb/articles/q164/5/39.asp
   or
   http://support.microsoft.com/default.aspx?scid=kb;EN-US;q164539
}
var
  DllVer, S: string;
begin
  S := 'n/a';
  if FIE3 or FIE4 then
  begin
    DllVer := GetDllVersion('schannel.dll', 'FileDescription');
    if DllVer = 'PCT/SSL Security Provider (Export Version)'
      then S := '40-bit or Server Gated Cryptography (SGC)';
    if DllVer = 'PCT/SSL Security Provider (US and Canada Use Only)' then S := '128-bit';
    exit
  end;
  //-chb undocumented and not yet fully tested
  if FIE5 or FIE6 then
  begin
    DllVer := GetDllVersion('schannel.dll', 'FileDescription');
    //-chb IE5
    //-chb Contributed by Andrew Plus <aeplus2001@yahoo.com.au>
    if DllVer = 'TLS / SSL Security Provider (Export Version)'
      then S := '40-bit or Server Gated Cryptography (SGC)';
    if DllVer = 'TLS / SSL Security Provider (US/Canada Only, Not for Export)' then S := '128-bit';
    //-chb IE5.5/IE6
    if DllVer = 'TLS / SSL Security Provider' then S := '128-bit'
  end;
  Result := S
end;

function TIEInstVer.GetUpdateVersion: string;
var
  reg: TRegistry;
  S: string;
begin
  S := 'n/a';
  try
    Reg := TRegistry.Create;
    with reg do
    try
      RootKey := HKEY_LOCAL_MACHINE;
      OpenKey('\Software\Microsoft\Windows\CurrentVersion\Internet Settings\', false);
      if (ValueExists('MinorVersion')) and not (length(trim(ReadString('MinorVersion'))) = 0)
        then S := ReadString('MinorVersion')
    finally
      CloseKey;
      Free
    end;
    Result := S
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetUserAgent: string;
var
  reg: TRegistry;
  S: string;
begin
  S := 'n/a';
  try
    Reg := TRegistry.Create;
    with reg do
    try
      RootKey := HKEY_CURRENT_USER;
      OpenKey('\Software\Microsoft\Windows\CurrentVersion\Internet Settings\', false);
      if (ValueExists('User Agent')) and not (length(trim(ReadString('User Agent'))) = 0)
        then S := ReadString('User Agent')
    finally
      CloseKey;
      Free
    end;
    Result := S
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetShellPath(FolderName: string): string;
var
  Reg: TRegistry;
  S: string;
begin
  S := 'n/a';
  try
    Reg := TRegistry.Create;
    with reg do
    try
      Rootkey := HKEY_CURRENT_USER;
      OpenKey('SOFTWARE\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders', false);
      if (ValueExists(FolderName)) and not (length(trim(ReadString(FolderName))) = 0)
        then S := ReadString(FolderName)
    finally
      CloseKey;
      Free
    end;
    Result := S
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetDLDir: string;
var
  reg: TRegistry;
  S: string;
begin
  S := 'n/a';
  try
    Reg := TRegistry.Create;
    with Reg do
    try
      RootKey := HKEY_CURRENT_USER;
      OpenKey('Software\Microsoft\Internet Explorer', False);
      if (ValueExists('Download Directory')) and not (length(trim(ReadString('Download Directory'))) = 0)
        then S := ReadString('Download Directory')
    finally
      CloseKey;
      Free
    end;
    Result := S
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetLinksDir: string;
var
  reg: TRegistry;
  S: string;
begin
  S := 'n/a';
  try
    if FFavorites <> 'n/a' then
    begin
      Reg := TRegistry.Create;
      with Reg do
      try
        RootKey := HKEY_CURRENT_USER;
        OpenKey('Software\Microsoft\Internet Explorer\Toolbar', False);
        if (ValueExists('LinksFolderName')) and not (length(trim(ReadString('LinksFolderName'))) = 0)
          then S := FFavorites + '\' + ReadString('LinksFolderName')
      finally
        CloseKey;
        Free
      end
    end;
    Result := S
  except
    Result := 'n/a'
  end
end;

function TIEInstVer.GetMisc(Key: string): string;
{
   Q176497 - How to Change the Internet Explorer Window Title
   http://support.microsoft.com/support/kb/articles/q176/4/97.asp
   or
   http://support.microsoft.com/default.aspx?scid=kb;EN-US;q176497
}
var
  reg: TRegistry;
  S: string;
begin
  S := 'n/a';
  try
    Reg := TRegistry.Create;
    with Reg do
    try
      RootKey := HKEY_CURRENT_USER;
      OpenKey('Software\Microsoft\Internet Explorer\Main', False);
      if (ValueExists(Key)) and not (length(trim(ReadString(Key))) = 0) then S := ReadString(Key)
    finally
      CloseKey;
      Free
    end;
    Result := S
  except
    Result := 'n/a'
  end
end;

procedure TIEInstVer.GetNoneS(Value: string);
begin {} end;

procedure TIEInstVer.GetData;
begin
  FIE1 := false;
  FIE2 := false;
  FIE3 := false;
  FIE4 := false;
  FIE5 := false;
  FIE55 := false;
  FIE6 := false;
  //-chb FIE6x := false;
  FInstalled := false;
  FShVersionNumber := GetDllVersion('shdocvw.dll', 'FileVersion');
  FVersionNumber := GetIEVersion;
  if FVersionNumber <> 'n/a' then FVersionDescription := GetDescription(FVersionNumber, true)
  else FVersionDescription := GetDescription(FShVersionNumber, true);
  FEncryption := GetEncryption;
  FWindowTitle := GetMisc('Window Title');
  if FWindowTitle = 'n/a' then FWindowTitle := 'Microsoft Internet Explorer';
  FPreviousVersionNumber := GetPreviousVersion;
  FPreviousVersionDescription := GetDescription(FPreviousVersionNumber, false);
  FCookies := GetShellPath('Cookies');
  FFavorites := GetShellPath('Favorites');
  FHistory := GetShellPath('History');
  FCache := GetShellPath('Cache');
  FDownload := GetDLDir;
  FLinks := GetLinksDir;
  FSave := GetMisc('Save Directory');
  FLocalPage := GetMisc('Local Page');
  FSearchPage := GetMisc('Search Page');
  FStartPage := GetMisc('Start Page');
  FUpdateVersion := GetUpdateVersion;
  FUserAgent := GetUserAgent
end;

procedure TIEInstVer.TriggerEvents;
begin
  if not (csDesigning in ComponentState) then
  begin
    if (FIE1) and (Assigned(FOnIE1)) then FOnIE1(Self);
    if (FIE2) and (Assigned(FOnIE2)) then FOnIE2(Self);
    if (FIE3) and (Assigned(FOnIE3)) then FOnIE3(Self);
    if (FIE4) and (Assigned(FOnIE4)) then FOnIE4(Self);
    if (FIE5) and (Assigned(FOnIE5)) then FOnIE5(Self);
    if (FIE55) and (Assigned(FOnIE55)) then FOnIE55(Self);
    if (FIE6) and (Assigned(FOnIE6)) then FOnIE6(Self);
    //-chb if (FIE6x) and (Assigned(FOnIE6x)) then FOnIE6x(Self);
    if not (FIE1) and (Assigned(FOnNotIE1)) then FOnNotIE1(Self);
    if not (FIE2) and (Assigned(FOnNotIE2)) then FOnNotIE2(Self);
    if not (FIE3) and (Assigned(FOnNotIE3)) then FOnNotIE3(Self);
    if not (FIE4) and (Assigned(FOnNotIE4)) then FOnNotIE4(Self);
    if not (FIE5) and (Assigned(FOnNotIE5)) then FOnNotIE5(Self);
    if not (FIE55) and (Assigned(FOnNotIE55)) then FOnNotIE55(Self);
    if not (FIE6) and (Assigned(FOnNotIE6)) then FOnNotIE6(Self);
    //-chb if not (FIE6x) and (Assigned(FOnNotIE6x)) then FOnNotIE6x(Self);
    if (FInstalled) and (Assigned(FOnInstalled)) then FOnInstalled(Self);
    if not (FInstalled) and (Assigned(FOnNotInstalled)) then FOnNotInstalled(Self);
    if (FUpd) and (Assigned(FOnUpd)) then
    try FOnUpd(Self)finally FUpd := false end
  end
end;

procedure TIEInstVer.UpdateValues;
begin
  GetData;
  if FTriggerUp then
  begin
    FUpd := true;
    TriggerEvents
  end
end;

constructor TIEInstVer.Create(AOwner: TComponent);
var
  i: word;
  CompCount: byte;
begin
  inherited Create(AOwner);

  CompCount := 0;
  if (csDesigning in ComponentState) then
  begin
    with (AOwner) do
    begin
      for i := 0 to ComponentCount - 1 do
        if Components[i] is TIEInstVer then inc(CompCount);
      if CompCount > 1 then
        raise TDoubleInst.Create('There already exists an Instance of IEInstVer on ' + AOwner.Name)
    end
  end;

  FAbout := #73#69#73#110#115#116#86#101#114#32 + IEIVersion + #32#32#91 + IEIDate +
    #93#32#45#32 + CRG + #32 + ARR;
  FTriggerCr := True;
  FTriggerUp := True;
  GetData
end;

procedure TIEInstVer.Loaded;
begin
  inherited Loaded;
  if FTriggerCr then TriggerEvents
end;

end.
