unit IEInstVerL;

{
  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------

  IEInstVer
  Copyright  2001, 2002 Ch. Bergmann
  All rights reserved.

  ------------------------------------------------------------------------------

  All contents of this file and all other files included in the IEInstVer
  archive are Copyright  2001, 2002 Ch. Bergmann - AS NOT OTHERWISE MARKED.

  Use and/or distribution of them requires acceptance of the following
  License Agreement.

  ------------------------------------------------------------------------------

  LICENSE AGREEMENT 1.2:

  "Author" herein refers Ch. Bergmann (the creator of the IEInstVer component).
  "Software" refers to all files bearing this notice, as well as any other
  files and source code included with IEInstVer (typically part of a .zip
  archive), and all content in them, regardless of whether any modifications
  have been made.

  1) This notice may not be removed or altered from any source distribution.

  2) The origin of this software must not be misrepresented, you must not
     claim that you wrote the original software.

  3) The Software may NOT be redistributed in ANY FORM (including but not
     limited to .DPK, .DCR, .PAS, .DCU) after you have obtained it (typically
     by downloading from the Authors site) without the express written
     permission of the Author.

  4) You are permitted to compile the Software into any kind of applications
     (Freeware, Shareware, Commercial).
     If you use this software in a product, an acknowledgment in the product
     documentation or the product itself would be appreciated but is not
     required.

  5) This software is provided "AS-IS", without any express or implied
     warranty. In no event shall the Author be held liable for any damages
     arising from the use of/inability to use this software.

  6) Full backward compatibility in future versions of the Software is not
     guaranteed. In no event shall the Author be held liable for any
     inconvenience or damages arising from lack of backward compatibility.

  7) The Author may terminate your rights to use the Software for any reason
     at his sole discretion.

  8) If you do not agree to any of these terms, you are not permitted to use
     the Software in any way, and all copies of it must be deleted from your
     system(s).

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
}

interface

uses
  Windows, Messages, Classes, Forms, Dialogs, Registry, SysUtils;

type
  TIEInstVerL = class(TComponent)
  private
    FAbout: string;
    FTriggerCr: Boolean;
    FTriggerUp: Boolean;
    FUpd: Boolean;
    FIE1: Boolean;
    FIE2: Boolean;
    FIE3: Boolean;
    FIE4: Boolean;
    FIE5: Boolean;
    FIE55: Boolean;
    FIE6: Boolean;
    //-chb FIE6x: Boolean;
    FInstalled: Boolean;
    FShVersionNumber: string;
    FVersionNumber: string;
    FBuild: string;

    FOnIE1: TNotifyEvent;
    FOnIE2: TNotifyEvent;
    FOnIE3: TNotifyEvent;
    FOnIE4: TNotifyEvent;
    FOnIE5: TNotifyEvent;
    FOnIE55: TNotifyEvent;
    FOnIE6: TNotifyEvent;
    //-chb FOnIE6x: TNotifyEvent;
    FOnNotIE1: TNotifyEvent;
    FOnNotIE2: TNotifyEvent;
    FOnNotIE3: TNotifyEvent;
    FOnNotIE4: TNotifyEvent;
    FOnNotIE5: TNotifyEvent;
    FOnNotIE55: TNotifyEvent;
    FOnNotIE6: TNotifyEvent;
    //-chb FOnNotIE6x: TNotifyEvent;
    FOnInstalled: TNotifyEvent;
    FOnNotInstalled: TNotifyEvent;
    FOnUpd: TNotifyEvent;

    function GetSystemDir: string;
    function GetDllVersion(Dll, IType: string): string;
    function GetIEVersion: string;
    procedure GetState(VerNr: string);
    procedure GetNoneS(Value: string);
    procedure GetData;

  protected
    procedure Loaded; override;

  public
    constructor Create(AOwner: TComponent); override;
    procedure TriggerEvents;
    procedure UpdateValues;

    property IEInstalled: Boolean read FInstalled;
    property IE1Installed: Boolean read FIE1;
    property IE2Installed: Boolean read FIE2;
    property IE3Installed: Boolean read FIE3;
    property IE4Installed: Boolean read FIE4;
    property IE5Installed: Boolean read FIE5;
    property IE5_5Installed: Boolean read FIE55;
    property IE6Installed: Boolean read FIE6;
    //-chb property IE6_xInstalled: Boolean read FIE6x;
    property VersionNumberRegistry: string read FVersionNumber;
    property BuildNumberRegistry: string read FBuild;
    property VersionNumberShDocVwDll: string read FShVersionNumber;

  published
    property About: string read FAbout write GetNoneS stored False;
    property TriggerEventsOnLoaded: Boolean read FTriggerCr write FTriggerCr default true;
    property TriggerEventsOnUpdate: Boolean read FTriggerUp write FTriggerUp default true;

    property OnIEInstalled: TNotifyEvent read FOnInstalled write FOnInstalled;
    property OnIE1Installed: TNotifyEvent read FOnIE1 write FOnIE1;
    property OnIE2Installed: TNotifyEvent read FOnIE2 write FOnIE2;
    property OnIE3Installed: TNotifyEvent read FOnIE3 write FOnIE3;
    property OnIE4Installed: TNotifyEvent read FOnIE4 write FOnIE4;
    property OnIE5Installed: TNotifyEvent read FOnIE5 write FOnIE5;
    property OnIE5_5Installed: TNotifyEvent read FOnIE55 write FOnIE55;
    property OnIE6Installed: TNotifyEvent read FOnIE6 write FOnIE6;
    //-chb property OnIE6_xInstalled: TNotifyEvent read FOnIE6x write FOnIE6x;
    property OnNotIEInstalled: TNotifyEvent read FOnNotInstalled write FOnNotInstalled;
    property OnNotIE1Installed: TNotifyEvent read FOnNotIE1 write FOnNotIE1;
    property OnNotIE2Installed: TNotifyEvent read FOnNotIE2 write FOnNotIE2;
    property OnNotIE3Installed: TNotifyEvent read FOnNotIE3 write FOnNotIE3;
    property OnNotIE4Installed: TNotifyEvent read FOnNotIE4 write FOnNotIE4;
    property OnNotIE5Installed: TNotifyEvent read FOnNotIE5 write FOnNotIE5;
    property OnNotIE5_5Installed: TNotifyEvent read FOnNotIE55 write FOnNotIE55;
    property OnNotIE6Installed: TNotifyEvent read FOnNotIE6 write FOnNotIE6;
    //-chb property OnNotIE6_xInstalled: TNotifyEvent read FOnNotIE6x write FOnNotIE6x;
    property OnUpdateValues: TNotifyEvent read FOnUpd write FOnUpd;
  end;

implementation

const
  IEIVersion = '4.9 (Lite)';
  IEIDate = '2002/10/06';

type
  TDoubleInst = class(Exception);

function TIEInstVerL.GetSystemDir: string;
var
  Len: DWORD;
  SysDir: PChar;
  S: string;
begin
  S := 'n/a';
  try
    Len := MAX_PATH + 1;
    GetMem(SysDir, Len);
    try
      if Windows.GetSystemDirectory(SysDir, Len) <> 0 then S := SysDir
    finally
      FreeMem(SysDir)
    end;
    Result := S
  except
    Result := 'n/a'
  end
end;

function TIEInstVerL.GetDllVersion(Dll, IType: string): string;
var
  Info, InfoData, LangPtr: Pointer;
  InfoSize: LongInt;
  Len: DWORD;
  FName: Pchar;
  Infotype, S: string;
begin
  S := 'n/a';
  try
    Len := MAX_PATH + 1;
    FName := Pchar(GetSystemDir + '\' + Dll);
    InfoType := IType;
    InfoSize := GetFileVersionInfoSize(Fname, Len);
    if (InfoSize > 0) then
    begin
      GetMem(Info, InfoSize);
      try
        if GetFileVersionInfo(FName, Len, InfoSize, Info) then
        begin
          Len := 255;
          if VerQueryValue(Info, '\VarFileInfo\Translation', LangPtr, Len) then
            InfoType := Format('\StringFileInfo\%0.4x%0.4x\%s'#0, [LoWord(LongInt(LangPtr^)), HiWord(LongInt(LangPtr^)), InfoType]);
          if VerQueryValue(Info, Pchar(InfoType), InfoData, len) then S := strPas(InfoData)
        end;
      finally
        FreeMem(Info, InfoSize)
      end
    end;
    Result := S
  except
    Result := 'n/a'
  end
end;

function TIEInstVerL.GetIEVersion: string;
{
   Q164539 - How to Determine Which Version of Internet Explorer is Installed
   http://support.microsoft.com/support/kb/articles/q164/5/39.asp
   or
   http://support.microsoft.com/default.aspx?scid=kb;EN-US;q164539
}
var
  Reg: TRegistry;
  Ver, S: string;
begin
  S := 'n/a';
  Ver := FShVersionNumber;
  try
    Reg := TRegistry.Create;
    with Reg do
    try
      RootKey := HKEY_LOCAL_MACHINE;
      OpenKey('Software\Microsoft\Internet Explorer', False);
        //-chb IE4/5/6
      if ValueExists('Version') and not (Ver = 'n/a') and not (copy(Ver, 0, 4) = '4.70')
        then S := ReadString('Version')
      else
          //-chb IE1/2/3
      begin
        if ValueExists('IVer') then
        begin
                //-chb IE1
          if ReadString('IVer') = '100.' then S := '4.40.210/308';
                //-chb IE2 WinNT
          if (ReadString('IVer') = '101.') then S := '4.40.520-NT';
                //-chb IE2 Win95
          if (ValueExists('Build')) and (ReadString('IVer') = '102.') and (ReadString('Build') = '520')
            then S := '4.40.520-95';
                //-chb IE3
          if (ValueExists('Build')) and (ReadString('IVer') = '103.')
            then S := '4.70.' + ReadString('Build')
        end
      end;
        //-chb Build#
      if (ValueExists('Build')) and not (length(trim(ReadString('Build'))) = 0)
        then FBuild := ReadString('Build')
      else FBuild := 'n/a'
    finally
      Reg.CloseKey;
      Reg.Free
    end;
    Result := S
  except
    Result := 'n/a'
  end
end;

procedure TIEInstVerL.GetState(VerNr: string);
begin
  try
    if not (VerNr = 'n/a') then
    begin
      FInstalled := true;

      //-chb IE1
      if VerNr = '4.40.210/308' then FIE1 := true;

      //-chb IE2
      if copy(VerNr, 0, 8) = '4.40.520' then
      begin
        FIE2 := true;
        FVersionNumber := '4.40.520'
      end;

      //-chb IE3
      if copy(VerNr, 0, 4) = '4.70' then FIE3 := true;

      //-chb IE4
      if (copy(VerNr, 0, 4) = '4.71') or (copy(VerNr, 0, 4) = '4.72') then FIE4 := true;

      //-chb IE5
      if VerNr[1] = '5' then
      begin
        FIE5 := true;
        //-chb IE5.5
        if VerNr[3] = '5' then FIE55 := true
      end;

      //-chb IE6
      if VerNr[1] = '6' then
      begin
        FIE6 := true;
        //-chb IE6.x
        //-chb if VerNr[3] = 'x' then FIE6x := true
      end
    end
  except
    FInstalled := false;
    FIE1 := false;
    FIE2 := false;
    FIE3 := false;
    FIE4 := false;
    FIE5 := false;
    FIE55 := false;
    FIE6 := false;
    //-chb FIE6x := false
  end
end;

procedure TIEInstVerL.GetNoneS(Value: string);
begin {} end;

procedure TIEInstVerL.GetData;
begin
  FIE1 := false;
  FIE2 := false;
  FIE3 := false;
  FIE4 := false;
  FIE5 := false;
  FIE55 := false;
  FIE6 := false;
  //-chb FIE6x := false;
  FInstalled := false;
  FShVersionNumber := GetDllVersion('shdocvw.dll', 'FileVersion');
  FVersionNumber := GetIEVersion;
  if FVersionNumber <> 'n/a' then GetState(FVersionNumber) else GetState(FShVersionNumber)
end;

procedure TIEInstVerL.TriggerEvents;
begin
  if not (csDesigning in ComponentState) then
  begin
    if (FIE1) and (Assigned(FOnIE1)) then FOnIE1(Self);
    if (FIE2) and (Assigned(FOnIE2)) then FOnIE2(Self);
    if (FIE3) and (Assigned(FOnIE3)) then FOnIE3(Self);
    if (FIE4) and (Assigned(FOnIE4)) then FOnIE4(Self);
    if (FIE5) and (Assigned(FOnIE5)) then FOnIE5(Self);
    if (FIE55) and (Assigned(FOnIE55)) then FOnIE55(Self);
    if (FIE6) and (Assigned(FOnIE6)) then FOnIE6(Self);
    //-chb if (FIE6x) and (Assigned(FOnIE6x)) then FOnIE6x(Self);
    if not (FIE1) and (Assigned(FOnNotIE1)) then FOnNotIE1(Self);
    if not (FIE2) and (Assigned(FOnNotIE2)) then FOnNotIE2(Self);
    if not (FIE3) and (Assigned(FOnNotIE3)) then FOnNotIE3(Self);
    if not (FIE4) and (Assigned(FOnNotIE4)) then FOnNotIE4(Self);
    if not (FIE5) and (Assigned(FOnNotIE5)) then FOnNotIE5(Self);
    if not (FIE55) and (Assigned(FOnNotIE55)) then FOnNotIE55(Self);
    if not (FIE6) and (Assigned(FOnNotIE6)) then FOnNotIE6(Self);
    //-chb if not (FIE6x) and (Assigned(FOnNotIE6x)) then FOnNotIE6x(Self);
    if (FInstalled) and (Assigned(FOnInstalled)) then FOnInstalled(Self);
    if not (FInstalled) and (Assigned(FOnNotInstalled)) then FOnNotInstalled(Self);
    if (FUpd) and (Assigned(FOnUpd)) then
    try FOnUpd(Self)finally FUpd := false end
  end
end;

procedure TIEInstVerL.UpdateValues;
begin
  GetData;
  if FTriggerUp then
  begin
    FUpd := true;
    TriggerEvents
  end
end;

constructor TIEInstVerL.Create(AOwner: TComponent);
var
  i: word;
  CompCount: byte;
begin
  inherited Create(AOwner);

  CompCount := 0;
  if (csDesigning in ComponentState) then
  begin
    with (AOwner) do
    begin
      for i := 0 to ComponentCount - 1 do
        if Components[i] is TIEInstVerL then inc(CompCount);
      if CompCount > 1 then
        raise TDoubleInst.Create('There already exists an Instance of IEInstVerL on ' + AOwner.Name)
    end
  end;

  FAbout := #73#69#73#110#115#116#86#101#114#32 + IEIVersion + #32#32#91 + IEIDate + #93#32#45#32 +
    #67#111#112#121#114#105#103#104#116#32#169#32#50#48#48#49#44#32#50#48#48#50#32#32#67#104#46#32 +
    #66#101#114#103#109#97#110#110#46#32#65#108#108#32#82#105#103#104#116#115#32#82#101#115#101#114 +
    #118#101#100#46;
  FTriggerCr := True;
  FTriggerUp := True;
  GetData
end;

procedure TIEInstVerL.Loaded;
begin
  inherited Loaded;
  if FTriggerCr then TriggerEvents
end;

end.
