{*******************************************************}
{                                                       }
{       Borland Delphi Visual Component Library         }
{                                                       }
{       Copyright (c) 1997,99 Inprise Corporation       }
{                                                       }
{*******************************************************}

unit FxReg;

interface

uses
  DesignIntf, DesignEditors, CHARTREG, CHART;

type
  TDSSChartCompEditor = class(TChartCompEditor)
  public
    procedure Edit; override;
  end;

  procedure Register;

implementation

uses
  SysUtils, Classes, FxGrid, FxPivSrc, Fxdb, FxDConst,
  Forms, Graphics, StdCtrls, ComCtrls, FxStore, Controls,
  DBTables, FxGraph, bdeconst, FxButton, FxDimEdt, FxDCube,
  EDITCHAR, FxDimPager, FxCommon;

  { TSourceEditor }

type
  TSourceEditor = class(TComponentEditor)
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

procedure TSourceEditor.ExecuteVerb(Index: Integer);
var
  Source: TFxSource;
begin
  Source := Component as TFxSource;
  case Index of
    0:
    begin
      Source.SparseRows := not Source.SparseRows;
      Source.SparseCols := Source.SparseRows;
    end;
  end;
end;

function TSourceEditor.GetVerb(Index: Integer): string;
var
  Source: TFxSource;
begin
  Source := Component as TFxSource;
  case Index of
    0:
    begin
      if Source.SparseRows then
        Result := sSourceVerb0
      else
        Result := sSourceVerb1;
    end;
  end;
end;

function TSourceEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

 { TCubeEditor }

type
  TCubeEditor = class(TComponentEditor)
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

procedure TCubeEditor.ExecuteVerb(Index: Integer);
var
  Cube: TFxCube;
begin
  Cube := Component as TFxCube;
  case Index of
    0: ShowFDCubeEditor(Cube);
    1: begin
//      if (Cube.DataSet is TQuery) then
//        ShowDssQueryEditor(Designer,TQuery(Cube.DataSet));
    end;
  end;
end;

function TCubeEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0: Result := SCubeVerb0;
    1: Result := SCubeVerb1;
  end;
end;

function TCubeEditor.GetVerbCount: Integer;
//var
//  Cube: TFxCube;
begin
//  Cube := Component as TFxCube;
//  if (Cube.DataSet is TQuery) and not (Cube.DataSet is TDecisionQuery) then
//    Result := 2
//  else
    Result := 1;
end;

  { TDSSChartCompEditor }

procedure TDSSChartCompEditor.Edit;
begin
  EditDSSChart(nil, TCustomChart(Component));
end;

  { TGridEditor }

type
  TGridEditor = class(TComponentEditor)
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

procedure TGridEditor.ExecuteVerb(Index: Integer);
var
  Grid: TFxGrid;
begin
  Grid := Component as TFxGrid;
  case Index of
    0: Grid.Totals := not Grid.Totals;
  end;
end;

function TGridEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0: Result := sGridVerb0;
  end;
end;

function TGridEditor.GetVerbCount: Integer;
begin
  Result := 1;
end;

  { TDecisionGridDimsProperty }

type
  TDecisionGridDimsProperty = class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
  end;

function TDecisionGridDimsProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog];
end;

function TDecisionGridDimsProperty.GetValue: String;
begin
  Result := '(' + TDisplayDims.ClassName + ')';
end;

procedure TDecisionGridDimsProperty.Edit;
begin
  ShowDisplayDimEditor(Designer, GetComponent(0) as TFxGrid);
end;

  { TDimensionMapProperty }

type
  TDimensionMapProperty = class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
  end;

function TDimensionMapProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog];
end;

function TDimensionMapProperty.GetValue: String;
begin
  Result := 'TDimensionMaps';
end;

procedure TDimensionMapProperty.Edit;
begin
  ShowFDCubeEditor(GetComponent(0)as TFxCube);
end;

  { Registration }

procedure Register;
begin
  RegisterComponents(sComponentTabName, [TFxCube,
    TFxSource, TFxPivot, TFxGrid, TFxChart,
    TDimPager,TSumCombo]);

  RegisterNonActiveX([TFxCube, TFxSource,
                      TFxPivot, TFxGrid, TFxChart,
                      TCustomFxGrid, TFxCustomChart], axrIncludeDescendants);

  RegisterComponentEditor(TFxCube, TCubeEditor);
  RegisterComponentEditor(TFxSource, TSourceEditor);
  RegisterComponentEditor(TFxGrid, TGridEditor);
  RegisterComponentEditor(TFxChart,TDSSChartCompEditor);

  RegisterPropertyEditor(TypeInfo(TCollection), TFxGrid, 'Dimensions', TDecisionGridDimsProperty);
//  RegisterPropertyEditor(TypeInfo(TCollection), TFxCube, 'DimensionMap', TDimensionMapProperty);
  RegisterPropertyEditor(TypeInfo(TDate), nil, '', TDateProperty);
end;

end.

