unit TedRBFnet;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, Grids;

type
  //set of available activation functions
  TFActivate = (Gauss);

  //set of methods to find the centers of the hidden layer neurons
  TCenterMethod = (Direct,Subset,Cluster);

  TDoubleArray = array of double;

  //data structure for training the net
  TTrainingValue   = record
                       input: TDoubleArray;
                       output: TDoubleArray;
                     end;
  TTrainingPattern = array of TTrainingValue;

  //hidden layer
  THiddenNeuron = record
                    center: TDoubleArray;  //gets Dimension of number of
                  end;                         //input neurons

  //output layer
  TOutputNeuron = record
                    weight: TDoubleArray;  //one for each hidden neuron
                  end;

  TTedRBFnet = class;

  //beginning of declaration for topology
  //topology of the network: (#input, #hidden, #output)
  TTopology = class(TPersistent)
  private
    FNumberOfInputNeurons: Byte;
    FNumberOfHiddenNeurons: Byte;
    FNumberOfOutputNeurons: Byte;
    FTedRBFnet: TTedRBFnet;

    procedure SetNumberOfInputNeurons(const NumbOfInputsIn: Byte);
    procedure SetNumberOfHiddenNeurons(const NumbOfHidIn: Byte);
    procedure SetNumberOfOutputNeurons(const NumbOfOutputsIn: Byte);

  public
    constructor Create();
    destructor Destroy(); override;

  published
    property NumberOfInputNeurons: Byte read FNumberOfInputNeurons write
                                                SetNumberOfInputNeurons;
    property NumberOfHiddenNeurons: Byte read FNumberOfHiddenNeurons write
                                                SetNumberOfHiddenNeurons;
    property NumberOfOutputNeurons: Byte read FNumberOfOutputNeurons write
                                                SetNumberOfOutputNeurons;
  end;
  //end of topology declaration

  //beginning of declaration for TedRBFnet
  //network with radialbasis functions
  TTedRBFnet = class(TComponent)
  private
    FTrainingInit: Boolean; //trainingpattern initialised?
    FTedRBFnetInit: Boolean; //is the net initialised before learning?
    FTrainingPattern: TTrainingPattern;
    FTopology: TTopology;
    FWeightMin: double; //limits for initialising the weights of the output
    FWeightMax: double; //neurons
    FSeedWeights: Integer;  //for randomseed
    FActivate: TFActivate; //type of activation function in the hidden layer
    FCenterMethod: TCenterMethod; // method for initialising the centers
                                  //in the hidden layer
    FEta: double; //for the weights of the output layer
    FSigma: double; //standard deviation for the radialsymmetric base functions

    //hidden layer:
    FHiddenNeurons: array of THiddenNeuron;

    //output layer:
    FOutputNeurons: array of TOutputNeuron;

    //for the published properties
    procedure SetFWeightMax(const WeightMaxIn: double);
    procedure SetFWeightMin(const WeightMinIn: double);
    procedure SetFSeedWeights(const SeedWeightsIn: Integer);
    procedure SetFSigma(const SigmaIn: double);
    procedure SetLearningRate(const LearningRateIn: double);

    //initialising the centers of hidden layer with the inputs via TRBF.Init
    procedure InitCentersDirect();
    procedure InitCentersRandomSubset();
    procedure InitCentersCluster();

    procedure InitOutputLayer();

    procedure CopyArray(var LCopy: array of double;
                         const LOriginal: array of double);

    //gets euclidic distance
    function DistanceSquare(x,c: array of double): double;

    //get output of the net with hidden layer results
    procedure getCompleteOutput(const input: TDoubleArray;
        var OutputHidden: TDoubleArray; var OutputNet: TDoubleArray);
    procedure errorHandling(errorIndex: byte);

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy(); override;

    //initialising training
    procedure SetTrainingPattern(const doubles: TDoubleArray; var
                        numberOfTrainingValues: integer);

    //init net
    procedure init();

    //learning
    procedure learning(var numberOfLearnCycles: Integer;
                         const abortBound: double=-1);

    function  errorOfTrainingData(): double;

    //delivers net output with arbitrary input
    procedure getOutput(const input: TDoubleArray; var OutputNet: TDoubleArray);

    //get training data
    procedure getTrainingPattern(var pattern: TTrainingPattern);

    function isInit(): boolean;



  published
    property Topology : TTopology read FTopology;
    property WeightMin: double read FWeightMin write SetFWeightMin;
    property WeightMax: double read FWeightMax write SetFWeightMax;
    property SeedWeights: Integer read FSeedWeights write SetFSeedWeights;
    property FuncActivate: TFActivate read FActivate write FActivate;
    property CenterMethod: TCenterMethod read FCenterMethod write
                                                FCenterMethod;
    property Sigma: double read FSigma write SetFSigma;
    property LearningRate: double read FEta write SetLearningRate;
  end;
  ///end of declaration for TedRBFnet

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Tedsoft', [TTedRBFnet]);
end;

//implementation of TTopology
constructor TTopology.Create();
begin
  inherited Create();

  //defaults
  NumberOfInputNeurons:=2;
  NumberOfHiddenNeurons:=3;
  NumberOfOutputNeurons:=4;
  FTedRBFnet := nil;
end;

procedure TTopology.SetNumberOfInputNeurons(const NumbOfInputsIn: Byte);
begin
  if ((NumbOfInputsIn > 0) and (NumbOfInputsIn <> FNumberOfInputNeurons)) then
  begin
    FNumberOfInputNeurons:=NumbOfInputsIn;

    // new trainingpattern, new init of TedRBFnet required
    if assigned(FTedRBFnet) then
    begin
      FTedRBFnet.FTrainingInit := False;
      FTedRBFnet.FTedRBFnetInit := False;
    end;
  end;
end;

procedure TTopology.SetNumberOfHiddenNeurons(const NumbOfHidIn: Byte);
begin
  if ((NumbOfHidIn > 0) and (NumbOfHidIn <> FNumberOfHiddenNeurons)) then
  begin
    FNumberOfHiddenNeurons:=NumbOfHidIn;

    // new init of TedRBFnet required
    if assigned(FTedRBFnet) then
    begin
      FTedRBFnet.FTedRBFnetInit := False;
    end;
  end;
end;

procedure TTopology.SetNumberOfOutputNeurons(const NumbOfOutputsIn: Byte);
begin
  if ((NumbOfOutputsIn > 0) and (NumbOfOutputsIn <> FNumberOfOutputNeurons))then
  begin
    FNumberOfOutputNeurons:=NumbOfOutputsIn;

    // new trainingpattern, new init of TedRBFnet required
    if assigned(FTedRBFnet) then
    begin
      FTedRBFnet.FTrainingInit := False;
      FTedRBFnet.FTedRBFnetInit := False;
    end;
  end;
end;

destructor TTopology.Destroy();
begin
  inherited Destroy();
end;
//end TTopology

//implementation TTedRBFnet
constructor TTedRBFnet.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FTopology := TTopology.Create();
  FTopology.FTedRBFnet := Self;

  //defaults
  FWeightMin:=-1;
  FWeightMax:=1;
  FSeedWeights:=-1;
  FActivate:=Gauss;
  FCenterMethod:=Cluster;
  FSigma:=0.5;
  FEta:=0.05;

  //flags for unspecified training, no initialisation of layers so far
  FTrainingInit:=False;
  FTedRBFnetInit:=False;
end;

procedure TTedRBFnet.SetFWeightMin(const WeightMinIn: double);
begin
  if ((WeightMinIn < 0) and (WeightMinIn >= -1)) then
  begin
    FWeightMin:=WeightMinIn;
  end;
end;

procedure TTedRBFnet.SetFWeightMax(const WeightMaxIn: double);
begin
  if ((WeightMaxIn >= 0) and (WeightMaxIn <= 1)) then
  begin
    FWeightMax:=WeightMaxIn;
  end;
end;

procedure TTedRBFnet.SetFSeedWeights(const SeedWeightsIn: Integer);
begin
  FSeedWeights:=SeedWeightsIn;
end;

procedure TTedRBFnet.SetFSigma(const SigmaIn: double);
begin
  if (SigmaIn > 0) then
  begin
    FSigma:=SigmaIn;
  end
end;

procedure TTedRBFnet.SetLearningRate(const LearningRateIn: double);
begin
  if (LearningRateIn > 0) then
  begin
    FEta:=LearningRateIn;
  end;
end;

procedure TTedRBFnet.SetTrainingPattern(const doubles: TDoubleArray; var
                        numberOfTrainingValues: integer);
var

  counterOfDoubles, i,j: integer;
begin
  numberOfTrainingValues:=length(doubles) div (FTopology.NumberOfInputNeurons+
                                             FTopology.NumberOfOutputNeurons);

  //get memory for trainingpattern
  setLength(FTrainingPattern,numberOfTrainingValues);
  for i:=0 to numberOfTrainingValues-1 do
  begin
    setLength(FTrainingPattern[i].input,
          FTopology.numberOfInputNeurons);
  setLength(FTrainingPattern[i].output,
          FTopology.numberOfOutputNeurons);
  end;

  counterOfDoubles:=0;
  for i:=0 to numberOfTrainingValues-1 do
  begin
    for j:=0 to FTopology.NumberOfInputNeurons-1 do
    begin
      FTrainingPattern[i].input[j]:=doubles[counterOfdoubles];
      counterOfDoubles:=counterOfDoubles+1;
    end;
    for j:=0 to FTopology.NumberOfOutputNeurons-1 do
    begin
      FTrainingPattern[i].output[j]:=doubles[counterOfdoubles];
      counterOfDoubles:=counterOfDoubles+1;
    end;
  end;
  FTrainingInit:=true;
end;



procedure TTedRBFnet.Init();
var
  i: integer;
begin
  if FTrainingInit then
  begin
    //catching wrong inputs by user
    if ((CenterMethod = Cluster) and (FTopology.numberOfHiddenNeurons >
                             Length(FTrainingPattern))) then
    begin
      errorHandling(1);
      exit;
    end;
    if ((CenterMethod = Direct) and (Length(FTrainingPattern) <>
                                FTopology.numberOfHiddenNeurons)) then
    begin
      errorHandling(2);
      exit;
    end;
    if ((CenterMethod = Subset) and (Length(FTrainingPattern) <
                                FTopology.numberOfHiddenNeurons)) then
    begin
      errorHandling(1);
      exit;
    end;

    setLength(FHiddenNeurons,FTopology.numberOfHiddenNeurons);
    for i:=0 to FTopology.numberOfHiddenNeurons-1 do
    begin
       //each center has dimension of inputvectors
      setlength(FHiddenNeurons[i].center,FTopology.numberOfInputNeurons);
    end;

    //init the centers
    case CenterMethod of
      Direct: InitCentersDirect(); //centers=inputs
      Subset: InitCentersRandomSubset(); //centers=randomsubset of inputs
      Cluster: InitCentersCluster(); //kmeans clustering
    end;

    InitOutputLayer();
    FTedRBFnetInit:=True;
  end
  else
  begin
    errorHandling(3);
  end;
end;

procedure TTedRBFnet.InitOutputLayer();
VAR
   i,j: Integer;

begin
  RandSeed:=FSeedWeights;

  SetLength(FOutputNeurons, FTopology.NumberOfOutputNeurons);

  for i:=0 to FTopology.NumberOfOutputNeurons-1 do
  begin
    //one weight for each hidden neuron
    SetLength(FOutputNeurons[i].Weight, FTopology.NumberOfHiddenNeurons);
    for j:=0 to FTopology.NumberOfHiddenNeurons-1 do
    begin
      //starting value between weightmin..weightmax
      FOutputNeurons[i].Weight[j] := Random * (FWeightMax-FWeightMin) + WeightMin;
    end;
  end;
end;


procedure TTedRBFnet.CopyArray(var LCopy: array of double;
                         const LOriginal: array of double);
VAR
  i: Integer;
begin
  if (Length(LCopy) <> Length(LOriginal)) then
  begin
    errorHandling(4);
    exit;
  end;

  for i:=0 to High(LOriginal) do
    LCopy[i] := LOriginal[i];


end;

procedure TTedRBFnet.InitCentersRandomSubset();
VAR
  i, fortune: Integer;
  choice: array of byte;  //array with the indices of the available inputs
begin
  RandSeed := -1;

  setLength(choice,High(FTrainingPattern)+1);
  for i:=0 to High(choice) do
  begin
    //initialising with the indices
    choice[i]:=i;
  end;

  for i:=0 to FTopology.NumberOfHiddenNeurons-1 do
  begin
    //choosing from input vectors
    fortune := Round (Random * (High(choice) + 1) - 0.5);
    //this vector will be a center
    CopyArray(FHiddenNeurons[i].Center,FTrainingPattern[choice[fortune]].input);
    //this vector wont be a choice no more
    choice[fortune]:=choice[High(choice)];
    SetLength(choice, High(choice));
  end;
end;


// Basic-Minimum-Squared-Error procedure for kmeans-clustering of the hidden
// layers centers
// initial condition for prototypes: order of input
// ref. Duda und Hart: Pattern Classification and Scene Analysis, 1973. p. 227
procedure TTedRBFnet.InitCentersCluster();
VAR
  i, j, k, counter,

  //number Of Training values
  numberOfTrainingVec,

  // number of clusters, is equal numberOfHiddenNeurons
    numberOfClust,
  //the currently investigated elements index and its cluster
  PivotElem, PivotPart,

  //index of the new cluster of the pivot_element
  IndexOfNewCluster:                               Integer;

  //centers of clusters, mean[i]: ith clusters center
  mean : array of TDoubleArray;

  //number of vectors in each cluster
  sizeOfCluster: array of Integer;

  //clusters: [index of cluster][index of element]
  //                      [index of elements component],

  Partition: array of array of TDoubleArray;

  //errorsum for each cluster
  errorSum : array of double;

  //min: this value has to be undercut to change clusters
  min, ErrorChange: double;

begin

  RandSeed := -1;

  numberOfClust := FTopology.numberOfHiddenNeurons;

  //get memory for mean
  SetLength(mean, numberOfClust);
  for i:=0 to numberOfClust-1 do
    SetLength(mean[i], FTopology.numberOfInputNeurons);
  SetLength(sizeOfCluster, numberOfClust);
  SetLength(errorSum, numberOfClust);
  numberOfTrainingVec:=Length(FTrainingPattern);

  //initial condition: size of each cluster is equal (as far as possible)

  for i:=0 to numberOfClust-2 do
    sizeOfCluster[i] := numberOfTrainingVec Div numberOfClust;

  sizeOfCluster[numberOfClust-1] := numberOfTrainingVec - (numberOfClust-1) *
        (numberOfTrainingVec Div numberOfClust);

  SetLength(Partition, numberOfClust);
  for i:=0 to numberOfClust-1 do
  begin
    SetLength(Partition[i], sizeOfCluster[i]);
    for j:=0 to sizeOfCluster[i]-1 do
      //size of input vectors
      SetLength(Partition[i][j], FTopology.numberOfInputNeurons);
  end;

  counter := 0;
  //clusters are being filled
  for i:=0 to numberOfClust-1 do
  begin
    for j:=0 to sizeOfCluster[i]-1 do
    begin
      CopyArray(Partition[i][j],FTrainingPattern[counter].input);
      counter := counter + 1;
    end;
  end;

  //set mean 0 first
  for i:=0 to numberOfClust-1 do
    for j:=0 to FTopology.numberOfInputNeurons-1 do
      mean[i][j]:=0;

  for i:=0 to numberOfClust-1 do
  begin
    for j:=0 to sizeOfCluster[i]-1 do
    begin
      for k:=0 to FTopology.numberOfInputNeurons-1 do
          mean[i][k] := mean[i][k] + Partition[i][j][k];
    end;
  end;

  //normalize mean
  for i:=0 to numberOfClust-1 do
    for j:=0 to FTopology.numberOfInputNeurons-1 do

      mean[i][j] := mean[i][j] / sizeOfCluster[i];
  //mean is now calculated

  //calculation of errorsum
  for i:=0 to numberOfClust-1 do
  begin
    errorsum[i] := 0;
    for j:=0 to sizeOfCluster[i]-1 do
      errorsum[i] := errorsum[i] + DistanceSquare(Partition[i][j], mean[i]);
  end;


  counter := 0;
  //optimize now clusters in numberOfTrainingVec attempts
  while (counter < numberOfTrainingVec) do
  begin
    //choose random pivot element
    PivotPart := Trunc(Random*numberOfClust-0.00001);
    PivotElem := Trunc(Random * sizeOfCluster[PivotPart] - 0.00001);

    //partitions with only one element should stay
    if( sizeOfCluster[PivotPart] > 1 ) then
    begin
      //look at description of algorithm to understand the next steps
      min := sizeOfCluster[PivotPart] / (sizeOfCluster[PivotPart] - 1) *
           DistanceSquare(Partition[PivotPart][PivotElem], mean[PivotPart]);
      IndexOfNewCluster := PivotPart;
      //is there a cheaper partition?
      for j:=0 to numberOfClust-1 do
      begin
        if(j <> PivotPart) then
        begin
          ErrorChange := sizeOfCluster[j] / (sizeOfCluster[j] + 1)
           * DistanceSquare(Partition[PivotPart][PivotElem], mean[j]);
          if (ErrorChange < min) then
          begin
            min := ErrorChange;
            IndexOfNewCluster := j;
          end;
        end;
      end;


      //there is a cheaper partition, so rearrange clusters
      if(IndexOfNewCluster <> PivotPart) then
      begin
        SetLength(Partition[IndexOfNewCluster], Length(Partition[IndexOfNewCluster]) + 1);
        SetLength(Partition[IndexOfNewCluster][High(Partition[IndexOfNewCluster])],
                  FTopology.NumberOfInputNeurons);
        //get new memory for moving element
        for k:=0 to FTopology.NumberOfInputNeurons-1 do
          Partition[IndexOfNewCluster][High(Partition[IndexOfNewCluster])][k] := 0;
        CopyArray(Partition[IndexOfNewCluster][High(Partition[IndexOfNewCluster])],
                  Partition[PivotPart][PivotElem]);

        //change errorsum
        errorSum[PivotPart] := errorSum[PivotPart] -
                      sizeOfCluster[PivotPart] / (sizeOfCluster[PivotPart] - 1)
                          * DistanceSquare(Partition[PivotPart][PivotElem],
                                           mean[PivotPart]);

        //remove pivot-element from its old cluster
                CopyArray(Partition[PivotPart][PivotElem],
                  Partition[PivotPart][High(Partition[PivotPart])]);
        SetLength(Partition[PivotPart], High(Partition[PivotPart]));


        ErrorSum[IndexOfNewCluster] := ErrorSum[IndexOfNewCluster] + min;

        for k:=0 to FTopology.NumberOfInputNeurons-1 do
        begin
          mean[PivotPart][k] := mean[PivotPart][k] -
                (Partition[IndexOfNewCluster][High(Partition[IndexOfNewCluster])][k] -
                 mean[PivotPart][k]) / (sizeOfCluster[PivotPart] - 1);

          mean[IndexOfNewCluster][k] := mean[IndexOfNewCluster][k] +
                (Partition[IndexOfNewCluster][High(Partition[IndexOfNewCluster])][k] -
            mean[IndexOfNewCluster][k])/(sizeOfCluster[IndexOfNewCluster] + 1);
        end;

        //update sizes of clusters
        sizeOfCluster[PivotPart] := sizeOfCluster[PivotPart] - 1;
        sizeOfCluster[IndexOfNewCluster] := sizeOfCluster[IndexOfNewCluster] + 1;

        //restart algorithm
        counter := 0;
      end
      else
      //no cheaper partition found
      begin

        counter := counter + 1;
      end;
    end else
    // pivot element was the only one in its partition
    begin
      counter := counter + 1;
    end;
  end;

  //clustering finished, update centers of hidden layer
  for i:=0 to numberOfClust-1 do
    CopyArray(FHiddenNeurons[i].Center, mean[i]);
end;



procedure TTedRBFnet.InitCentersDirect();
var
  i: Integer;
begin
  for i:=0 to FTopology.NumberOfHiddenNeurons-1 do
    CopyArray(FHiddenNeurons[i].Center,FTrainingPattern[i].input);
end;

procedure TTedRBFnet.getOutput(const input: TDoubleArray; var OutputNet:
                                TDoubleArray);
var
  OutputHidden: TDoubleArray;
begin
  setLength(OutputHidden,FTopology.numberOfHiddenNeurons);
  getCompleteOutput(input,outputHidden,outputNet);
end;

procedure TTedRBFnet.getCompleteOutput(const input: TDoubleArray;
              var outputHidden: TDoubleArray; var outputNet: TDoubleArray);
var
  i,j: integer;
begin
// catch undefined inputs
  if not FTedRBFnetInit then
  begin
    errorHandling(5);
    exit;
  end;
  if ((length(OutputHidden) < FTopology.numberOfHiddenNeurons)) or
     ((length(OutputNet) < FTopology.numberOfOutputNeurons)) then
  begin
    errorHandling(6);
    exit;
  end;

  case FActivate of
    Gauss:
    begin
      //running through the hidden layer
      for i:=0 to FTopology.NumberOfHiddenNeurons-1 do
      begin
        OutputHidden[i] :=
                exp(-DistanceSquare(input,
                   FHiddenNeurons[i].Center)/(2 * Fsigma * Fsigma) );
      end;


      //output layer
      for i:=0 to FTopology.NumberOfOutputNeurons-1 do
      begin
        OutputNet[i] := 0;
        //running through the weights of the ith neuron of the outputlayer
        for j:=0 to FTopology.NumberOfHiddenNeurons-1 do
        begin
          OutputNet[i] := OutputNet[i] +
          FOutputNeurons[i].weight[j] * OutputHidden[j];
        end;
      end;
    end;
  end;
end;

function TTedRBFnet.errorOfTrainingData(): double;
var
  error: double;
  trainingIndex,i : integer;
  OutputHidden, OutputNet: array of TDoubleArray;
begin
  if (not FTedRBFnetInit) then
  begin
    errorHandling(9);
    errorOfTrainingData:=0;
    exit;
  end;

  //get datastructure for outputs
  SetLength(OutputHidden, Length(FTrainingPattern));
  SetLength(OutputNet, Length(FTrainingPattern));
  for i:=0 to High(FTrainingPattern) do
  begin
    SetLength(OutputHidden[i], FTopology.NumberOfHiddenNeurons);
    SetLength(OutputNet[i], FTopology.NumberOfOutputNeurons);
  end;

  //compare net output with training output
  error:=0;
  for trainingIndex:=0 to High(FTrainingPattern) do
  begin
    getCompleteOutput(FTrainingPattern[trainingIndex].input,
        OutputHidden[trainingIndex],
        OutputNet[trainingIndex]);
    error:=error+sqrt(DistanceSquare(FTrainingPattern[trainingIndex].output,
                      OutputNet[trainingIndex]));;
  end;
  errorOfTrainingData:=error;
end;



function TTedRBFnet.DistanceSquare(x,c: array of double): double;
VAR
   i: Integer;
   d: double;
begin
     d:=0;

     for i:=0 to Length(x)-1 do
         d := d + (x[i]-c[i]) * (x[i]-c[i]);

     DistanceSquare := d;
end;

procedure TTedRBFnet.getTrainingPattern(var pattern: TTrainingPattern);
var
  i: integer;
begin
  if (not FTrainingInit) then
  begin
    errorHandling(7);
    exit;
  end;
  setLength(pattern,length(FTrainingPattern));
  for i:=0 to length(FTrainingPattern)-1 do
  begin
    setLength(pattern[i].input,FTopology.numberOfInputNeurons);
    copyArray(pattern[i].input,FTrainingPattern[i].input);
    setLength(pattern[i].output,FTopology.numberOfOutputNeurons);
    copyArray(pattern[i].output,FTrainingPattern[i].output);
  end;
end;

procedure TTedRBFnet.errorHandling(errorIndex: byte);
begin
  case errorIndex of
    1: showMessage('Check your number of input vectors!');
    2: showMessage('Check your number of centers!');
    3: showMessage('Must have a training-pattern');
    4: showMessage('Arrays arent of the same length');
    5: showMessage('Net is not initialised;');
    6: showMessage('Lengths of arrays are not adequate');
    7: showMessage('Training not initialised.');
    8: showMessage('Check number of learning cycles');
    9: showMessage('Net is not initialised');
  end;
end;

destructor TTedRBFnet.Destroy();
begin
  FTopology.Free();
  inherited Destroy();
end;

function TTedRBFnet.isInit(): boolean;
begin
  isInit:=FTedRBFnetInit;
end;

procedure TTedRBFnet.learning(var numberOfLearnCycles: Integer;
                         const abortBound: double=-1);
var
  //outputs of hidden layer and output layer for the whole training pattern
  OutputHidden, OutputNet: array of TDoubleArray;
  error, sum: double;
  i,j,k,cycleIndex,trainingIndex: integer;
begin
  // catch undefined inputs by user
  if (numberOfLearnCycles < 1)then
  begin
    errorHandling(8);
    exit;
  end;
  if (not FTedRBFnetInit) then
  begin
    errorHandling(9);
    exit;
  end;

  //get datastructure for outputs
  SetLength(OutputHidden, Length(FTrainingPattern));
  SetLength(OutputNet, Length(FTrainingPattern));
  for i:=0 to High(FTrainingPattern) do
  begin
    SetLength(OutputHidden[i], FTopology.NumberOfHiddenNeurons);
    SetLength(OutputNet[i], FTopology.NumberOfOutputNeurons);
  end;

  //get state of the net with training inputs
  error:=0;
  for trainingIndex:=0 to High(FTrainingPattern) do
  begin
    getCompleteOutput(FTrainingPattern[trainingIndex].input,
        OutputHidden[trainingIndex],
        OutputNet[trainingIndex]);
    error:=error+sqrt(DistanceSquare(FTrainingPattern[trainingIndex].output,
                      OutputNet[trainingIndex]));;
  end;

  cycleIndex:=0;
  while ((cycleIndex < numberOfLearnCycles) and (error > abortBound)) do
  begin
    //start learning weights of output neurons
    for i:=0 to FTopology.numberOfOutputNeurons-1 do
    begin
      //running through weights of ith output neuron
      for j:=0 to FTopology.numberOfHiddenNeurons-1 do
      begin
        sum := 0;
        //delta rule
        for k:=0 to High(FTrainingPattern) do
        begin
          sum := sum + OutputHidden[k][j] * (FTrainingPattern[k].output[i] -
                                             OutputNet[k][i]);
        end;
        FOutputNeurons[i].weight[j] := FOutputNeurons[i].weight[j]
                                              +  FEta * sum;
      end;

    end;  //end of learning

    //get error after learning and update net output
    error:=0;
    for trainingIndex:=0 to High(FTrainingPattern) do
    begin
      getCompleteOutput(FTrainingPattern[trainingIndex].input,
        OutputHidden[trainingIndex],
        OutputNet[trainingIndex]);
      error := error +
                   sqrt(DistanceSquare(FTrainingPattern[trainingIndex].output,
                      OutputNet[trainingIndex]));
    end;
    cycleIndex:=cycleIndex+1;
  end; //end of learning cycles
  numberOfLearnCycles:=cycleIndex;
end;

end.
