{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with the
License. You may obtain a copy of the License at
http://www.mozilla.org/NPL/NPL-1_1Final.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: mwGenericLex.pas, released April, 2001.

The Initial Developer of the Original Code is Martin Waldenburg
(Martin.Waldenburg@T-Online.de).
Portions created by Martin Waldenburg are Copyright (C) 2001 Martin Waldenburg.
All Rights Reserved.

Alternatively, the contents of this file may be used under the terms of the
GNU General Public License Version 2 or later (the "GPL"), in which case
the provisions of the GPL are applicable instead of those above.
If you wish to allow use of your version of this file only under the terms
of the GPL and not to allow others to use your version of this file
under the MPL, indicate your decision by deleting the provisions above and
replace them with the notice and other provisions required by the GPL.
If you do not delete the provisions above, a recipient may use your version
of this file under either the MPL or the GPL.

Contributor(s): _____________________________________.


Last Modified: 11. November 02
Current Version: 1.3

Notes: This program is a fast generic scriptable lexical analyser.

Modification history:

Known Issues:
-----------------------------------------------------------------------------}
{$A+}
unit mwLexStates;

interface

uses
  {$IfDef MsWindows}
  Windows, SysUtils, Classes, Dialogs;
  {$EndIf}

  {$IfDef Linux}
  SysUtils, Types, Classes, Libc, QDialogs;
  {$EndIf}

  const
  piAtEnd = High(Word);
  piZero = High(Word) - 1;
  piUnknown = High(Word) - 2;
  piSymbol = 0;
  piSpace = 1;
  piLineEnd = 2;
  piInnerLineEnd = 3;
  piIdent = 4;
  piKeyWord = 5;
  piString = 6;
  piComment = 7;
  piNumber = 8;
  piAssembler = 9;
  piBadString = 10;
  piAsmKeyWord = 11;

type

  TmwAnyBits = (
    BitZero,
    BitOne,
    BitTwo,
    BitThree,
    BitFour,
    BitFive,
    BitSix,
    BitSeven
  );

  TmwAnyBitSet = Set of TmwAnyBits;

  PmwCharSet = ^TmwCharSet;
  TmwCharSet = set of Char;
  TmwWideChar = array[0..1] of Char;
  PmwWideCharSet = ^TmwWideCharSet;
  TmwWideCharSet = array[#0..#255] of TmwCharSet;

  TmwLexStateKind = (
    skAny,
    skCharSet,
    skGenericLex,
    skIdentifiers,
    skInnerLineEnd,
    skKey,
    skLineEnd,
    skNotCharSet,
    skNotKey,
    skTillKey,
    skTillNotKey,
    skTillKeyMultiLine,
    skTillNotKeyMultiLine,

    skKeyInSensitive,
    skNotKeyInSensitive,
    skTillKeyInSensitive,
    skTillNotKeyInSensitive,
    skTillKeyMultiLineInSensitive,
    skTillNotKeyMultiLineInSensitive,

    skWideAny,
    skWideCharSet,
    skWideIdentifiers,
    skWideInnerLineEnd,
    skWideKey,
    skWideLineEnd,
    skWideNotCharSet,
    skWideNotKey,
    skWideLex,
    skWideTillKey,
    skWideTillNotKey,
    skWideTillKeyMultiLine,
    skWideTillNotKeyMultiLine,

    skWideKeyInSensitive,
    skWideNotKeyInSensitive,
    skWideTillKeyInSensitive,
    skWideTillNotKeyInSensitive,
    skWideTillKeyMultiLineInSensitive,
    skWideTillNotKeyMultiLineInSensitive,

    skSpecial
  );

  TmwLexState = class;

  TmwAnyProc = procedure(Sender: TmwLexState) of object;

  PmwAnsiSelector = ^TmwAnsiSelector;
  TmwAnsiSelector =  array[#0..#255] of TmwLexState;
  TmwWideSelector = class;

  TmwLexStateKeys = packed record
    case Integer of
      0: (Key: PChar);
      1: (WKey: PWideChar);
      2: (CharSet: PmwCharSet);
      3: (WCharSet: PmwWideCharSet);
      4: (Ansi: PmwAnsiSelector);
      5: (Wide: TmwWideSelector);
  end;

  TmwLexStateData = packed record
    Kind: TmwLexStateKind;
    Bits: TmwAnyBitSet;
    Max: Byte;
    Min: Byte;
    ExId: Word;
    Id: Word;
    Len: Word;
    Count: Word;
  end;

  TmwWideSelectorItem =  packed record
    WC: WideChar;
    Any: TmwLexState;
  end;
  
  PmwWideSelectorSubList = ^TmwWideSelectorSubList;
  TmwWideSelectorSubList = array[0..0] of TmwWideSelectorItem;

  TmwWideSelectorSubListItem = packed record
    List: PmwWideSelectorSubList;
    Count: Integer;
  end;

  TmwWideSelector = class(TObject)
  private
    Owner: TmwLexState;
    fLargest: WideChar;
    fList: array[#0..#255] of TmwWideSelectorSubListItem;
    fSmallest: WideChar;
    function Compare(C1, C2: WideChar): Integer;
    procedure Insert(C: WideChar; Any: TmwLexState; Index: Integer);
    function GetContent(Index: WideChar): TmwLexState;
  public
    constructor Create(aOwner: TmwLexState);
    destructor Destroy; override;
    procedure Add(C: WideChar; Any: TmwLexState);
    procedure Clear;
    property Content[Index: WideChar]: TmwLexState read GetContent; default;
  end;

  PmwLexOptionsList = ^TmwLexOptionsList;
  TmwLexOptionsList = array[0..0] of TmwLexState;

  TmwGenericLexer = class;

  TmwLexState = class(TPersistent)
  private
    procedure SetKeyString(const Value: String);
    procedure SetWideKeyString(const Value: WideString);
    function GetCharSet: TmwCharSet;
    procedure SetCharSet(const Value: TmwCharSet);
    function GetWideCharSet(Index: WideChar): Boolean;
    procedure SetWideCharSet(Index: WideChar; const Value: Boolean);
    function GetNodes(Index: Integer): TmwLexState;
    function GetInSensitive: Boolean;
    function GetMultiline: Boolean;
    function GetOptional: Boolean;
    function GetSkip: Boolean;
    function GetTerminal: Boolean;
    procedure SetInSensitive(const Value: Boolean);
    procedure SetMultiline(const Value: Boolean);
    procedure SetOptional(const Value: Boolean);
    procedure SetSkip(const Value: Boolean);
    procedure SetTerminal(const Value: Boolean);
  protected
    fFollow: TmwLexState;
    fToRegress: TmwLexState;
    fToReStore: TmwLexState;
    fOnDoSomething: TmwAnyProc;
    List: PmwLexOptionsList;
    Data: TmwLexStateData;
    Keys: TmwLexStateKeys;
    OwningLex: TmwGenericLexer;
    Owner: TmwLexState;
    function Add(const Value: TmwLexState): Integer; virtual;
    function Insert(Index: Integer; const Value: TmwLexState): Integer;
    property Options[Index: Integer]: TmwLexState read GetNodes; default;
  public
    destructor Destroy; override;
    procedure Clear; virtual;
    procedure SetWideSet(WS: WideString);
    property CharSet: TmwCharSet read GetCharSet write SetCharSet;
    property Key: PChar read Keys.Key;
    property KeyString: String write SetKeyString;
    procedure Negate;
    procedure NegateTill;
    property WideCharSet[Index: WideChar]: Boolean read GetWideCharSet write SetWideCharSet;
    property WideKey: PWideChar read Keys.WKey;
    property WideKeyString: WideString write SetWideKeyString;
    procedure SetGenericValues(aOwner: TmwLexState; IsOptional: Boolean; IsInSensitive: Boolean; anId, AnExId: Word);
  published
    property ExId: Word read Data.ExId write Data.ExId;
    property Follow: TmwLexState read fFollow write fFollow;
    property Id: Word read Data.Id write Data.Id;
    property InSensitive: Boolean read GetInSensitive write SetInSensitive;
    property Kind: TmwLexStateKind read Data.Kind write Data.Kind;
    property Len: Word read Data.Len write Data.Len;
    property Max: Byte read Data.Max write Data.Max;
    property Min: Byte read Data.Min write Data.Min;
    property MultiLine: Boolean read GetMultiline write SetMultiline;
    property Optional: Boolean read GetOptional write SetOptional;
    property Skip: Boolean read GetSkip write SetSkip;
    property Terminal: Boolean read GetTerminal write SetTerminal;
    property ToRegress: TmwLexState read fToRegress write fToRegress;
    property ToReStore: TmwLexState read fToReStore write fToReStore;
    property OnDoSomething: TmwAnyProc read fOnDoSomething write fOnDoSomething;
  end;


  TmwKeyWordSubList = array[0..0] of TmwLexState;
  PmwKeyWordSubList = ^TmwKeyWordSubList;

  TmwKeyWordSub = packed Record
    List: PmwKeyWordSubList;
    Count: Integer;
  end;
  
  TmwKeyWordList = array[Char] of TmwKeyWordSub;

  TmwSpecial = class(TmwLexState)
  private
    fSpecialKind: TmwLexStateKind;
  protected
    function Match(var P: PChar; TheEnd: PChar): TmwLexState; virtual; abstract;
  public
    property SpecialKind: TmwLexStateKind read fSpecialKind write fSpecialKind;
  end;

  TmwIdentifiers = class(TmwSpecial)
  private
    function Compare(aKey: PChar; aLen: Integer; Item: TmwLexState): Boolean;
    procedure Delete(Index: Integer; KeyIndex: Char);
    function GetContent: TmwCharSet;
    procedure SetContent(const Value: TmwCharSet);
  protected
    ContentArray: array[Char] of ByteBool;
    KeyList: TmwKeyWordList;
    procedure Insert(Index: Integer; Item: TmwLexState);
  public
    constructor Create(AOwner: TmwLexState; IsOptional: Boolean; aSet, aContent: TmwCharset);
    procedure Clear; override;
    procedure AddKey(const aKey: String; IsSensitive: Boolean; anId, AnExId: Word; RestoreIt: TmwLexState; IsTerminal: Boolean);
    function GetKeyWord(P: PChar; aLen: Integer; var AnIndex: Integer): TmwLexState;
    function Match(var P: PChar; TheEnd: PChar): TmwLexState; override;
    procedure Remove(S: String);
    property Content: TmwCharSet read GetContent write SetContent;
  end;

  TmwWideIdentifiers = class(TmwSpecial)
  private
    function Compare(aKey: PWideChar; aLen: Integer; Item: TmwLexState): Boolean;
    procedure Delete(Index: Integer; KeyIndex: Char);
    function GetWideCharContent(Index: WideChar): Boolean;
    procedure SetWideCharContent(Index: WideChar; const Value: Boolean);
  protected
    fWideContent: TmwWideCharSet;
    KeyList: TmwKeyWordList;
    procedure Insert(Index: Integer; Item: TmwLexState);
  public
    constructor Create(AParent: TmwLexState; aSet, aContent: WideString);
    procedure Clear; override;
    procedure AddKey(const aKey: WideString; IsSensitive: Boolean; anId, AnExId: Word; RestoreIt: TmwLexState; IsTerminal: Boolean);
    function GetKeyWord(P: PWideChar; aLen: Integer; var AnIndex: Integer): TmwLexState;
    function Match(var P: PWideChar; TheEnd: PWideChar): TmwLexState; reintroduce;
    procedure Remove(S: WideString);
    procedure SetWideContent(WS: WideString);
    property WideCharContent[Index: WideChar]: Boolean read GetWideCharContent write SetWideCharContent;
  end;

  TmwLexInitProc = procedure(Lex: TmwGenericLexer);
  TmwLexInitMethod = procedure(Lex: TmwGenericLexer) of object;

  TmwCharPointers = packed record
    Ansi: PChar;
    Wide: PWideChar;
  end;

  TmwGenericLexer = class(TmwLexState)
  private
    fMBCS: Boolean;
    FInitMethod: TmwLexInitMethod;
    FInitProc: TmwLexInitProc;
    function GetAtEnd: Boolean;
    procedure SetOrigin(const Value: PChar);
    procedure SetInitMethod(const Value: TmwLexInitMethod);
    procedure SetInitProc(const Value: TmwLexInitProc);
    procedure SetTheEnd(const Value: PChar);
  protected
    Current: TmwLexState;
    fAny: TmwLexState;
    fInnerLineEnd: TmwLexState;
    fOrigin: TmwCharPointers;
    FProcessing: Boolean;
    fRange: TmwLexState;
    fRun: TmwCharPointers;
    fStart: TmwCharPointers;
    fTheEnd: TmwCharPointers;
    Selector: array[Low(TmwLexStateKind)..High(TmwLexStateKind)] of function: Boolean of object;
    function Add(const Value: TmwLexState): Integer; override;
    procedure AddToSelector(Item: TmwLexState); virtual;
    procedure AddKeyInsensitive(Item: TmwLexState); virtual;
    function ApplyAny: Boolean;
    function ApplyCharSet: Boolean;
    function ApplyKey: Boolean;
    function ApplyLineEnd: Boolean;
    function ApplyNotCharSet: Boolean;
    function ApplyNotKey: Boolean;
    function ApplyTillKey: Boolean;
    function ApplyTillNotKey: Boolean;
    function ApplyTillKeyMultiLine: Boolean;
    function ApplyTillNotKeyMultiLine: Boolean;

    function ApplyKeyInSensitive: Boolean;
    function ApplyNotKeyInSensitive: Boolean;
    function ApplyTillKeyInSensitive: Boolean;
    function ApplyTillNotKeyInSensitive: Boolean;
    function ApplyTillKeyMultiLineInSensitive: Boolean;
    function ApplyTillNotKeyMultiLineInSensitive: Boolean;
    
    function ApplyWideAny: Boolean;
    function ApplyWideCharSet: Boolean;
    function ApplyWideKey: Boolean;
    function ApplyWideLineEnd: Boolean;
    function ApplyWideNotCharSet: Boolean;
    function ApplyWideNotKey: Boolean;
    function ApplyWideTillKey: Boolean;
    function ApplyWideTillNotKey: Boolean;
    function ApplyWideTillKeyMultiLine: Boolean;
    function ApplyWideTillNotKeyMultiLine: Boolean;

    function ApplyWideKeyInSensitive: Boolean;
    function ApplyWideNotKeyInSensitive: Boolean;
    function ApplyWideTillKeyInSensitive: Boolean;
    function ApplyWideTillNotKeyInSensitive: Boolean;
    function ApplyWideTillKeyMultiLineInSensitive: Boolean;
    function ApplyWideTillNotKeyMultiLineInSensitive: Boolean;

    function ApplySpecial: Boolean; virtual;
    procedure ClearSelector; virtual;
    procedure CreateHelpers; virtual;
    function GetCurrent(P: Pointer): TmwLexState; virtual;
    function GetToken: string; virtual;
    function IncMBCRun: Boolean; virtual;
    function IncRun: Boolean; virtual;
    function IncWideRun: Boolean; virtual;
    procedure InitSelector;
    function Match: Boolean; virtual;
    function MatchOptions: Boolean;
    function MinToMax: Boolean;
    procedure ReleaseHelpers; virtual;
  public
    constructor Create(aOwner: TmwLexState);
    procedure SetStartData(Ptr: Pointer; Len: Integer);
    property AtEnd: Boolean read GetAtEnd;
    property InitProc: TmwLexInitProc read FInitProc write SetInitProc;
    property InitMethod: TmwLexInitMethod read FInitMethod write SetInitMethod;
    property MBCS: Boolean read fMBCS write fMBCS;
    function Next: Boolean;
    property Origin: PChar read FOrigin.Ansi write SetOrigin;
    property Processing: Boolean read fProcessing;
    property Range: TmwLexState read fRange write fRange;
    property TheEnd: PChar read fTheEnd.Ansi write SetTheEnd;
    property Token: string read GetToken;
  end;

  TmwWideLex = class(TmwGenericLexer)
  private
    function GetWideToken: WideString;
  protected
    function Add(const Value: TmwLexState): Integer; override;
    procedure AddToSelector(Item: TmwLexState); override;
    procedure AddKeyInsensitive(Item: TmwLexState); override;
    function ApplySpecial: Boolean; override;
    procedure ClearSelector; override;
    procedure CreateHelpers; override;
    function GetCurrent(P: Pointer): TmwLexState; override;
    procedure ReleaseHelpers; override;
  public
    property Token: WideString read GetWideToken;
  end;

function CreateAny(aOwner: TmwLexState; IsOptional: Boolean; anId, AnExId: Word): TmwLexState;
function CreateInnerLineEnd(aOwner: TmwLexState; IsOptional: Boolean): TmwLexState;
function CreateKey(aOwner: TmwLexState; const S: String; IsOptional: Boolean; IsInSensitive: Boolean; anId, AnExId: Word): TmwLexState;
function CreateLineEnd(aOwner: TmwLexState; IsOptional: Boolean): TmwLexState;
function CreateNotKey(aOwner: TmwLexState; const S: String; IsOptional: Boolean; IsInSensitive: Boolean; anId, AnExId: Word): TmwLexState;
function CreateNotSet(aOwner: TmwLexState; const aSet: TmwCharSet; IsOptional: Boolean; anId, AnExId: Word): TmwLexState;
function CreateSet(aOwner: TmwLexState; const aSet: TmwCharSet; IsOptional: Boolean; anId, AnExId: Word): TmwLexState;
function CreateTillKey(aOwner: TmwLexState; const S: String; IsOptional: Boolean; IsInSensitive: Boolean; anId, AnExId: Word): TmwLexState;
function CreateTillNotKey(aOwner: TmwLexState; const S: String; IsOptional: Boolean; IsInSensitive: Boolean; anId, AnExId: Word): TmwLexState;
function CreateWideAny(aOwner: TmwLexState; IsOptional: Boolean; anId, AnExId: Word): TmwLexState;
function CreateWideInnerLineEnd(aOwner: TmwLexState; IsOptional: Boolean): TmwLexState;
function CreateWideKey(aOwner: TmwLexState; const WS: WideString; IsOptional: Boolean; IsInSensitive: Boolean; anId, AnExId: Word): TmwLexState;
function CreateWideLineEnd(aOwner: TmwLexState; IsOptional: Boolean): TmwLexState;
function CreateWideNotKey(aOwner: TmwLexState; const WS: WideString; IsOptional: Boolean; IsInSensitive: Boolean; anId, AnExId: Word): TmwLexState;
function CreateWideNotSet(aOwner: TmwLexState; const WS: WideString; IsOptional: Boolean; anId, AnExId: Word): TmwLexState;
function CreateWideSet(aOwner: TmwLexState; const WS: WideString; IsOptional: Boolean; anId, AnExId: Word): TmwLexState;
function CreateWideTillKey(aOwner: TmwLexState; const WS: WideString; IsOptional: Boolean; IsInSensitive: Boolean; anId, AnExId: Word): TmwLexState;
function CreateWideTillNotKey(aOwner: TmwLexState; const WS: WideString; IsOptional: Boolean; IsInSensitive: Boolean; anId, AnExId: Word): TmwLexState;

function CompareWideCharI(WC1, WC2: WideChar): Boolean;

function IsWideUpperCase(const aWideChar: WideChar): Boolean;

function WideLowerChar(const aWideChar: WideChar): WideChar;

implementation

type
  TmwWidePair = array[0..1] of WideChar;
  PmwWidePairSubList = ^TmwWidePairSubList;
  TmwWidePairSubList = array[0..0] of TmwWidePair;

  TmwSubListItem = packed record
    List: PmwWidePairSubList;
    Count: Integer;
  end;

  TmwWidePairList = class(TObject)
  private
    fLargest: WideChar;
    Selector: array[#0..#255] of TmwSubListItem;
    fSmallest: WideChar;
    function Compare(C1, C2: WideChar): Integer;
    procedure Insert(C: WideChar; Index: Integer);
    function GetContent(Index: WideChar): WideChar;
  protected
    function GetPair(Index: WideChar): TmwWidePair;
  public
    destructor Destroy; override;
    procedure Add(C: WideChar);
    procedure Clear;
    property Content[Index: WideChar]: WideChar read GetContent; default;
  end;

var
  CompTable: array[#0..#255] of Char;
  PairList: TmwWidePairList;
  LeadTable: array[#0..#255] of ByteBool;

procedure InitTables;
var
  I: Char;
  S: String;
  WC: WideChar;
  J: WideChar;
begin
  for I := #0 to #255 do
  begin
    LeadTable[I]:= I in LeadBytes;
    S:= AnsiLowerCase(I);
    Case S = '' of
      True: CompTable[I] := I;
      False: CompTable[I] := S[1];
    end;
  end;

  for J := Low(WideChar) to High(WideChar) do
  begin
      {$IFDEF LINUX}
    if (IsAlphaW(Integer(J)) <> 0) and (IsUpper(Integer(J)) <> 0) then
      {$ENDIF}
      {$IFDEF MSWINDOWS}
    if IsCharAlphaW(J) then
      {$ENDIF}
    begin
      WC:= WideLowerCase(J)[1];
      if WC <> J then
        PairList.Add(J);
    end;
  end;
end;

function CompareWideCharI(WC1, WC2: WideChar): Boolean;
begin
  Result:= True;
  if WC1 <> WC2 then
    Result:= PairList[WC1] = PairList[WC2];
end;

function IsWideUpperCase(const aWideChar: WideChar): Boolean;
begin
  Result:= PairList[aWideChar] <> aWideChar;
end;

function WideLowerChar(const aWideChar: WideChar): WideChar;
begin
  Result:= PairList[aWideChar];
end;

{ TmwWidePairList }
procedure TmwWidePairList.Add(C: WideChar);
var
  aKey: Char;
  SubList: PmwWidePairSubList;
  SubCount: Integer;
  First, Last, I: Integer;
  Larger: ByteBool;
begin
  aKey:= PChar(@C)^;
  SubCount:= Selector[aKey].Count;
  I:= 0;
  if SubCount > 0 then
  begin
    SubList:= Selector[aKey].List;
    First:=0;
    Last:= SubCount-1;
    Larger:= False;
    while First<=Last do
    begin
      I:=(First+Last)shr 1;
      Case Compare(C, SubList[I][0]) of
        -1:
          begin
            Last:=I-1;
            Larger:=False;
          end;
        0: Exit;
        1:
          begin
            First:=I+1;
            Larger:=True;
          end;
      end;
    end;
    Case Larger of
      True: Insert(C, I+1);
      False: Insert(C, I);
    end;
  end else Insert(C, 0);
end;

procedure TmwWidePairList.Clear;
var
  I: Char;
  Sub: TmwSubListItem;
begin
  for I:= #0 to #255 do
  begin
    Sub:= Selector[I];
    ReallocMem(Sub.List, 0);
    Sub.Count:= 0;
    Selector[I]:= Sub;
  end;
end;

function TmwWidePairList.Compare(C1, C2: WideChar): Integer;
begin
  Result:= 0;
  if C1 < C2 then Result:= -1 else
    if C1 > C2 then Result:= 1;
end;

destructor TmwWidePairList.Destroy;
begin
  Clear;
  inherited Destroy;
end;

function TmwWidePairList.GetContent(Index: WideChar): WideChar;
var
  aKey: Char;
  SubList: PmwWidePairSubList;
  SubCount: Integer;
  First, Last, Temp: Integer;
begin
  Result:= Index;
  if (Index < fSmallest) or (Index > fLargest) then Exit;
  aKey:= PChar(@Index)^;
  SubCount:= Selector[aKey].Count;
  if SubCount = 0 then Exit;
  case SubCount > 1 of
    True:
      begin
        First:= 0;
        Last:= SubCount -1;
        SubList:= Selector[aKey].List;
        while First <= Last do
        begin
          Temp:= (First + Last) div 2;
          case Compare(Index, SubList[Temp][0]) of
           1: First:= Temp +1;
           0:
             begin
               Result:= SubList[Temp][1];
               Exit;
             end;
           -1: Last:= Temp-1;
          end;
        end;
      end;
    False:
      begin
        SubList:= Selector[aKey].List;
        Result:= SubList[0][1];
      end;
  end;
end;

function TmwWidePairList.GetPair(Index: WideChar): TmwWidePair;
var
  aKey: Char;
  SubList: PmwWidePairSubList;
  SubCount: Integer;
  First, Last, Temp: Integer;
begin
  Result[0]:= Index;
  Result[1]:= Index;
  if (Index < fSmallest) or (Index > fLargest) then Exit;
  aKey:= PChar(@Index)^;
  SubCount:= Selector[aKey].Count;
  if SubCount = 0 then Exit;
  case SubCount > 1 of
    True:
      begin
        First:= 0;
        Last:= SubCount -1;
        SubList:= Selector[aKey].List;
        while First <= Last do
        begin
          Temp:= (First + Last) div 2;
          case Compare(Index, SubList[Temp][0]) of
           1: First:= Temp +1;
           0:
             begin
               Result:= SubList[Temp];
               Exit;
             end;
           -1: Last:= Temp-1;
          end;
        end;
      end;
    False:
      begin
        SubList:= Selector[aKey].List;
        Result:= SubList[0];
      end;
  end;
end;

procedure TmwWidePairList.Insert(C: WideChar; Index: Integer);
var
  aKey: Char;
  Sub: TmwSubListItem;
begin
  if Ord(fSmallest) = 0 then fSmallest:= C else
    if C < fSmallest then fSmallest:= C;
  if C > fLargest then C:= fLargest;
  aKey:= TmwWideChar(C)[0];
  Sub:= Selector[aKey];
  ReallocMem(Sub.List, (Sub.Count +1) * SizeOf(TmwWidePair));
  if Index < Sub.Count then
    System.Move(Sub.List[Index], Sub.List[Index + 1],
      (Sub.Count - Index) * SizeOf(TmwWidePair));
  Sub.List[Index][0] := C;
  Sub.List[Index][1] := WideLowerCase(C)[1];
  inc(Sub.Count);

  Selector[aKey]:= Sub;
end;

{ TmwWideSelector }

procedure TmwWideSelector.Add(C: WideChar; Any: TmwLexState);
var
  aKey: Char;
  SubList: PmwWideSelectorSubList;
  SubCount: Integer;
  First, Last, I: Integer;
  Larger: ByteBool;
begin
  aKey:= TmwWideChar(C)[0];
  SubCount:= fList[aKey].Count;
  I:= 0;
  if SubCount > 0 then
  begin
    SubList:= fList[aKey].List;
    First:=0;
    Last:= SubCount-1;
    Larger:= False;
    while First<=Last do
    begin
      I:=(First+Last)shr 1;
      Case Compare(C, SubList[I].WC) of
        -1:
          begin
            Last:=I-1;
            Larger:=False;
          end;
        0: Exit;
        1:
          begin
            First:=I+1;
            Larger:=True;
          end;
      end;
    end;
    Case Larger of
      True: Insert(C, Any, I+1);
      False: Insert(C, Any, I);
    end;
  end else Insert(C, Any, 0);
end;

procedure TmwWideSelector.Clear;
var
  I: Char;
  Sub: TmwWideSelectorSubListItem;
begin
  for I:= #0 to #255 do
  begin
    Sub:= fList[I];
    ReallocMem(Sub.List, 0);
    Sub.Count:= 0;
    fList[I]:= Sub;
  end;
end;

function TmwWideSelector.Compare(C1, C2: WideChar): Integer;
begin
  Result:= 0;
  if C1 < C2 then Result:= -1 else
    if C1 > C2 then Result:= 1;
end;

constructor TmwWideSelector.Create(aOwner: TmwLexState);
begin
  Owner:= aOwner;
end;

destructor TmwWideSelector.Destroy;
begin
  Clear;
  inherited Destroy;
end;

function TmwWideSelector.GetContent(Index: WideChar): TmwLexState;
var
  aKey: Char;
  SubList: PmwWideSelectorSubList;
  SubCount: Integer;
  First, Last, Temp: Integer;
begin
  Result:= Owner.OwningLex.fAny;
  if (Index < fSmallest) or (Index > fLargest) then Exit;
  aKey:= PChar(@Index)^;
  SubCount:= fList[aKey].Count;
  if SubCount = 0 then Exit;
  case SubCount > 1 of
    True:
      begin
        First:= 0;
        Last:= SubCount -1;
        SubList:= fList[aKey].List;
        while First <= Last do
        begin
          Temp:= (First + Last) div 2;
          case Compare(Index, SubList[Temp].WC) of
           1: First:= Temp +1;
           0:
             begin
               Result:= SubList[Temp].Any;
               Exit;
             end;
           -1: Last:= Temp-1;
          end;
        end;
      end;
    False:
      begin
        SubList:= fList[aKey].List;
        Result:= SubList[0].Any;
      end;
  end;
end;

procedure TmwWideSelector.Insert(C: WideChar; Any: TmwLexState; Index: Integer);
var
  aKey: Char;
  Sub: TmwWideSelectorSubListItem;
begin
  if Ord(fSmallest) = 0 then fSmallest:= C;
  fLargest:= C;
  aKey:= TmwWideChar(C)[0];
  Sub:= fList[aKey];
  ReallocMem(Sub.List, (fList[aKey].Count +1) * SizeOf(TmwWideSelectorItem));
  if Index < fList[aKey].Count then
    System.Move(Sub.List[Index], Sub.List[Index + 1],
      (fList[aKey].Count - Index) * SizeOf(TmwWideSelectorItem));
  Sub.List[Index].WC := C;
  Sub.List[Index].Any := Any;
  inc(Sub.Count);
  fList[aKey]:= Sub;
end;

{ TmwLexState }

function TmwLexState.Add(const Value: TmwLexState): Integer;
begin
  Result:= -1;
  if Value <> nil then
  begin
    case Data.Kind in [skGenericLex, skWideLex] of
      True: Value.OwningLex:= TmwGenericLexer(Self);
      False: Value.OwningLex:= OwningLex;
    end;
    case Value.Optional of
      True:
        case Data.Kind in [skGenericLex, skWideLex] of
          True:
            case Value.Kind of
              skAny, skWideAny:
                case TmwGenericLexer(Self).fAny <> nil of
                 True: Result:= Insert(Data.Count, Value);
                  False: TmwGenericLexer(Self).fAny:= Value;
                end;
             skInnerLineEnd, skWideInnerLineEnd:
                case TmwGenericLexer(Self).fInnerLineEnd <> nil of
                 True: Value.Free;
                  False: TmwGenericLexer(Self).fInnerLineEnd:= Value;
                end;
            else Result:= Insert(Data.Count, Value);
            end;
          False: Result:= Insert(Data.Count, Value);
        end;
      False:
        case fFollow <> nil of
          True:
            begin
              Value.Free;
              raise Exception.Create('Follow is already in use');
            end;
          False: fFollow:= Value;
        end;
    end;
  end;
end;

procedure TmwLexState.Clear;
var
  I: Integer;
begin
  if Data.Count > 0 then
  begin
    for I:= 0 to Data.Count -1 do
    if List[I] <> nil then
      List[I].Free;
    if fFollow <> nil then
      fFollow.Free;
    ReallocMem(List, 0);
  end;
end;

destructor TmwLexState.Destroy;
begin
  Clear;
  ReallocMem(Keys.Key, 0);
  inherited Destroy;
end;

function TmwLexState.GetCharSet: TmwCharSet;
begin
  Result:= Keys.CharSet^;
end;

function TmwLexState.GetInSensitive: Boolean;
begin
  Result:= BitZero in Data.Bits;
end;

function TmwLexState.GetMultiline: Boolean;
begin
  Result:= Bitone in Data.Bits;
end;

function TmwLexState.GetNodes(Index: Integer): TmwLexState;
begin
  Result := nil;
  if (Index >= 0) and (Index < Data.Count) then
    Result := List^[Index];
end;

function TmwLexState.GetOptional: Boolean;
begin
  Result:= BitTwo in Data.Bits;
end;

function TmwLexState.GetSkip: Boolean;
begin
  Result:= BitThree in Data.Bits;
end;

function TmwLexState.GetTerminal: Boolean;
begin
  Result:= BitFour in Data.Bits;
end;

function TmwLexState.GetWideCharSet(Index: WideChar): Boolean;
begin
  Result:= TmwWideChar(Index)[1] in Keys.WCharSet[TmwWideChar(Index)[0]];
end;

function TmwLexState.Insert(Index: Integer;
  const Value: TmwLexState): Integer;
begin
  if Index < 0 then
    Index:= 0 else
      if Index > Data.Count then
        Index:= Data.Count;
  ReallocMem(List, (Data.Count +1) * SizeOf(TmwLexState));
  if Index < Data.Count then
    System.Move(List^[Index], List^[Index + 1],
      (Data.Count - Index) * SizeOf(TmwLexState));
  List^[Index] := Value;
  Result:= Index;
  inc(Data.Count);
end;

procedure TmwLexState.Negate;
begin
  case Data.Kind of
    skCharSet: Data.Kind:= skNotCharSet;
    skKey: Data.Kind:= skNotKey;
    skNotCharSet: Data.Kind:= skCharSet;
    skNotKey: Data.Kind:= skKey;
    skTillKey: Data.Kind:= skTillNotKey;
    skTillNotKey: Data.Kind:= skTillKey;

    skWideCharSet: Data.Kind:= skWideNotCharSet;
    skWideKey: Data.Kind:= skWideNotKey;
    skWideNotCharSet: Data.Kind:= skWideCharSet;
    skWideNotKey: Data.Kind:= skWideKey;
    skWideTillKey: Data.Kind:= skWideTillNotKey;
    skWideTillNotKey: Data.Kind:= skWideTillKey;
  end;
end;

procedure TmwLexState.NegateTill;
begin
  case Data.Kind of
    skKey: Data.Kind:= skTillKey;
    skNotKey: Data.Kind:= skTillNotKey;
    skTillKey: Data.Kind:= skKey;
    skTillNotKey: Data.Kind:= skNotKey;

    skWideKey: Data.Kind:= skWideTillKey;
    skWideNotKey: Data.Kind:= skWideTillNotKey;
    skWideTillKey: Data.Kind:= skWideKey;
    skWideTillNotKey: Data.Kind:= skWideNotKey;
  end;
end;

procedure TmwLexState.SetCharSet(const Value: TmwCharSet);
begin
  if Data.Len <> 32 then
  begin
    Data.Len:= 32;
    ReAllocMem(Keys.Key, 32);
    Data.Kind:= skCharSet;
  end;
  Keys.CharSet^:= Value;
end;

procedure TmwLexState.SetGenericValues(aOwner: TmwLexState; IsOptional,
  IsInSensitive: Boolean; anId, AnExId: Word);
begin
  Owner:= aOwner;
  Optional:= IsOptional;
  InSensitive:= IsInSensitive;
  Data.Id:= AnId;
  Data.ExId:= AnExId;
  if Owner <> nil then
    aOwner.Add(Self);
end;

procedure TmwLexState.SetInSensitive(const Value: Boolean);
begin
  case Value of
    True:
      begin
        Include(Data.Bits, BitZero);
        case Data.Kind of
          skKey: Data.Kind:= skKeyInSensitive;
          skNotKey: Data.Kind:= skNotKeyInSensitive;
          skTillKey: Data.Kind:= skTillKeyInSensitive;
          skTillNotKey: Data.Kind:= skTillNotKeyInSensitive;
          skTillKeyMultiLine: Data.Kind:= skTillKeyMultiLineInSensitive;
          skTillNotKeyMultiLine: Data.Kind:= skTillNotKeyMultiLineInSensitive;
          skWideKey: Data.Kind:= skWideKeyInSensitive;
          skWideNotKey: Data.Kind:= skWideNotKeyInSensitive;
          skWideTillKey: Data.Kind:= skWideTillKeyInSensitive;
          skWideTillNotKey: Data.Kind:= skWideTillNotKeyInSensitive;
          skWideTillKeyMultiLine: Data.Kind:= skWideTillKeyMultiLineInSensitive;
          skWideTillNotKeyMultiLine: Data.Kind:= skWideTillNotKeyMultiLineInSensitive;
        end;
      end;
    False:
      begin
        Exclude(Data.Bits, BitZero);
        case Data.Kind of
          skKeyInSensitive: Data.Kind:= skKey;
          skNotKeyInSensitive: Data.Kind:= skNotKey;
          skTillKeyInSensitive: Data.Kind:= skTillKey;
          skTillNotKeyInSensitive: Data.Kind:= skTillNotKey;
          skTillKeyMultiLineInSensitive: Data.Kind:= skTillKeyMultiLine;
          skTillNotKeyMultiLineInSensitive: Data.Kind:= skTillNotKeyMultiLine;
          skWideKeyInSensitive: Data.Kind:= skWideKey;
          skWideNotKeyInSensitive: Data.Kind:= skWideNotKey;
          skWideTillKeyInSensitive: Data.Kind:= skWideTillKey;
          skWideTillNotKeyInSensitive: Data.Kind:= skWideTillNotKey;
          skWideTillKeyMultiLineInSensitive: Data.Kind:= skWideTillKeyMultiLine;
          skWideTillNotKeyMultiLineInSensitive: Data.Kind:= skWideTillNotKeyMultiLine;
        end;
      end;
  end;
end;

procedure TmwLexState.SetKeyString(const Value: String);
begin
  Data.Len:= Length(Value);
  ReAllocMem(Keys.Key, Len);
  Move(Value[1], Keys.Key^, Len);
  Data.Kind:= skKey;
end;

procedure TmwLexState.SetMultiline(const Value: Boolean);
begin
  case Value of
    True:
      begin
        Include(Data.Bits, BitOne);
        case Data.Kind of
          skTillKey: Data.Kind:= skTillKeyMultiline;
          skTillNotKey: Data.Kind:= skTillNotKeyMultiline;
          skWideTillKey: Data.Kind:= skWideTillKeyMultiline;
          skWideTillNotKey: Data.Kind:= skWideTillNotKeyMultiline;
          skWideTillKeyInSensitive: Data.Kind:= skWideTillKeyMultilineInSensitive;
          skWideTillNotKeyInSensitive: Data.Kind:= skWideTillNotKeyMultilineInSensitive;
        end;
      end;
    False:
      begin
        Exclude(Data.Bits, BitOne);
        case Data.Kind of
          skTillKeyMultiline: Data.Kind:= skTillKey;
          skTillNotKeyMultiline: Data.Kind:= skTillNotKey;
          skWideTillKeyMultiline: Data.Kind:= skWideTillKey;
          skWideTillNotKeyMultiline: Data.Kind:= skWideTillNotKey;
          skWideTillKeyMultilineInSensitive: Data.Kind:= skWideTillKeyInSensitive;
          skWideTillNotKeyMultilineInSensitive: Data.Kind:= skWideTillNotKeyInSensitive;
        end;
      end;
  end;
end;

procedure TmwLexState.SetOptional(const Value: Boolean);
begin
  case Value of
    True: Include(Data.Bits, BitTwo);
    False: Exclude(Data.Bits, BitTwo);
  end;
end;

procedure TmwLexState.SetSkip(const Value: Boolean);
begin
  case Value of
    True: Include(Data.Bits, BitThree);
    False: Exclude(Data.Bits, BitThree);
  end;
end;

procedure TmwLexState.SetTerminal(const Value: Boolean);
begin
  case Value of
    True: Include(Data.Bits, BitFour);
    False: Exclude(Data.Bits, BitFour);
  end;
end;

procedure TmwLexState.SetWideCharSet(Index: WideChar;
  const Value: Boolean);
begin
  if Data.Len <> 8192 then
  begin
    Data.Len:= 8192;
    ReAllocMem(Keys.Key, 8192);
    FillChar(Keys.Key^, 8192, 0);
    Data.Kind:= skWideCharSet;
  end;
  case Value of
    True: Include(Keys.WCharSet[TmwWideChar(Index)[0]], TmwWideChar(Index)[1]);
    False: Exclude(Keys.WCharSet[TmwWideChar(Index)[0]], TmwWideChar(Index)[1]);
  end;
end;

procedure TmwLexState.SetWideKeyString(const Value: WideString);
begin
  Data.Len:= Length(Value);
  ReAllocMem(Keys.Key, Len *2);
  Move(Value[1], Keys.Key^, Len *2);
  Data.Kind:= skWideKey;
end;

procedure TmwLexState.SetWideSet(WS: WideString);
var
  I: Integer;
begin
  Data.Len:= 0;
  for I:= 1 to Length(WS) do
  SetWideCharSet(WS[I], True);
  Data.Kind:= skWideCharSet;
end;

function CreateAny(aOwner: TmwLexState; IsOptional: Boolean;
  anId, AnExId: Word): TmwLexState;
begin
  Result:= TmwLexState.Create;
  Result.Data.Kind := skAny;
  Result.SetGenericValues(aOwner, IsOptional, False, anId, AnExId);
end;

function CreateInnerLineEnd(aOwner: TmwLexState;
  IsOptional: Boolean): TmwLexState;
begin
  Result:= TmwLexState.Create;
  Result.Data.Kind := skInnerLineEnd;
  Result.SetGenericValues(aOwner, IsOptional, False, piInnerLineEnd, piInnerLineEnd);
end;

function CreateKey(aOwner: TmwLexState; const S: String; IsOptional: Boolean;
  IsInSensitive: Boolean; anId, AnExId: Word): TmwLexState;
begin
  Result:= TmwLexState.Create;
  Result.SetKeyString(S);
  Result.Data.Kind := skKey;
  Result.SetGenericValues(aOwner, IsOptional, IsInSensitive, anId, AnExId);
end;

function CreateLineEnd(aOwner: TmwLexState;
  IsOptional: Boolean): TmwLexState;
begin
  Result:= TmwLexState.Create;
  Result.Data.Kind := skLineEnd;
  Result.SetGenericValues(aOwner, IsOptional, False, piLineEnd, piLineEnd);
end;

function CreateNotKey(aOwner: TmwLexState; const S: String; IsOptional: Boolean;
  IsInSensitive: Boolean; anId, AnExId: Word): TmwLexState;
begin
  Result:= TmwLexState.Create;
  Result.SetKeyString(S);
  Result.Data.Kind:= skNotKey;
  Result.SetGenericValues(aOwner, IsOptional, IsInSensitive, anId, AnExId);
end;

function CreateNotSet(aOwner: TmwLexState; const aSet: TmwCharSet;
  IsOptional: Boolean; anId, AnExId: Word): TmwLexState;
begin
  Result:= TmwLexState.Create;
  Result.CharSet:= aSet;
  Result.Data.Kind:= skNotCharSet;
  Result.SetGenericValues(aOwner, IsOptional, False, anId, AnExId);
end;

function CreateSet(aOwner: TmwLexState; const aSet: TmwCharSet;
  IsOptional: Boolean; anId, AnExId: Word): TmwLexState;
begin
  Result:= TmwLexState.Create;
  Result.CharSet:= aSet;
  Result.Data.Kind:= skCharSet;
  Result.SetGenericValues(aOwner, IsOptional, False, anId, AnExId);
end;

function CreateTillKey(aOwner: TmwLexState; const S: String; IsOptional: Boolean;
  IsInSensitive: Boolean; anId, AnExId: Word): TmwLexState;
begin
  Result:= TmwLexState.Create;
  Result.SetKeyString(S);
  Result.Data.Kind:= skTillKey;
  Result.SetGenericValues(aOwner, IsOptional, IsInSensitive, anId, AnExId);
end;

function CreateTillNotKey(aOwner: TmwLexState; const S: String;
  IsOptional: Boolean; IsInSensitive: Boolean; anId, AnExId: Word): TmwLexState;
begin
  Result:= TmwLexState.Create;
  Result.SetKeyString(S);
  Result.Data.Kind:= skTillNotKey;
  Result.SetGenericValues(aOwner, IsOptional, IsInSensitive, anId, AnExId);
end;

function CreateWideAny(aOwner: TmwLexState; IsOptional: Boolean;
  anId, AnExId: Word): TmwLexState;
begin
  Result:= TmwLexState.Create;
  Result.Data.Kind := skWideAny;
  Result.SetGenericValues(aOwner, True, False, anId, AnExId);
end;

function CreateWideInnerLineEnd(aOwner: TmwLexState;
  IsOptional: Boolean): TmwLexState;
begin
  Result:= TmwLexState.Create;
  Result.Data.Kind := skWideInnerLineEnd;
  Result.SetGenericValues(aOwner, IsOptional, False, piInnerLineEnd, piInnerLineEnd);
end;

function CreateWideKey(aOwner: TmwLexState; const WS: WideString;
  IsOptional: Boolean; IsInSensitive: Boolean; anId, AnExId: Word): TmwLexState;
begin
  Result:= TmwLexState.Create;
  Result.SetWideKeyString(WS);
  Result.Data.Kind:= skWideKey;
  Result.SetGenericValues(aOwner, IsOptional, IsInSensitive, anId, AnExId);
end;

function CreateWideLineEnd(aOwner: TmwLexState;
  IsOptional: Boolean): TmwLexState;
begin
  Result:= TmwLexState.Create;
  Result.Data.Kind := skWideLineEnd;
  Result.SetGenericValues(aOwner, IsOptional, False, piLineEnd, piLineEnd);
end;

function CreateWideNotKey(aOwner: TmwLexState; const WS: WideString;
  IsOptional: Boolean; IsInSensitive: Boolean; anId, AnExId: Word): TmwLexState;
begin
  Result:= TmwLexState.Create;
  Result.SetWideKeyString(WS);
  Result.Data.Kind:= skWideNotKey;
  Result.SetGenericValues(aOwner, IsOptional, IsInSensitive, anId, AnExId);
end;

function CreateWideNotSet(aOwner: TmwLexState; const WS: WideString;
  IsOptional: Boolean; anId, AnExId: Word): TmwLexState;
begin
  Result:= TmwLexState.Create;
  Result.SetWideSet(WS);
  Result.Data.Kind:= skWideNotCharset;
  Result.SetGenericValues(aOwner, IsOptional, False, anId, AnExId);
end;

function CreateWideSet(aOwner: TmwLexState; const WS: WideString;
  IsOptional: Boolean; anId, AnExId: Word): TmwLexState;
begin
  Result:= TmwLexState.Create;
  Result.SetWideSet(WS);
  Result.Data.Kind:= skWideCharset;
  Result.SetGenericValues(aOwner, IsOptional, False, anId, AnExId);
end;

function CreateWideTillKey(aOwner: TmwLexState; const WS: WideString;
  IsOptional: Boolean; IsInSensitive: Boolean; anId, AnExId: Word): TmwLexState;
begin
  Result:= TmwLexState.Create;
  Result.SetWideKeyString(WS);
  Result.Data.Kind:= skWideTillKey;
  Result.SetGenericValues(aOwner, IsOptional, IsInSensitive, anId, AnExId);
end;

function CreateWideTillNotKey(aOwner: TmwLexState; const WS: WideString;
  IsOptional: Boolean; IsInSensitive: Boolean; anId, AnExId: Word): TmwLexState;
begin
  Result:= TmwLexState.Create;
  Result.SetWideKeyString(WS);
  Result.Data.Kind:= skWideTillNotKey;
  Result.SetGenericValues(aOwner, IsOptional, IsInSensitive, anId, AnExId);
end;

{ TmwIdentifiers }

procedure TmwIdentifiers.AddKey(const aKey: String; IsSensitive: Boolean; anId, AnExId: Word; RestoreIt: TmwLexState; IsTerminal: Boolean);
var
  Item: TmwLexState;
  Sub: TmwKeyWordSub;
  SubKey: Char;
begin
  if aKey = '' then Exit;
  Item:= CreateKey(Self, aKey, True, IsSensitive, anId, AnExId);
  Item.ToReStore:= RestoreIt;
  Item.Terminal:= IsTerminal;
  SubKey:= CompTable[Item.Key^];
  Sub:= KeyList[SubKey];
  ReallocMem(Sub.List, (Sub.Count +1) * SizeOf(TmwLexState));
  Sub.List[Sub.Count]:= Item;
  inc(Sub.Count);
  KeyList[SubKey]:= Sub;
end;

procedure TmwIdentifiers.Clear;
var
  I: Char;
  Sub: TmwKeyWordSub;
begin
 for I:= #0 to #255 do
 if KeyList[I].Count > 0 then
 begin
   Sub:= KeyList[I];
   ReallocMem(Sub.List, 0);
   Sub.Count:= 0;
   KeyList[I]:= Sub;
 end;
 inherited Clear;
end;

function TmwIdentifiers.Compare(aKey: PChar;  aLen: Integer;
  Item: TmwLexState):  Boolean;
var
  I: Integer;
begin
  if aLen <> Item.Len then
  begin
    Result:= False;
    Exit;
  end;
  Result:= True;
  case Item.InSensitive of
    True:
      for I:= aLen -1 downto 1 do
      if CompTable[aKey[I]] <> CompTable[Item.Key[I]] then
      begin
        Result:= False;
        Exit;
      end;
    False:
      for I:= aLen -1 downto 0 do
      if aKey[I] <> Item.Key[I] then
      begin
        Result:= False;
        Exit;
      end;
  end;
end;

constructor TmwIdentifiers.Create(AOwner: TmwLexState; IsOptional: Boolean; aSet, aContent: TmwCharset);
begin
  inherited Create;
  CharSet:= aSet;
  Content:= aContent;
  Data.Kind:= skSpecial;
  SpecialKind:= skIdentifiers;
  SetGenericValues(aOwner, IsOptional, False, piIdent, piIdent);
end;

procedure TmwIdentifiers.Delete(Index: Integer; KeyIndex: Char);
var
  Sub: TmwKeyWordSub;
  SubKey: Char;
begin
  SubKey:= CompTable[KeyIndex];
  Sub:= KeyList[SubKey];
  if Assigned(Sub.List[Index]) then
    Sub.List[Index].Free;
  dec(Sub.Count);
  ReallocMem(Sub.List, Sub.Count);
  if Index < Sub.Count then
  System.Move(Sub.List[Index + 1], Sub.List[Index],
        (Sub.Count - Index) * SizeOf(TmwLexState));
  KeyList[SubKey]:= Sub;
end;

function TmwIdentifiers.GetContent: TmwCharSet;
var
  I: Char;
begin
  for I:= #0 to #255 do
  Case ContentArray[I] of
    True: Include(Result, I);
    False: Exclude(Result, I);
  end;
end;

function TmwIdentifiers.GetKeyWord(P: PChar; aLen: Integer; var AnIndex: Integer): TmwLexState;
var
  I: Integer;
  SubKey: Char;
  Sub: TmwKeyWordSub;
begin
  Result:= Self;
  AnIndex:= -1;
  SubKey:= CompTable[P^];
  Sub:= KeyList[SubKey];
  for I:= 0 to Sub.Count -1  do
  if Compare(P, aLen, Sub.List[I]) then
  begin
    Result:= Sub.List[I];
    AnIndex:= I;
    Exit;
  end;
end;

procedure TmwIdentifiers.Insert(Index: Integer; Item: TmwLexState);
var
  SubKey: Char;
  Sub: TmwKeyWordSub;
begin
  SubKey:= CompTable[Item.Key^];
  Sub:= KeyList[SubKey];
  ReallocMem(Sub.List, (Sub.Count +1) * SizeOf(TmwLexState));
  if Index < Sub.Count then
    System.Move(Sub.List[Index], Sub.List[Index + 1],
      (Sub.Count - Index) * SizeOf(TmwLexState));
  Sub.List[Index] := Item;
  inc(Sub.Count);
  KeyList[SubKey]:= Sub;
end;

function TmwIdentifiers.Match(var P: PChar; TheEnd: PChar): TmwLexState;
var
  aLen: Integer;
  anIndex: Integer;
  Temp: PChar;
begin
  Result:= Self;
  Temp:= P;
  inc(P);
  aLen:= 1;
  while (P < TheEnd) and (ContentArray[P^]) do
  begin
    inc(P);
    inc(aLen);
  end;
  if Temp < P then
    Result:= GetKeyWord(Temp, aLen, anIndex);
end;

procedure TmwIdentifiers.Remove(S: String);
var
  P: PChar;
  AnIndex: Integer;
  Temp: TmwLexState;
begin
  if S = '' then Exit;
  P:= PChar(S);
  Temp:= GetKeyWord(P, Length(S), AnIndex);
  if AnIndex > -1 then
  begin
    Delete(AnIndex, Temp.Key^);
  end;
end;

procedure TmwIdentifiers.SetContent(const Value: TmwCharSet);
var
  I: Char;
begin
  for I:= #0 to #255 do
  Case I in Value of
    True: ContentArray[I]:= True;
    False: ContentArray[I]:= False;
  end;
end;

{ TmwGenericLexer }

function TmwGenericLexer.Add(const Value: TmwLexState): Integer;
begin
  Result:= inherited Add(Value);
  case Value.Kind of
    skCharSet, skNotCharSet, skIdentifiers,
    skKey, skKeyInsensitive, skLineEnd, skSpecial: AddToSelector(Value);
  end;
end;

procedure TmwGenericLexer.AddKeyInsensitive(Item: TmwLexState);
begin
  Keys.Ansi[Item.Key^] := Item;
  if CompTable[Item.Key^] <> Item.Key^ then
    Keys.Ansi[CompTable[Item.Key^]] := Item;
end;

procedure TmwGenericLexer.AddToSelector(Item: TmwLexState);
var
  I: Char;
begin
  case Item.Kind of
    skCharSet:
      for I := #0 to #255 do
      if I in Item.CharSet then
        Keys.Ansi[I] := Item;
    skNotCharSet:
      for I := #0 to #255 do
      if not (I in Item.CharSet) then
        Keys.Ansi[I] := Item;
    skIdentifiers:
      for I := #0 to #255 do
      if I in Item.CharSet then
        Keys.Ansi[I] := Item;
    skKey: Keys.Ansi[Item.Key^] := Item;
    skKeyInSensitive: AddKeyInsensitive(Item);
    skLineEnd:
      begin
        Keys.Ansi[#10] := Item;
        Keys.Ansi[#13] := Item;
      end;
    skSpecial:
      case TmwSpecial(Item).SpecialKind of
        skCharSet:
          for I := #0 to #255 do
          if I in Item.CharSet then
            Keys.Ansi[I] := Item;
        skNotCharSet:
          for I := #0 to #255 do
          if not (I in Item.CharSet) then
            Keys.Ansi[I] := Item;
        skIdentifiers:
          for I := #0 to #255 do
          if I in Item.CharSet then
            Keys.Ansi[I] := Item;
        skKey: Keys.Ansi[Item.Key^] := Item;
        skKeyInSensitive: AddKeyInsensitive(Item);
      end;
  end;
end;

function TmwGenericLexer.ApplyAny: Boolean;
begin
  Result:= IncMBCRun;
end;

function TmwGenericLexer.ApplyCharSet: Boolean;
var
  Temp: PChar;
begin
  Temp:= fRun.Ansi;
  while fRun.Ansi^ in Current.CharSet do
  case IncRun of
    False: Break;
  end;
  Result:= Temp < fRun.Ansi;
end;

function TmwGenericLexer.ApplyKey: Boolean;
var
  I: Integer;
  Last: PChar;
begin
  Result:= False;
  Last:= fRun.Ansi + Current.Data.Len -1;
  case Last >= fTheEnd.Ansi of
    True: Exit;
  end;
  Result:= True;
  for I := Current.Data.Len -1 downto 0 do
  case Current.Keys.Key[I] = Last^ of
    True: dec(Last);
    False:
      begin
        Result:= False;
        break;
      end;
  end;
  case Result of
    True: inc(fRun.Ansi, Current.Data.Len);
  end;
end;

function TmwGenericLexer.ApplyKeyInSensitive: Boolean;
var
  I: Integer;
  Last: PChar;
begin
  Result:= False;
  Last:= fRun.Ansi + Current.Data.Len -1;
  case Last >= fTheEnd.Ansi of
    True: Exit;
  end;
  Result:= True;
  for I := Current.Data.Len -1 downto 0 do
  case CompTable[Current.Keys.Key[I]] = CompTable[Last^] of
    True: dec(Last);
    False:
      begin
        Result:= False;
        break;
      end;
  end;
  case Result of
    True: inc(fRun.Ansi, Current.Data.Len);
  end;
end;

function TmwGenericLexer.ApplyLineEnd: Boolean;
var
  Temp: PChar;
begin
  Temp:= fRun.Ansi;
  if fRun.Ansi < fTheEnd.Ansi then
  case fRun.Ansi^ of
    #13:
      begin
        inc(fRun.Ansi);
        if fRun.Ansi < fTheEnd.Ansi then
          if fRun.Ansi^ = #10 then
            inc(fRun.Ansi);
      end;
    #10: inc(fRun.Ansi);
  end;
  Result:= Temp < fRun.Ansi;
end;

function TmwGenericLexer.ApplyNotCharSet: Boolean;
var
  Temp: PChar;
begin
  Temp:= fRun.Ansi;
  while not (fRun.Ansi^ in Current.Charset) do
  case IncRun of
    False: Break;
  end;
  Result:= Temp < fRun.Ansi;
end;

function TmwGenericLexer.ApplyNotKey: Boolean;
var
  Temp: PChar;
begin
  Temp:= fRun.Ansi;
  Result:= True;
  case ApplyKey of
    True: Result:= False;
    False: Result:= IncRun;
  end;
  case Result of
    False: fRun.Ansi := Temp;
  end;
end;

function TmwGenericLexer.ApplyNotKeyInSensitive: Boolean;
var
  Temp: PChar;
begin
  Temp:= fRun.Ansi;
  Result:= True;
  case ApplyKeyInSensitive of
    True: Result:= False;
    False: Result:= IncRun;
  end;
  case Result of
    False: fRun.Ansi := Temp;
  end;
end;

function TmwGenericLexer.ApplySpecial: Boolean;
var
  Temp: PChar;
begin
  Temp:= fRun.Ansi;
  Current:= TmwSpecial(Current).Match(fRun.Ansi, fTheEnd.Ansi);
  Result:= Temp < fRun.Ansi;
end;

function TmwGenericLexer.ApplyTillKey: Boolean;
var
  TempRun: PChar;
begin
  TempRun := fRun.Ansi;
  Result := False;
  while Result = False do
  begin
    case fRun.Ansi^ of
      #10, #13: Exit;
    else
      begin
        case Current.Keys.Key^ = fRun.Ansi^ of
          True:
            case ApplyKey of
              True: Result:= True;
              False:
                case IncMBCRun of
                  False:
                    begin
                      fRun.Ansi:= TempRun;
                      Exit;
                    end;
                end;
            end;
          False:
            case IncMBCRun of
              False:
                begin
                  fRun.Ansi:= TempRun;
                  Exit;
                end;
            end;
        end;
      end;
    end;
  end;
end;

function TmwGenericLexer.ApplyTillKeyInSensitive: Boolean;
var
  TempRun: PChar;
begin
  TempRun := fRun.Ansi;
  Result := False;
  while Result = False do
  begin
    case fRun.Ansi^ of
      #10, #13: Exit;
    else
      begin
        case CompTable[Current.Keys.Key^] = CompTable[fRun.Ansi^] of
          True:
            case ApplyKey of
              True: Result:= True;
              False:
                case IncMBCRun of
                  False:
                    begin
                      fRun.Ansi:= TempRun;
                      Exit;
                    end;
                end;
            end;
          False:
            case IncMBCRun of
              False:
                begin
                  fRun.Ansi:= TempRun;
                  Exit;
                end;
            end;
        end;
      end;
    end;
  end;
end;

function TmwGenericLexer.ApplyTillKeyMultiLine: Boolean;
begin
  Result := False;
  while Result = False do
  begin
    case fRun.Ansi^ of
      #10, #13:
        begin
          fRange := fInnerLineEnd;
          fInnerLineEnd.ToReStore := Current;
          fProcessing:= False;
          Exit;
        end;
    else
      begin
        case Current.Keys.Key^ = fRun.Ansi^ of
          True:
            case ApplyKey of
              True:
                begin
                  Result:= True;
                  Range:= Current.OwningLex;
                  fProcessing:= False;
                  Exit;
                end;
              False:
                case IncMBCRun of
                  False:
                    begin
                      Range:= Current;
                      fProcessing:= False;
                      Exit;
                    end;
                end;
            end;
          False:
            case IncMBCRun of
              False:
                begin
                  Range:= Current;
                  fProcessing:= False;
                  Exit;
                end;
            end;
        end;
      end;
    end;
  end;
end;

function TmwGenericLexer.ApplyTillKeyMultiLineInSensitive: Boolean;
begin
  Result := False;
  while Result = False do
  begin
    case fRun.Ansi^ of
      #10, #13:
        begin
          fRange := fInnerLineEnd;
          fInnerLineEnd.ToReStore := Current;
          fProcessing:= False;
          Exit;
        end;
    else
      begin
        case CompTable[Current.Keys.Key^] = CompTable[fRun.Ansi^] of
          True:
            case ApplyKey of
              True:
                begin
                  Result:= True;
                  Range:= Current.OwningLex;
                  fProcessing:= False;
                  Exit;
                end;
              False:
                case IncMBCRun of
                  False:
                    begin
                      Range:= Current;
                      fProcessing:= False;
                      Exit;
                    end;
                end;
            end;
          False:
            case IncMBCRun of
              False:
                begin
                  Range:= Current;
                  fProcessing:= False;
                  Exit;
                end;
            end;
        end;
      end;
    end;
  end;
end;

function TmwGenericLexer.ApplyTillNotKey: Boolean;
begin
  Result:= ApplyTillKey;
  case Result of
    True: fRun.Ansi := fRun.Ansi - Current.Len;
  end;
end;

function TmwGenericLexer.ApplyTillNotKeyInSensitive: Boolean;
begin
  Result:= ApplyTillKeyInSensitive;
  case Result of
    True: fRun.Ansi := fRun.Ansi - Current.Len;
  end;
end;

function TmwGenericLexer.ApplyTillNotKeyMultiLine: Boolean;
begin
  Result:= ApplyTillKeyMultiLine;
  case Result of
    True: fRun.Ansi := fRun.Ansi - Current.Len;
  end;
end;

function TmwGenericLexer.ApplyTillNotKeyMultiLineInSensitive: Boolean;
begin
  Result:= ApplyTillKeyMultiLineInSensitive;
  case Result of
    True: fRun.Ansi := fRun.Ansi - Current.Len;
  end;
end;

function TmwGenericLexer.ApplyWideAny: Boolean;
begin
  Result:= IncWideRun;
end;

function TmwGenericLexer.ApplyWideCharSet: Boolean;
var
  Temp: PWideChar;
begin
  Temp:= fRun.Wide;
  while Current.WideCharSet[fRun.Wide^]  do
  case IncWideRun of
    False: Break;
  end;
  Result:= Temp < fRun.Wide;
end;

function TmwGenericLexer.ApplyWideKey: Boolean;
var
  I: Integer;
  Last: PWideChar;
begin
  Result:= False;
  Last:= fRun.Wide + Current.Data.Len -1;
  case Last >= fTheEnd.Wide of
    True: Exit;
  end;
  Result:= True;
  for I := Current.Data.Len -1 downto 0 do
  case Current.Keys.WKey[I] = Last^ of
    True: dec(Last);
    False:
      begin
        Result:= False;
        break;
      end;
  end;
  case Result of
    True: inc(fRun.Wide, Current.Data.Len);
  end;
end;

function TmwGenericLexer.ApplyWideKeyInSensitive: Boolean;
var
  I: Integer;
  Last: PWideChar;
begin
  Result:= False;
  Last:= fRun.Wide + Current.Data.Len -1;
  case Last >= fTheEnd.Wide of
    True: Exit;
  end;
  Result:= True;
  for I := Current.Data.Len -1 downto 0 do
  case CompareWideCharI(Current.Keys.WKey[I], Last^) of
    True: dec(Last);
    False:
      begin
        Result:= False;
        break;
      end;
  end;
  case Result of
    True: inc(fRun.Wide, Current.Data.Len);
  end;
end;

function TmwGenericLexer.ApplyWideLineEnd: Boolean;
var
  Temp: PWideChar;
begin
  Temp:= fRun.Wide;
  if fRun.Wide < fTheEnd.Wide then
  case fRun.Wide^ of
    #13:
      begin
        inc(fRun.Wide);
        if fRun.Wide < fTheEnd.Wide then
          if fRun.Wide^ = #10 then
            inc(fRun.Wide);
      end;
    #10: inc(fRun.Wide);
  end;
  Result:= Temp < fRun.Wide;
end;

function TmwGenericLexer.ApplyWideNotCharSet: Boolean;
var
  Temp: PWideChar;
begin
  Temp:= fRun.Wide;
  while not Current.WideCharSet[fRun.Wide^]  do
  case IncWideRun of
    False: Break;
  end;
  Result:= Temp < fRun.Wide;
end;

function TmwGenericLexer.ApplyWideNotKey: Boolean;
var
  Temp: PWideChar;
begin
  Temp:= fRun.Wide;
  Result:= True;
  case ApplyWideKey of
    True: Result:= False;
    False: Result:= IncRun;
  end;
  case Result of
    False: fRun.Wide := Temp;
  end;
end;

function TmwGenericLexer.ApplyWideNotKeyInSensitive: Boolean;
var
  Temp: PWideChar;
begin
  Temp:= fRun.Wide;
  Result:= True;
  case ApplyWideKeyInSensitive of
    True: Result:= False;
    False: Result:= IncRun;
  end;
  case Result of
    False: fRun.Wide := Temp;
  end;
end;

function TmwGenericLexer.ApplyWideTillKey: Boolean;
var
  TempRun: PWideChar;
begin
  TempRun := fRun.Wide;
  Result := False;
  while Result = False do
  begin
    case fRun.Wide^ of
      #10, #13: Exit;
    else
      begin
        case Current.Keys.WKey^ = fRun.Wide^ of
          True:
            case ApplyWideKey of
              True: Result:= True;
              False:
                case IncWideRun of
                  False:
                    begin
                      fRun.Wide:= TempRun;
                      Exit;
                    end;
                end;
            end;
          False:
            case IncWideRun of
              False:
                begin
                  fRun.Wide:= TempRun;
                  Exit;
                end;
            end;
        end;
      end;
    end;
  end;
end;

function TmwGenericLexer.ApplyWideTillKeyInSensitive: Boolean;
var
  TempRun: PWideChar;
begin
  TempRun := fRun.Wide;
  Result := False;
  while Result = False do
  begin
    case fRun.Wide^ of
      #10, #13: Exit;
    else
      begin
        case CompareWideCharI(Current.Keys.WKey^, fRun.Wide^) of
          True:
            case ApplyWideKey of
              True: Result:= True;
              False:
                case IncWideRun of
                  False:
                    begin
                      fRun.Wide:= TempRun;
                      Exit;
                    end;
                end;
            end;
          False:
            case IncWideRun of
              False:
                begin
                  fRun.Wide:= TempRun;
                  Exit;
                end;
            end;
        end;
      end;
    end;
  end;
end;

function TmwGenericLexer.ApplyWideTillKeyMultiLine: Boolean;
begin
  Result := False;
  while Result = False do
  begin
    case fRun.Wide^ of
      #10, #13:
        begin
          fRange := fInnerLineEnd;
          fInnerLineEnd.ToReStore := Current;
          fProcessing:= False;
          Exit;
        end;
    else
      begin
        case Current.Keys.WKey^ = fRun.Wide^ of
          True:
            case ApplyKey of
              True:
                begin
                  Result:= True;
                  Range:= Current.OwningLex;
                  fProcessing:= False;
                  Exit;
                end;
              False:
                case IncWideRun of
                  False:
                    begin
                      Range:= Current;
                      fProcessing:= False;
                      Exit;
                    end;
                end;
            end;
          False:
            case IncWideRun of
              False:
                begin
                  Range:= Current;
                  fProcessing:= False;
                  Exit;
                end;
            end;
        end;
      end;
    end;
  end;
end;

function TmwGenericLexer.ApplyWideTillKeyMultiLineInSensitive: Boolean;
begin
  Result := False;
  while Result = False do
  begin
    case fRun.Wide^ of
      #10, #13:
        begin
          fRange := fInnerLineEnd;
          fInnerLineEnd.ToReStore := Current;
          fProcessing:= False;
          Exit;
        end;
    else
      begin
        case CompareWideCharI(Current.Keys.WKey^, fRun.Wide^) of
          True:
            case ApplyKey of
              True:
                begin
                  Result:= True;
                  Range:= Current.OwningLex;
                  fProcessing:= False;
                  Exit;
                end;
              False:
                case IncWideRun of
                  False:
                    begin
                      Range:= Current;
                      fProcessing:= False;
                      Exit;
                    end;
                end;
            end;
          False:
            case IncWideRun of
              False:
                begin
                  Range:= Current;
                  fProcessing:= False;
                  Exit;
                end;
            end;
        end;
      end;
    end;
  end;
end;

function TmwGenericLexer.ApplyWideTillNotKey: Boolean;
begin
  Result:= ApplyWideTillKey;
  case Result of
    True: fRun.Ansi := fRun.Ansi - Current.Len *2;
  end;
end;

function TmwGenericLexer.ApplyWideTillNotKeyInSensitive: Boolean;
begin
  Result:= ApplyWideTillKeyInSensitive;
  case Result of
    True: fRun.Ansi := fRun.Ansi - Current.Len *2;
  end;
end;

function TmwGenericLexer.ApplyWideTillNotKeyMultiLine: Boolean;
begin
  Result:= ApplyWideTillKeyMultiLine;
  case Result of
    True: fRun.Ansi := fRun.Ansi - Current.Len *2;
  end;
end;

function TmwGenericLexer.ApplyWideTillNotKeyMultiLineInSensitive: Boolean;
begin
  Result:= ApplyWideTillKeyMultiLineInSensitive;
  case Result of
    True: fRun.Ansi := fRun.Ansi - Current.Len *2;
  end;
end;

function TmwGenericLexer.GetAtEnd: Boolean;
begin
  Result:= fRun.Ansi >= fTheEnd.Ansi;
end;

procedure TmwGenericLexer.ClearSelector;
var
  I: Char;
begin
  for I:= #0 to #255 do
    Keys.Ansi[I]:= fAny;
end;

constructor TmwGenericLexer.Create(aOwner: TmwLexState);
begin
  inherited Create;
  FRange:= Self;
  OwningLex:= Self;
  InitSelector;
  SetGenericValues(aOwner, True, False, piUnknown, piUnknown);
  CreateHelpers;
end;

procedure TmwGenericLexer.CreateHelpers;
begin
  Data.Kind:= skGenericLex;
  New(Keys.Ansi);
  fAny:= CreateAny(Self, True, piSymbol, piSymbol);
  ClearSelector;
  fInnerLineEnd:= CreateInnerLineEnd(Self, True);
end;

function TmwGenericLexer.GetCurrent(P: Pointer): TmwLexState;
begin
  Result:= Keys.Ansi[PChar(P)^];
end;

function TmwGenericLexer.GetToken: string;
begin
  SetLength(Result, fRun.Ansi - fStart.Ansi);
  Move(fStart.Ansi^, PChar(Result)^, fRun.Ansi - fStart.Ansi);
end;

function TmwGenericLexer.IncMBCRun: Boolean;
begin
  Result:= False;
  if fRun.Ansi < fTheEnd.Ansi then
  case MBCS of
    True:
      case LeadTable[fRun.Ansi^] of
        True:
          begin
            Result:= True;
            if fRun.Ansi +1 < fTheEnd.Ansi then
              inc(fRun.Ansi, StrCharLength(fRun.Ansi))else
                inc(fRun.Ansi);
          end;
        False:
          begin
            Result:= True;
            inc(fRun.Ansi);
          end;
      end;
    False:
      begin
        Result:= True;
        inc(fRun.Ansi);
      end;
  end;
end;

function TmwGenericLexer.IncRun: Boolean;
begin
  Result:= False;
  if fRun.Ansi < fTheEnd.Ansi then
  begin
    Result:= True;
    inc(fRun.Ansi);
  end;
end;

function TmwGenericLexer.IncWideRun: Boolean;
begin
  Result:= False;
  if fRun.Wide < fTheEnd.Wide then
  begin
    Result:= True;
    inc(fRun.Wide);
  end;
end;

procedure TmwGenericLexer.InitSelector;
begin
  Selector[skAny]:= ApplyAny;
  Selector[skCharSet]:= ApplyCharSet;
  Selector[skInnerLineEnd]:= ApplyLineEnd;
  Selector[skIdentifiers]:= ApplySpecial;
  Selector[skKey]:= ApplyKey;
  Selector[skLineEnd]:= ApplyLineEnd;
  Selector[skNotCharSet]:= ApplyNotCharSet;
  Selector[skNotKey]:= ApplyNotKey;
  Selector[skTillKey]:= ApplyTillKey;
  Selector[skTillNotKey]:= ApplyTillNotKey;
  Selector[skTillKeyMultiLine]:= ApplyTillKeyMultiLine;
  Selector[skTillNotKeyMultiLine]:= ApplyTillNotKeyMultiLine;

  Selector[skKeyInSensitive]:= ApplyKeyInSensitive;
  Selector[skNotKeyInSensitive]:= ApplyNotKeyInSensitive;
  Selector[skTillKeyInSensitive]:= ApplyTillKeyInSensitive;
  Selector[skTillNotKeyInSensitive]:= ApplyTillNotKeyInSensitive;
  Selector[skTillKeyMultiLineInSensitive]:= ApplyTillKeyMultiLineInSensitive;
  Selector[skTillNotKeyMultiLineInSensitive]:= ApplyTillNotKeyMultiLineInSensitive;

  Selector[skWideAny]:= ApplyWideAny;
  Selector[skWideCharSet]:= ApplyWideCharSet;
  Selector[skWideInnerLineEnd]:= ApplyWideLineEnd;
  Selector[skWideIdentifiers]:= ApplySpecial;
  Selector[skWideKey]:= ApplyWideKey;
  Selector[skWideLineEnd]:= ApplyWideLineEnd;
  Selector[skWideNotCharSet]:= ApplyWideNotCharSet;
  Selector[skWideNotKey]:= ApplyWideNotKey;
  Selector[skWideTillKey]:= ApplyWideTillKey;
  Selector[skWideTillNotKey]:= ApplyWideTillNotKey;
  Selector[skWideTillKeyMultiLine]:= ApplyWideTillKeyMultiLine;
  Selector[skWideTillNotKeyMultiLine]:= ApplyWideTillNotKeyMultiLine;

  Selector[skWideKeyInSensitive]:= ApplyWideKeyInSensitive;
  Selector[skWideNotKeyInSensitive]:= ApplyWideNotKeyInSensitive;
  Selector[skWideTillKeyInSensitive]:= ApplyWideTillKeyInSensitive;
  Selector[skWideTillNotKeyInSensitive]:= ApplyWideTillNotKeyInSensitive;
  Selector[skWideTillKeyMultiLineInSensitive]:= ApplyWideTillKeyMultiLineInSensitive;
  Selector[skWideTillNotKeyMultiLineInSensitive]:= ApplyWideTillNotKeyMultiLineInSensitive;

  Selector[skSpecial]:= ApplySpecial;
end;

function TmwGenericLexer.Match: Boolean;
begin
  Result:= MinToMax;
  case Result  of
    True:
      if Current <> nil then
      begin
        Data.Id := Current.Data.Id;
        Data.ExId := Current.Data.ExId;
        if Current.ToRegress <> nil then
        begin
          Current:= Current.ToRegress;
          Result:= Match;
        end;
      end;
    False: Result:= MatchOptions
  end;
end;

function TmwGenericLexer.MatchOptions: Boolean;
var
  I: Integer;
  Temp: TmwLexState;
begin
  Result:= False;
  for I := 0 to Current.Data.Count - 1 do
  begin
    Temp:= Current;
    Current := Current[I];
    Result:= Match;
    case Result of
      True: Exit;
      False: Current:= Temp;
    end;
  end;
end;

function TmwGenericLexer.MinToMax: Boolean;
var
  I: Integer;
  Temp, Temp2: PChar;
begin
  Temp := fRun.Ansi;
  Result := True;
  if (Current.Max = 0) and (Current.Min = 0) then Selector[Current.Kind] else
  begin
    for I := 0 to Current.Min - 1 do
    begin
      case Selector[Current.Kind] of
        False:
          begin
            Result:= False;
            Break;
          end;
      end;
    end;
    if Result then
    for I := Current.Min to Current.Max - 1 do
    begin
      Temp2 := fRun.Ansi;
      case Selector[Current.Kind] of
        False:
          begin
            fRun.Ansi := Temp2;
            Break;
          end;
      end;
    end;
  end;
  case Result of
    False: fRun.Ansi := Temp;
  end;
end;

function TmwGenericLexer.Next: Boolean;
begin
  Result:= True;
  Id:= piUnKnown;
  ExId:= piUnKnown;
  fStart.Ansi := fRun.Ansi;
  case (fRange = Self) or (fRange = nil) of
    True: Current:= GetCurrent(fStart.Ansi);
    False:
      begin
        Current := Range;
        if Current.Data.Kind in [skGenericLex, skWideLex] then
          Current:= TmwGenericLexer(Current).GetCurrent(fStart.Ansi);
      end;
  end;
  fProcessing:= True;
  while Processing do
  begin
    Result := Match;
    case Result of
      True:
        begin
          case Assigned(fOnDoSomething) of
            True: fOnDoSomething(Self);
          end;
          case Current.Terminal of
            True: fRange:= Current.OwningLex.OwningLex;
          end;
          if Current.fToReStore <> nil then
          begin
            fRange:= Current.fToReStore;
            Exit;
          end;
          case Current.fFollow <> nil of
            True:
              begin
                Current:= Current.fFollow;
                if Current.Kind in [skGenericLex, skWideLex] then
                begin
                  fRange:= Current;
                  Exit;
                end;
              end;
            False: fProcessing:= False;
          end;
        end;
      False: fProcessing:= False;
    end;
  end;
end;

procedure TmwGenericLexer.ReleaseHelpers;
begin
  Dispose(Keys.Ansi);
  fInnerLineEnd.Free;
  fAny.Free;
end;

procedure TmwGenericLexer.SetInitMethod(const Value: TmwLexInitMethod);
begin
  if Assigned(Value) then
  begin
    ClearSelector;
    Clear;
    FInitMethod := Value;
    Value(Self);
  end;
end;

procedure TmwGenericLexer.SetInitProc(const Value: TmwLexInitProc);
begin
  if Assigned(Value) then
  begin
    ClearSelector;
    Clear;
    FInitProc := Value;
    Value(Self);
  end;
end;

procedure TmwGenericLexer.SetOrigin(const Value: PChar);
begin
  FOrigin.Ansi := Value;
  fRun.Ansi:= Value;
  fStart.Ansi:= Value;
end;

procedure TmwGenericLexer.SetStartData(Ptr: Pointer; Len: Integer);
begin
  Origin := Ptr;
  TheEnd:= PChar(Ptr) + Len;
end;

procedure TmwGenericLexer.SetTheEnd(const Value: PChar);
begin
  fTheEnd.Ansi := Value;
end;

{ TmwWideIdentifiers }

procedure TmwWideIdentifiers.AddKey(const aKey: WideString; IsSensitive: Boolean; anId, AnExId: Word; RestoreIt: TmwLexState; IsTerminal: Boolean);
var
  Item: TmwLexState;
  Sub: TmwKeyWordSub;
  SubKey: Char;
begin
  if aKey = '' then Exit;
  Item:= CreateKey(Self, aKey, True, IsSensitive, anId, AnExId);
  Item.ToReStore:= RestoreIt;
  Item.Terminal:= IsTerminal;
  SubKey:= TmwWideChar(PairList[Item.WideKey^])[0];
  Sub:= KeyList[SubKey];
  ReallocMem(Sub.List, (Sub.Count +1) * SizeOf(TmwLexState));
  Sub.List[Sub.Count]:= Item;
  inc(Sub.Count);
  KeyList[SubKey]:= Sub;
end;

procedure TmwWideIdentifiers.Clear;
var
  I: Char;
  Sub: TmwKeyWordSub;
begin
 for I:= #0 to #255 do
 begin
   Sub:= KeyList[I];
   ReallocMem(Sub.List, 0);
   Sub.Count:= 0;
   KeyList[I]:= Sub;
 end;
 inherited Clear;
end;

function TmwWideIdentifiers.Compare(aKey: PWideChar; aLen: Integer;
  Item: TmwLexState): Boolean;
var
  I: Integer;
begin
  if aLen <> Item.Len then
  begin
    Result:= False;
    Exit;
  end;
  Result:= True;
  case Item.InSensitive of
    True:
      for I:= aLen -1 downto 1 do
      if CompareWideCharI(aKey[I], Item.WideKey[I]) = False then
      begin
        Result:= False;
        Exit;
      end;
    False:
      for I:= aLen -1 downto 0 do
      if aKey[I] <> Item.WideKey[I] then
      begin
        Result:= False;
        Exit;
      end;
  end;
end;

constructor TmwWideIdentifiers.Create(AParent: TmwLexState; aSet, aContent: WideString);
begin
  SetWideSet(aSet);
  SetWideContent(aContent);
  Data.Kind:= skSpecial;
  SpecialKind:= skWideIdentifiers;
  SetGenericValues(aParent, True, False, piIdent, piIdent);
end;

procedure TmwWideIdentifiers.Delete(Index: Integer; KeyIndex: Char);
var
  Sub: TmwKeyWordSub;
  SubKey: Char;
begin
  SubKey:= CompTable[KeyIndex];
  Sub:= KeyList[SubKey];
  if Assigned(Sub.List[Index]) then
    Sub.List[Index].Free;
  dec(Sub.Count);
  ReallocMem(Sub.List, Sub.Count);
  if Index < Sub.Count then
  System.Move(Sub.List[Index + 1], Sub.List[Index],
        (Sub.Count - Index) * SizeOf(TmwLexState));
  KeyList[SubKey]:= Sub;
end;

function TmwWideIdentifiers.GetKeyWord(P: PWideChar; aLen: Integer; var AnIndex: Integer): TmwLexState;
var
  I: Integer;
  SubKey: Char;
  Sub: TmwKeyWordSub;
begin
  Result:= Self;
  AnIndex:= -1;
  SubKey:= TmwWideChar(PairList[P^])[0];
  Sub:= KeyList[SubKey];
  for I:= 0 to Sub.Count -1  do
  if Compare(P, aLen, Sub.List[I]) then
  begin
    Result:= Sub.List[I];
    AnIndex:= I;
    Exit;
  end;
end;

function TmwWideIdentifiers.GetWideCharContent(Index: WideChar): Boolean;
begin
  Result:= TmwWideChar(Index)[1] in fWideContent[TmwWideChar(Index)[0]];
end;

procedure TmwWideIdentifiers.Insert(Index: Integer; Item: TmwLexState);
var
  SubKey: Char;
  Sub: TmwKeyWordSub;
begin
  SubKey:= TmwWideChar(PairList[Item.WideKey^])[0];
  Sub:= KeyList[SubKey];
  ReallocMem(Sub.List, (Sub.Count +1) * SizeOf(TmwLexState));
  if Index < Sub.Count then
    System.Move(Sub.List[Index], Sub.List[Index + 1],
      (Sub.Count - Index) * SizeOf(TmwLexState));
  inc(Sub.Count);
  Sub.List[Index] := Item;
  KeyList[SubKey]:= Sub;
end;

function TmwWideIdentifiers.Match(var P: PWideChar; TheEnd: PWideChar): TmwLexState;
var
  aLen: Integer;
  anIndex: Integer;
  Temp: PWideChar;
begin
  Result:= Self;
  Temp:= P;
  inc(P);
  aLen:= 1;
  while (P < TheEnd) and (WideCharContent[P^]) do
  begin
    inc(P);
    inc(aLen);
  end;
  if Temp < P then
    Result:= GetKeyWord(Temp, aLen, anIndex);
end;

procedure TmwWideIdentifiers.Remove(S: WideString);
var
  P: PWideChar;
  aLen, AnIndex: Integer;
  Temp: TmwLexState;
begin
  aLen:= Length(S);
  if aLen = 0 then Exit;
  P:= PWideChar(S);
  Temp:= GetKeyWord(P, aLen, AnIndex);
  Delete(AnIndex, TmwWideChar(PairList[Temp.WideKey^])[0]);
end;

procedure TmwWideIdentifiers.SetWideCharContent(Index: WideChar;
  const Value: Boolean);
begin
  case Value of
    True: Include(fWideContent[TmwWideChar(Index)[0]], TmwWideChar(Index)[1]);
    False: Exclude(fWideContent[TmwWideChar(Index)[0]], TmwWideChar(Index)[1]);
  end;
end;

procedure TmwWideIdentifiers.SetWideContent(WS: WideString);
var
  I: Integer;
begin
  FillChar(fWideContent, 8192, 0);
  for I:= 1 to Length(WS) do
  SetWideCharContent(WS[I], True);
end;

{ TmwWideLex }

function TmwWideLex.Add(const Value: TmwLexState): Integer;
begin
  Result:= inherited Add(Value);
  case Value.Kind of
    skWideCharSet, skWideNotCharSet, skWideIdentifiers,
    skWideKey, skWideKeyInsensitive: AddToSelector(Value);
  end;
end;

procedure TmwWideLex.AddKeyInsensitive(Item: TmwLexState);
var
  Pair: TmwWidePair;
begin
  Pair:= PairList.GetPair(Item.Keys.WKey^);
  case Pair[0] <> Pair[1] of
    True:
      begin
         Keys.Wide.Add(Pair[0], Item);
          Keys.Wide.Add(Pair[1], Item);
      end;
    False: Keys.Wide.Add(Pair[0], Item);
  end;
end;

procedure TmwWideLex.AddToSelector(Item: TmwLexState);
var
  I: WideChar;
begin
  case Item.Kind of
    skWideCharSet:
      for I := #0 to #65535 do
      if Item.WideCharSet[I] then
        Keys.Wide.Add(I, Item);
    skWideNotCharSet:
      for I := #0 to #65535 do
      if not Item.WideCharSet[I] then
        Keys.Wide.Add(I, Item);
    skWideIdentifiers:
      for I := #0 to #65535 do
      if Item.WideCharSet[I] then
        Keys.Wide.Add(I, Item);
    skWideKey: Keys.Wide.Add(Item.WideKey^, Item);
    skWideKeyInsensitive: AddKeyInsensitive(Item);
    skWideLineEnd:
      begin
        Keys.Wide.Add(#10, Item);
        Keys.Wide.Add(#13, Item);
      end;
    skSpecial:
      case TmwSpecial(Item).SpecialKind of
        skWideCharSet:
          for I := #0 to #65535 do
          if Item.WideCharSet[I] then
            Keys.Wide.Add(I, Item);
        skWideNotCharSet:
          for I := #0 to #65535 do
          if not Item.WideCharSet[I] then
            Keys.Wide.Add(I, Item);
        skWideIdentifiers:
          for I := #0 to #65535 do
          if Item.WideCharSet[I] then
            Keys.Wide.Add(I, Item);
        skWideKey: Keys.Wide.Add(Item.WideKey^, Item);
        skWideKeyInsensitive: AddKeyInsensitive(Item);
      end;
  end;
end;

function TmwWideLex.ApplySpecial: Boolean;
var
  Temp: PWideChar;
begin
  Temp:= fRun.Wide;
  Current:= TmwSpecial(Current).Match(fRun.Ansi, fTheEnd.Ansi);
  Result:= Temp < fRun.Wide;
end;

procedure TmwWideLex.ClearSelector;
begin
  Keys.Wide.Clear;
end;

procedure TmwWideLex.CreateHelpers;
begin
  Data.Kind:= skWideLex;
  Keys.Wide:= TmwWideSelector.Create(Self);
  fAny:= CreateWideAny(Self, True, piSymbol, piSymbol);
  ClearSelector;
  fInnerLineEnd:= CreateWideInnerLineEnd(Self, True);
end;

function TmwWideLex.GetCurrent(P: Pointer): TmwLexState;
begin
  Result:= Keys.Wide[PWideChar(P)^];
end;

function TmwWideLex.GetWideToken: WideString;
begin
  SetLength(Result, (fRun.Ansi - fStart.Ansi) div 2);
  Move(fStart.Wide, PWideChar(Result)^, fRun.Ansi - fStart.Ansi);
end;

procedure TmwWideLex.ReleaseHelpers;
begin
  Keys.Wide.Free;
  fInnerLineEnd.Free;
  fAny.Free;
end;

initialization
  PairList:= TmwWidePairList.Create;
  InitTables;

finalization
  PairList.Free;
end.
