{------------------------------------------------------------------------------}
{                                                                              }
{  Office Assistant                                                            }
{  by Kambiz R. Khojasteh                                                      }
{                                                                              }
{  kambiz@delphiarea.com                                                       }
{  http://www.delphiarea.com                                                   }
{                                                                              }
{------------------------------------------------------------------------------}

{$I DELPHIAREA.INC}

unit oaAssist;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, oaDirect, oaStage, oaBaloon, oaTypes;

type

  {$IFNDEF DELPHI4_UP}
  TBiDiMode = (bdLeftToRight, bdRightToLeft, bdRightToLeftNoAlign,
    bdRightToLeftReadingOnly);
  {$ENDIF}

  TAssistant = class(TComponent)
  private
    Stage: TAssistantStage;
    Director: TAssistantDirector;
    DefaultDialog: TAssistantBalloon;
    Dialog: TAssistantBalloon;
    fAbout: TAboutMe;
    fCustomDialog: TAssistantBalloon;
    fDockSite: TWinControl;
    fHintBulb: Boolean;
    fFastShowHide: Boolean;
    fVisible: Boolean;
    fMoveStep: Byte;
    fDragging: Boolean;
    MovingTo: Boolean;
    MoveDest: TPoint;
    ShowingPhase: Boolean;
    HiddingPhase: Boolean;
    fOnPlayStart: TPlayEvent;
    fOnPlayStop: TPlayEvent;
    fOnDragStart: TNotifyEvent;
    fOnDragging: TNotifyEvent;
    fOnDragStop: TNotifyEvent;
    fOnIdle: TIdleEvent;
    fOnClick: TNotifyEvent;
    fOnDblClick: TNotifyEvent;
    fOnChange: TNotifyEvent;
    fOnShow: TNotifyEvent;
    fOnHide: TNotifyEvent;
    fOnResize: TNotifyEvent;
    fOnBalloonShow: TNotifyEvent;
    fOnBalloonHide: TNotifyEvent;
    fOnAdjustCustomLabel: TCustomLabelEvent;
    fOnTipsTipChange: TNotifyEvent;
    fOnGuideRequestChange: TNotifyEvent;
    fOnGuideTopicClick: TTopicClickEvent;
    fOnGuideSearchClick: TNotifyEvent;
    fOnGuideOptionsClick: TNotifyEvent;
    fOnHintTopicClick: TTopicClickEvent;
    fOnHintBulbClick: TNotifyEvent;
    procedure SetHintBulb(Value: Boolean);
    procedure SetMute(Value: Boolean);
    function GetMute: Boolean;
    function GetSpeed: Word;
    procedure SetSpeed(Value: Word);
    function GetPriority: TThreadPriority;
    procedure SetPriority(Value: TThreadPriority);
    procedure SetMoveStep(Value: Byte);
    procedure SetVisible(Value: Boolean);
    procedure SetAutoIdle(Value: Boolean);
    function GetAutoIdle: Boolean;
    procedure SetPopupMenu(Value: TPopupMenu);
    function GetPopupMenu: TPopupMenu;
    procedure SetBiDiMode(Value: TBiDiMode);
    function GetBiDiMode: TBiDiMode;
    procedure SetStageLeft(Value: Integer);
    function GetStageLeft: Integer;
    procedure SetStageTop(Value: Integer);
    function GetStageTop: Integer;
    function GetStageWidth: Integer;
    function GetStageHeight: Integer;
    function GetStageBoundsRect: TRect;
    function GetShowing: Boolean;
    procedure SetDraggable(Value: Boolean);
    function GetDraggable: Boolean;
    procedure SetActorLibrary(const Value: String);
    function GetActorLibrary: String;
    function GetActorName: String;
    function GetActorVerInfo(const InfoName: String): String;
    function GetAnimationCount: Word;
    function GetAnimations(Index: Word): TAnimation;
    procedure SetCustomDialog(Value: TAssistantBalloon);
    procedure SetDockSite(Value: TWinControl);
    procedure SetBalloonOptions(Value: TBalloonOptions);
    function GetBalloonOptions: TBalloonOptions;
    procedure SetMsgDlgOptions(Value: TMsgDlgOptions);
    function GetMsgDlgOptions: TMsgDlgOptions;
    procedure SetTipsDlgOptions(Value: TTipsDlgOptions);
    function GetTipsDlgOptions: TTipsDlgOptions;
    procedure SetGuideDlgOptions(Value: TGuideDlgOptions);
    function GetGuideDlgOptions: TGuideDlgOptions;
    procedure SetHintDlgOptions(Value: THintDlgOptions);
    function GetHintDlgOptions: THintDlgOptions;
    procedure SetBalloonAnchor(Value: TWinControl);
    function GetBalloonAnchor: TWinControl;
    function GetEmpty: Boolean;
    function GetPlaying: Boolean;
    function GetPlayingID: TRequestID;
    function GetPlayIdling: Boolean;
    function GetBalloonPos: TBalloonPos;
    function GetModalBalloon: Boolean;
    function GetVisibleBalloon: TBalloonKind;
    procedure FrameChanged(Sender: TObject);
    procedure PlayStarted(Sendr: TObject; RequestID: TRequestID);
    procedure PlayStopped(Sendr: TObject; RequestID: TRequestID);
    procedure IdleEntered(Sendr: TObject; Level: Integer; var Handled: Boolean);
    procedure StageMoving(Sender: TObject);
    procedure StageResize(Sender: TObject);
    procedure StageShow(Sender: TObject);
    procedure StageHide(Sender: TObject);
    procedure StageClick(Sender: TObject);
    procedure StageDblClick(Sender: TObject);
    procedure StageMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure StageDeactivate(Sender: TObject);
    procedure HintBulbClick(Sender: TObject);
    procedure BalloonShow(Sender: TObject);
    procedure BalloonHide(Sender: TObject);
    procedure AdjustCustomLabel(Sender: TObject; TheLabel: TControl);
    procedure TipsTipChanged(Sender: TObject);
    procedure GuideRequestChanged(Sender: TObject);
    procedure GuideGetBtnStates(Sender: TObject; var OptionsBtn, SearchBtn: Boolean);
    procedure GuideTopicClicked(Sender: TObject; TopicIndex: Integer);
    procedure GuideSearchClicked(Sender: TObject);
    procedure GuideOptionsClicked(Sender: TObject);
    procedure HintTopicClicked(Sender: TObject; TopicIndex: Integer);
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function MoveOneStep(SX, SY: Integer): Boolean;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Show(Fast: Boolean {$IFDEF DELPHI4_UP} = False {$ENDIF});
    procedure Hide(Fast: Boolean {$IFDEF DELPHI4_UP} = False {$ENDIF});
    procedure SetPosition(X, Y: Integer);
    {$IFDEF DELPHI4_UP}
    function Play(Index: Word): TRequestID; overload;
    function Play(const Name: String): TRequestID; overload;
    function Play(Action: TActorAction): TRequestID; overload;
    function Play(const Actions: array of TActorAction): TRequestID; overload;
    {$ELSE}
    function PlayIndex(Index: Word): TRequestID;
    function PlayName(Name: String): TRequestID;
    function PlayAction(Action: TActorAction): TRequestID;
    function PlayRandom(const Actions: array of TActorAction): TRequestID;
    function Play(V: Variant): TRequestID;
    {$ENDIF}
    function WaitFor(RequestID: TRequestID {$IFDEF DELPHI4_UP} = 0 {$ENDIF}): Boolean;
    procedure Stop(RequestID: TRequestID {$IFDEF DELPHI4_UP} = 0 {$ENDIF});
    procedure StopAll;
    function IsValid(RequestID: TRequestID): Boolean;
    function IsPlaying(Actions: TActorActions): Boolean;
    function GestureAt(X, Y: Integer): TRequestID;
    function GlanceAt(X, Y: Integer): TRequestID;
    function LookAt(X, Y: Integer): TRequestID;
    procedure MoveTo(X, Y: Integer);
    procedure CloseBalloon;
    function InputQuery(const ACaption, APrompt: String; var Value: String): Boolean;
    function MessageDlg(const Msg: String; AType: TMsgDlgType;
      AButtons: TMsgDlgButtons; HelpCtx: LongInt): Word;
    procedure Speak(const Msg: String; Timeout: Integer {$IFDEF DELPHI4_UP} = 0 {$ENDIF});
    procedure Think(const Msg: String; Timeout: Integer {$IFDEF DELPHI4_UP} = 0 {$ENDIF});
    procedure ShowTips;
    procedure ShowHint;
    procedure ShowGuide;
    procedure ShowCustom;
    function ShowGuideModal: Word;
    function ShowHintModal: Word;
    function ShowCustomModal: Word;
    procedure MoveBalloonToCursor;
    procedure MoveBalloonTo(Point: TPoint);
    procedure MoveBalloonToControl(Control: TControl);
    property Empty: Boolean read GetEmpty;
    property Showing: Boolean read GetShowing;
    property Playing: Boolean read GetPlaying;
    property PlayingID: TRequestID read GetPlayingID;
    property PlayIdling: Boolean read GetPlayIdling;
    property Dragging: Boolean read fDragging;
    property ModalBalloon: Boolean read GetModalBalloon;
    property BalloonPos: TBalloonPos read GetBalloonPos;
    property VisibleBalloon: TBalloonKind read GetVisibleBalloon;
    property BalloonAnchor: TWinControl read GetBalloonAnchor write SetBalloonAnchor;
    property CustomDialog: TAssistantBalloon read fCustomDialog write SetCustomDialog;
    property ActorName: String read GetActorName;
    property Width: Integer read GetStageWidth;
    property Height: Integer read GetStageHeight;
    property BoundsRect: TRect read GetStageBoundsRect;
    property ActorVerInfo[const InfoName: String]: String read GetActorVerInfo;
    property AnimationCount: Word read GetAnimationCount;
    property Animations[Index: Word]: TAnimation read GetAnimations;
  published
    property About: TAboutMe read fAbout write fAbout stored False;
    property ActorLibrary: String read GetActorLibrary write SetActorLibrary;
    property AutoIdle: Boolean read GetAutoIdle write SetAutoIdle default True;
    property BiDiMode: TBiDiMode read GetBiDiMode write SetBiDiMode default bdLeftToRight;
    property Draggable: Boolean read GetDraggable write SetDraggable default True;
    property DockSite: TWinControl read fDockSite write SetDockSite;
    property FastShowHide: Boolean read fFastShowHide write fFastShowHide default False;
    property StageLeft: Integer read GetStageLeft write SetStageLeft default 0;
    property StageTop: Integer read GetStageTop write SetStageTop default 0;
    property HintBulb: Boolean read fHintBulb write SetHintBulb default False;
    property PopupMenu: TPopupMenu read GetPopupMenu write SetPopupMenu;
    property Priority: TThreadPriority read GetPriority write SetPriority default tpNormal;
    property MoveStep: Byte read fMoveStep write SetMoveStep default 8;
    property Mute: Boolean read GetMute write SetMute default False;
    property Speed: Word read GetSpeed write SetSpeed default 100;
    property Balloon: TBalloonOptions read GetBalloonOptions write SetBalloonOptions;
    property MsgDlg: TMsgDlgOptions read GetMsgDlgOptions write SetMsgDlgOptions;
    property TipsDlg: TTipsDlgOptions read GetTipsDlgOptions write SetTipsDlgOptions;
    property GuideDlg: TGuideDlgOptions read GetGuideDlgOptions write SetGuideDlgOptions;
    property HintDlg: THintDlgOptions read GetHintDlgOptions write SetHintDlgOptions;
    property Visible: Boolean read fVisible write SetVisible default False;
    property OnAdjustCustomLabel: TCustomLabelEvent read fOnAdjustCustomLabel write fOnAdjustCustomLabel;
    property OnBalloonShow: TNotifyEvent read fOnBalloonShow write fOnBalloonShow;
    property OnBalloonHide: TNotifyEvent read fOnBalloonHide write fOnBalloonHide;
    property OnChange: TNotifyEvent read fOnChange write fOnChange;
    property OnClick: TNotifyEvent read fOnClick write fOnClick;
    property OnDblClick: TNotifyEvent read fOnDblClick write fOnDblClick;
    property OnDragging: TNotifyEvent read fOnDragging write fOnDragging;
    property OnDragStart: TNotifyEvent read fOnDragStart write fOnDragStart;
    property OnDragStop: TNotifyEvent read fOnDragStop write fOnDragStop;
    property OnHide: TNotifyEvent read fOnHide write fOnHide;
    property OnHintBulbClick: TNotifyEvent read fOnHintBulbClick write fOnHintBulbClick;
    property OnHintTopicClick: TTopicClickEvent read fOnHintTopicClick write fOnHintTopicClick;
    property OnIdle: TIdleEvent read fOnIdle write fOnIdle;
    property OnGuideRequestChange: TNotifyEvent read fOnGuideRequestChange write fOnGuideRequestChange;
    property OnGuideTopicClick: TTopicClickEvent read fOnGuideTopicClick write fOnGuideTopicClick;
    property OnGuideSearchClick: TNotifyEvent read fOnGuideSearchClick write fOnGuideSearchClick;
    property OnGuideOptionsClick: TNotifyEvent read fOnGuideOptionsClick write fOnGuideOptionsClick;
    property OnPlayStart: TPlayEvent read fOnPlayStart write fOnPlayStart;
    property OnPlayStop: TPlayEvent read fOnPlayStop write fOnPlayStop;
    property OnResize: TNotifyEvent read fOnResize write fOnResize;
    property OnShow: TNotifyEvent read fOnShow write fOnShow;
    property OnTipsTipChange: TNotifyEvent read fOnTipsTipChange write fOnTipsTipChange;
  end;

implementation

uses
  oaEngine;

constructor TAssistant.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Director := TAssistantDirector.Create;
  Director.Engine.OnFrameChange := FrameChanged;
  Director.OnPlayStart := PlayStarted;
  Director.OnPlayStop := PlayStopped;
  Director.OnIdle := IdleEntered;
  Stage := TAssistantStage.Create(Director);
  Stage.SetBounds(0, 0, 0, 0);
  Stage.Draggable := True;
  Stage.OnShow := StageShow;
  Stage.OnHide := StageHide;
  Stage.OnClick := StageClick;
  Stage.OnDblClick := StageDblClick;
  Stage.OnMouseUp := StageMouseUp;
  Stage.OnMoving := StageMoving;
  Stage.OnResize := StageResize;
  Stage.OnDeactivate := StageDeactivate;
  Stage.HintBulb.OnClick := HintBulbClick;
  DefaultDialog := TAssistantBalloon.Create(nil);
  DefaultDialog.Anchor := Stage;
  DefaultDialog.OnBalloonHide := BalloonHide;
  DefaultDialog.OnBalloonShow := BalloonShow;
  DefaultDialog.OnCustomLabel := AdjustCustomLabel;
  DefaultDialog.TipsDlgOptions.OnTipsTipChange := TipsTipChanged;
  DefaultDialog.GuideDlgOptions.OnGuideRequestChange := GuideRequestChanged;
  DefaultDialog.GuideDlgOptions.OnGuideTopicClick := GuideTopicClicked;
  DefaultDialog.GuideDlgOptions.OnGuideSearchClick := GuideSearchClicked;
  DefaultDialog.GuideDlgOptions.OnGuideOptionsClick := GuideOptionsClicked;
  DefaultDialog.GuideDlgOptions.OnGuideGetBtnStates := GuideGetBtnStates;
  DefaultDialog.HintDlgOptions.OnHintTopicClick := HintTopicClicked;
  Dialog := DefaultDialog;
  fMoveStep := 8;
end;

destructor TAssistant.Destroy;
begin
  Director.Terminate;
  Director.WaitFor;
  DefaultDialog.Free;
  Stage.Free;
  Director.Free;
  inherited Destroy;
end;

procedure TAssistant.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited Notification(Acomponent, Operation);
  if (Operation = opRemove) and (AComponent = DockSite) then
    DockSite := nil;
end;

procedure TAssistant.SetCustomDialog(Value: TAssistantBalloon);
var
  CurVisible: Boolean;
  CurAnchor: TWinControl;
begin
  if fCustomDialog <> Value then
  begin
    CurVisible := Dialog.Visible;
    CurAnchor := Dialog.Anchor;
    if Assigned(fCustomDialog) then
    begin
      fCustomDialog.OnBalloonShow := nil;
      fCustomDialog.OnBalloonHide := nil;
      fCustomDialog.Visible := False;
      fCustomDialog.Anchor := nil;
    end;
    fCustomDialog := Value;
    if Assigned(FCustomDialog) then
    begin
      if Dialog <> DefaultDialog then
        Dialog := fCustomDialog;
      fCustomDialog.OnBalloonShow := BalloonShow;
      fCustomDialog.OnBalloonHide := BalloonHide;
      fCustomDialog.Anchor := CurAnchor;
      fCustomDialog.Visible := CurVisible;
    end
    else
      Dialog := DefaultDialog;
  end;
end;

procedure TAssistant.SetDockSite(Value: TWinControl);
begin
  if DockSite <> Value then
  begin
    if (DockSite <> nil) and (Value = nil) and
       (csDestroying in DockSite.ComponentState)
    then
      Stage.Visible := False;
    fDockSite := Value;
    if DockSite <> nil then
    begin
      FreeNotification(Self);
      Stage.ParentWindow := DockSite.Handle;
    end
    else
      Stage.ParentWindow := 0;
    Stage.RefreshRegion;
  end;
end;

procedure TAssistant.SetMute(Value: Boolean);
begin
  Director.Engine.Mute := Value;
end;

function TAssistant.GetMute: Boolean;
begin
  Result := Director.Engine.Mute;
end;

procedure TAssistant.SetAutoIdle(Value: Boolean);
begin
  Director.AutoIdle := Value;
end;

function TAssistant.GetAutoIdle: Boolean;
begin
  Result := Director.AutoIdle;
end;

procedure TAssistant.SetMoveStep(Value: Byte);
begin
  if Value <> 0 then fMoveStep := Value;
end;

procedure TAssistant.SetSpeed(Value: Word);
begin
  if Value <> 0 then Director.Engine.Speed := Value;
end;

function TAssistant.GetSpeed: Word;
begin
  Result := Director.Engine.Speed;
end;

procedure TAssistant.SetPriority(Value: TThreadPriority);
begin
  Director.Engine.Priority := Value;
end;

function TAssistant.GetPriority: TThreadPriority;
begin
  Result := Director.Engine.Priority;
end;

procedure TAssistant.SetActorLibrary(const Value: String);
begin
  if CompareText(ActorLibrary, Value) <> 0 then
    try
      try
        Director.Actor.LibraryName := Value;
      finally
        if not Director.Actor.Empty then
        begin
          Stage.Perform(WM_QUERYNEWPALETTE, 0, 0);
          Stage.SetBounds(Stage.Left, Stage.Top,
            Director.Actor.Width, Director.Actor.Height);
          if Stage.Visible then
            Director.PlayAction(aaRestPose)
          else if Visible then
            Show(True);
        end
        else if Stage.Visible then
        begin
          Hide(True);
          fVisible := True;
        end;
        if Assigned(fOnChange) then
          fOnChange(Self);
      end;
    except
      if not (csLoading in ComponentState) then raise;
    end;
end;

function TAssistant.GetActorLibrary: String;
begin
  Result := Director.Actor.LibraryName;
end;

function TAssistant.GetActorName: String;
begin
  Result := Director.Actor.ActorName;
end;

function TAssistant.GetActorVerInfo(const InfoName: String): String;
begin
  Result := Director.Actor.VersionInfo(InfoName);
end;

function TAssistant.GetAnimationCount: Word;
begin
  Result := Director.Actor.AnimationCount;
end;

function TAssistant.GetAnimations(Index: Word): TAnimation;
begin
  Result := Director.Actor.Animations[Index];
end;

procedure TAssistant.SetHintBulb(Value: Boolean);
begin
  if HintBulb <> Value then
  begin
    fHintBulb := Value;
    Stage.HintBulb.Visible :=
      HintBulb and not MovingTo and not (VisibleBalloon in [bkHint, bkTips]);
  end;
end;

procedure TAssistant.SetPopupMenu(Value: TPopupMenu);
begin
  Stage.PopupMenu := Value;
end;

function TAssistant.GetPopupMenu: TPopupMenu;
begin
  Result := Stage.PopupMenu;
end;

procedure TAssistant.SetBiDiMode(Value: TBiDiMode);
begin
  {$IFDEF DELPHI4_UP}
  Stage.BiDiMode := Value;
  Dialog.BiDiMode := Value;
  {$ENDIF}
end;

function TAssistant.GetBiDiMode: TBiDiMode;
begin
  {$IFDEF DELPHI4_UP}
  Result := Stage.BiDiMode;
  {$ELSE}
  Result := bdLeftToRight;
  {$ENDIF}
end;

procedure TAssistant.SetStageLeft(Value: Integer);
begin
  if Stage.Left <> Value then
  begin
    MovingTo := True;
    try
      Stage.Left := Value;
      if (Dialog.Anchor = Stage) and Dialog.Visible then
        Dialog.Prepare;
    finally
      MovingTo := False;
    end;
  end;
end;

function TAssistant.GetStageLeft: Integer;
begin
  Result := Stage.Left;
end;

procedure TAssistant.SetStageTop(Value: Integer);
begin
  if Stage.Top <> Value then
  begin
    MovingTo := True;
    try
      Stage.Top := Value;
      if (Dialog.Anchor = Stage) and Dialog.Visible then
        Dialog.Prepare;
    finally
      MovingTo := False;
    end;
  end;
end;

function TAssistant.GetStageTop: Integer;
begin
  Result := Stage.Top;
end;

procedure TAssistant.SetPosition(X, Y: Integer);
begin
  if (Stage.Left <> X) or (Stage.Top <> Y) then
  begin
    MovingTo := True;
    try
      Stage.SetBounds(X, Y, Stage.Width, Stage.Height);
      if (Dialog.Anchor = Stage) and Dialog.Visible then
        Dialog.Prepare;
    finally
      MovingTo := False;
    end;
  end;
end;

function TAssistant.GetStageWidth: Integer;
begin
  Result := Stage.Width;
end;

function TAssistant.GetStageHeight: Integer;
begin
  Result := Stage.Height;
end;

function TAssistant.GetStageBoundsRect: TRect;
begin
  Result := Stage.BoundsRect;
end;

function TAssistant.GetShowing: Boolean;
begin
  Result := Stage.Visible;
end;

procedure TAssistant.SetDraggable(Value: Boolean);
begin
  Stage.Draggable := Value;
end;

function TAssistant.GetDraggable: Boolean;
begin
  Result := Stage.Draggable;
end;

procedure TAssistant.SetVisible(Value: Boolean);
begin
  if Visible <> Value then
  begin
    fVisible := Value;
    if Visible <> Stage.Visible then
      if not Visible then
        Hide(FastShowHide)
      else if not Director.Actor.Empty then
        Show(FastShowHide);
  end;
end;

procedure TAssistant.SetBalloonOptions(Value: TBalloonOptions);
begin
  if Value <> nil then DefaultDialog.Options.Assign(Value);
end;

function TAssistant.GetBalloonOptions: TBalloonOptions;
begin
  Result := DefaultDialog.Options;
end;

procedure TAssistant.SetMsgDlgOptions(Value: TMsgDlgOptions);
begin
  if Value <> nil then DefaultDialog.MsgDlgOptions.Assign(Value);
end;

function TAssistant.GetMsgDlgOptions: TMsgDlgOptions;
begin
  Result := DefaultDialog.MsgDlgOptions;
end;

procedure TAssistant.SetTipsDlgOptions(Value: TTipsDlgOptions);
begin
  if Value <> nil then DefaultDialog.TipsDlgOptions.Assign(Value);
end;

function TAssistant.GetTipsDlgOptions: TTipsDlgOptions;
begin
  Result := DefaultDialog.TipsDlgOptions;
end;

procedure TAssistant.SetGuideDlgOptions(Value: TGuideDlgOptions);
begin
  if Value <> nil then DefaultDialog.GuideDlgOptions.Assign(Value);
end;

function TAssistant.GetGuideDlgOptions: TGuideDlgOptions;
begin
  Result := DefaultDialog.GuideDlgOptions;
end;

procedure TAssistant.SetHintDlgOptions(Value: THintDlgOptions);
begin
  if Value <> nil then DefaultDialog.HintDlgOptions.Assign(Value);
end;

function TAssistant.GetHintDlgOptions: THintDlgOptions;
begin
  Result := DefaultDialog.HintDlgOptions;
end;

procedure TAssistant.SetBalloonAnchor(Value: TWinControl);
begin
  if Value = nil then
    Value := Stage;
  DefaultDialog.Anchor := Value;
  if fCustomDialog <> nil then
    fCustomDialog.Anchor := Value;
end;

function TAssistant.GetBalloonAnchor: TWinControl;
begin
  if Dialog.Anchor <> Stage then
    Result := Dialog.Anchor
  else
    Result := nil;
end;

procedure TAssistant.FrameChanged(Sender: TObject);
begin
  if Director.Enabled then Stage.RefreshRegion;
end;

procedure TAssistant.PlayStarted(Sendr: TObject; RequestID: TRequestID);
begin
  if Assigned(fOnPlayStart) then
    fOnPlayStart(Self, RequestID);
end;

procedure TAssistant.PlayStopped(Sendr: TObject; RequestID: TRequestID);
begin
  if Assigned(fOnPlayStop) and not (csDestroying in ComponentState) then
    fOnPlayStop(Self, RequestID);
end;

procedure TAssistant.IdleEntered(Sendr: TObject; Level: Integer;
  var Handled: Boolean);
begin
  if not Stage.Visible then
    Handled := True
  else if Assigned(fOnIdle) then
    fOnIdle(Self, Level, Handled);
end;

procedure TAssistant.StageMoving(Sender: TObject);
begin
  if not MovingTo then
    if not fDragging then
    begin
      fDragging := True;
      Director.IdleLevel := Low(TIdleLevel);
      if Dialog.Anchor = Stage then
        Dialog.InternalHide := True;
      if Assigned(fOnDragStart) then
        fOnDragStart(Self);
    end
    else if Assigned(fOnDragging) then
      fOnDragging(Self);
end;

procedure TAssistant.StageResize(Sender: TObject);
begin
  if not Director.Actor.Empty and Assigned(fOnResize) then
    fOnResize(Self);
end;

procedure TAssistant.StageMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if fDragging then
  begin
    fDragging := False;
    if Dialog.Anchor = Stage then
      Dialog.InternalHide := False;
    if Assigned(fOnDragStop) then
      fOnDragStop(Self);
  end;
  if Dialog.Visible and not (Dialog.BalloonKind in [bkSpeak, bkThink]) then
    Windows.SetForegroundWindow(Dialog.Handle);
  Director.IdleLevel := Low(TIdleLevel);
end;

procedure TAssistant.StageDeactivate(Sender: TObject);
begin
  if Dialog.IsModal and (Screen.ActiveForm <> Dialog) then
  begin
    if Dialog.Visible then
      SetForegroundWindow(Dialog.Handle)
    else
      SetForegroundWindow(Stage.Handle);
    MessageBeep(0);
  end;
end;

procedure TAssistant.StageShow(Sender: TObject);
begin
  if Assigned(fOnShow) then
    fOnShow(Self);
  if (Dialog.Anchor = Stage) and Dialog.Visible then
    Dialog.Prepare;
end;

procedure TAssistant.StageHide(Sender: TObject);
begin
  if not (csDestroying in ComponentState) then
  begin
    if (Dialog.Anchor = Stage) and Dialog.Visible then
      Dialog.Close;
    if Assigned(fOnHide) then
      fOnHide(Self);
  end;
end;

procedure TAssistant.HintBulbClick(Sender: TObject);
begin
  if Assigned(fOnHintBulbClick) then
    fOnHintBulbClick(Self);
end;

procedure TAssistant.StageClick(Sender: TObject);
begin
  if not (Dialog.IsModal or HiddingPhase) and Assigned(fOnClick) then
    fOnClick(Self);
end;

procedure TAssistant.StageDblClick(Sender: TObject);
begin
  if not (Dialog.IsModal or HiddingPhase) and Assigned(fOnDblClick) then
    fOnDblClick(Self);
end;

procedure TAssistant.Show(Fast: Boolean);
begin
  if not ShowingPhase and not Stage.Visible then
  begin
    ShowingPhase := True;
    try
      fVisible := True;
      Director.Enabled := True;
      if Fast or (Director.PlayAction(aaShow) = 0) then
        Director.PlayAction(aaRestPose);
      Stage.HintBulb.Visible := HintBulb and not (VisibleBalloon in [bkHint, bkTips]);
      Stage.Visible := True;
    finally
      ShowingPhase := False;
    end;
  end;
end;

procedure TAssistant.Hide(Fast: Boolean);
begin
  if not HiddingPhase and Stage.Visible then
  begin
    HiddingPhase := True;
    try
      Stage.HintBulb.Visible := False;
      if (Dialog.Anchor = Stage) and Dialog.Visible then
        Dialog.Hide;
      Director.StopAll;
      if not Fast then
        Director.WaitForRequest(Director.PlayAction(aaHide));
      Director.Enabled := False;
      fDragging := False;
      fVisible := False;
      Stage.Hide;
    finally
      HiddingPhase := False;
    end;
  end;
end;

{$IFDEF DELPHI4_UP}
function TAssistant.Play(Index: Word): TRequestID;
{$ELSE}
function TAssistant.PlayIndex(Index: Word): TRequestID;
{$ENDIF}
begin
  if not HiddingPhase then
    Result := Director.PlayIndex(Index)
  else
    Result := 0;
end;

{$IFDEF DELPHI4_UP}
function TAssistant.Play(const Name: String): TRequestID;
{$ELSE}
function TAssistant.PlayName(Name: String): TRequestID;
{$ENDIF}
begin
  if not HiddingPhase then
    Result := Director.PlayName(Name)
  else
    Result := 0;
end;

{$IFDEF DELPHI4_UP}
function TAssistant.Play(Action: TActorAction): TRequestID;
{$ELSE}
function TAssistant.PlayAction(Action: TActorAction): TRequestID;
{$ENDIF}
begin
  if not HiddingPhase then
    Result := Director.PlayAction(Action)
  else
    Result := 0;
end;

{$IFDEF DELPHI4_UP}
function TAssistant.Play(const Actions: array of TActorAction): TRequestID;
{$ELSE}
function TAssistant.PlayRandom(const Actions: array of TActorAction): TRequestID;
{$ENDIF}
begin
  if not HiddingPhase then
    Result := Director.PlayRandom(Actions)
  else
    Result := 0;
end;

{$IFNDEF DELPHI4_UP}
function TAssistant.Play(V: Variant): TRequestID;
begin
  if not HiddingPhase then
    case VarType(V) and varTypeMask of
      varString:
        Result := PlayName(String(V));
      varByte, varInteger:
        if TActorAction(V) in [Low(TActorAction)..High(TActorAction)] then
          Result := PlayAction(TActorAction(V))
        else
          Result := PlayIndex(Integer(V));
    else
      Result := 0;
    end
  else
    Result := 0;
end;
{$ENDIF}

function TAssistant.WaitFor(RequestID: TRequestID): Boolean;
begin
  if RequestID = 0 then
    Result := Director.WaitForRequest(Director.PlayingID)
  else
    Result := Director.WaitForRequest(RequestID);
end;

procedure TAssistant.Stop(RequestID: TRequestID);
begin
  if RequestID = 0 then
    Director.Stop(Director.PlayingID)
  else
    Director.Stop(RequestID);
end;

procedure TAssistant.StopAll;
begin
  Director.StopAll;
end;

function TAssistant.IsValid(RequestID: TRequestID): Boolean;
begin
  Result := Director.IsValidRequest(RequestID);
end;

function TAssistant.IsPlaying(Actions: TActorActions): Boolean;
begin
  Result := Director.IsPlaying(Actions);
end;

function TAssistant.GetEmpty: Boolean;
begin
  Result := Director.Actor.Empty;
end;

function TAssistant.GetPlaying: Boolean;
begin
  Result := Director.Playing;
end;

function TAssistant.GetPlayingID: TRequestID;
begin
  Result := Director.PlayingID;
end;

function TAssistant.GetPlayIdling: Boolean;
begin
  Result := Director.PlayIdling;
end;

function TAssistant.GetBalloonPos: TBalloonPos;
begin
  if Dialog.Visible then
    Result := Dialog.BalloonPos
  else
    Result := bpCenter;
end;

function TAssistant.GetVisibleBalloon: TBalloonKind;
begin
  if Dialog.Visible or Dialog.InternalHide then
    Result := Dialog.BalloonKind
  else
    Result := bkNone;
end;

function TAssistant.GetModalBalloon: Boolean;
begin
  if Dialog.Visible or Dialog.InternalHide then
    Result := Dialog.IsModal
  else
    Result := False;
end;

procedure TAssistant.CloseBalloon;
begin
  if Dialog.Visible then Dialog.Close
end;

function TAssistant.GestureAt(X, Y: Integer): TRequestID;
var
  Action: TActorAction;
  Origin: TPoint;
begin
  if Stage.Visible and not HiddingPhase then
  begin
    Origin.X := Stage.Left + Stage.Width div 2;
    Origin.Y := Stage.Top + Stage.Height div 2;
    Action := Director.ActionOfOrientation(oaGesture, Origin, Point(X, Y));
    Result := {$IFDEF DELPHI4_UP} Play {$ELSE} PlayAction {$ENDIF} (Action);
  end
  else
    Result := 0;
end;

function TAssistant.GlanceAt(X, Y: Integer): TRequestID;
var
  Action: TActorAction;
  Origin: TPoint;
begin
  if Stage.Visible and not HiddingPhase then
  begin
    Origin.X := Stage.Left + Stage.Width div 2;
    Origin.Y := Stage.Top + Stage.Height div 2;
    Action := Director.ActionOfOrientation(oaGlance, Origin, Point(X, Y));
    Result := {$IFDEF DELPHI4_UP} Play {$ELSE} PlayAction {$ENDIF} (Action);
  end
  else
    Result := 0;
end;

function TAssistant.LookAt(X, Y: Integer): TRequestID;
var
  Action: TActorAction;
  Origin: TPoint;
begin
  if Stage.Visible and not HiddingPhase then
  begin
    Origin.X := Stage.Left + Stage.Width div 2;
    Origin.Y := Stage.Top + Stage.Height div 2;
    Action := Director.ActionOfOrientation(oaLook, Origin, Point(X, Y));
    Result := {$IFDEF DELPHI4_UP} Play {$ELSE} PlayAction {$ENDIF} (Action);
  end
  else
    Result := 0;
end;

procedure TAssistant.MoveTo(X, Y: Integer);
var
  Action: TActorAction;
  BreakTime: DWORD;
begin
  MovingTo := True;
  try
    if Stage.Visible and not HiddingPhase and ((Stage.Left <> X) or (Stage.Top <> Y)) then
    begin
      Dialog.InternalHide := True;
      Stage.HintBulb.Visible := False;
      MoveDest := Point(X, Y);
      Action := Director.ActionOfOrientation(oaMove, Stage.BoundsRect.TopLeft, MoveDest);
      Director.StopAll;
      Director.WaitForRequest(Director.PlayAction(Action));
      Director.Enabled := False;
      try
        while MoveOneStep(FMoveStep, FMoveStep) do
        begin
          BreakTime := GetTickCount + 10;
          repeat
            Application.ProcessMessages;
          until GetTickCount > BreakTime;
        end;
      finally
        Director.Enabled := True;
        Director.PlayAction(aaRestPose);
        Dialog.InternalHide := False;
        Stage.HintBulb.Visible := HintBulb and not (VisibleBalloon in [bkHint, bkTips]);
      end;
    end;
  finally
    Stage.SetBounds(X, Y, Stage.Width, Stage.Height);
    MovingTo := False;
  end;
end;

function TAssistant.MoveOneStep(SX, SY: Integer): Boolean;
var
  Delta: TPoint;
  NewPos: TPoint;
begin
  NewPos := Point(Stage.Left, Stage.Top);
  Delta.X := MoveDest.X-Stage.Left;
  Delta.Y := MoveDest.Y-Stage.Top;
  if Abs(Delta.X) > Abs(Delta.Y) then
    SY := SX * Abs(Delta.Y) div Abs(Delta.X)
  else if Delta.Y <> 0 then
    SX := SY * Abs(Delta.X) div Abs(Delta.Y);
  if SX = 0 then SX := 1;
  if SY = 0 then SY := 1;
  if Delta.X <> 0 then
  begin
    if Delta.X > 0 then
    begin
      NewPos.X := Stage.Left + SX;
      if NewPos.X > MoveDest.X then
        NewPos.X := MoveDest.X;
    end
    else
    begin
      NewPos.X := Stage.Left - SX;
      if NewPos.X < MoveDest.X then
        NewPos.X := MoveDest.X;
    end;
  end;
  if Delta.Y <> 0 then
  begin
    if Delta.Y > 0 then
    begin
      NewPos.Y := Stage.Top + SY;
      if NewPos.Y > MoveDest.Y then
        NewPos.Y := MoveDest.Y;
    end
    else
    begin
      NewPos.Y := Stage.Top - SY;
      if NewPos.Y < MoveDest.Y then
        NewPos.Y := MoveDest.Y;
    end;
  end;
  Stage.SetBounds(NewPos.X, NewPos.Y, Stage.Width, Stage.Height);
  Result := (NewPos.X <> MoveDest.X) or (NewPos.Y <> MoveDest.Y);
end;

function TAssistant.InputQuery(const ACaption, APrompt: String;
  var Value: String): Boolean;
var
  Popup: TPopupMenu;
begin
  Dialog := DefaultDialog;
  Popup := Stage.PopupMenu;
  Stage.PopupMenu := nil;
  try
    Result := Dialog.InputQuery(ACaption, APrompt, Value);
  finally
    Stage.PopupMenu := Popup;
  end;
end;

function TAssistant.MessageDlg(const Msg: String; AType: TMsgDlgType;
  AButtons: TMsgDlgButtons; HelpCtx: LongInt): Word;
var
  Popup: TPopupMenu;
begin
  Dialog := DefaultDialog;
  Popup := Stage.PopupMenu;
  Stage.PopupMenu := nil;
  try
    Result := Dialog.ShowMsgDlg(Msg, AType, AButtons, HelpCtx);
  finally
    Stage.PopupMenu := Popup;
  end;
end;

procedure TAssistant.Speak(const Msg: String; Timeout: Integer);
begin
  Dialog := DefaultDialog;
  Dialog.ShowMessage(Msg, Timeout, bsSpeakShape);
end;

procedure TAssistant.Think(const Msg: String; Timeout: Integer);
begin
  Dialog := DefaultDialog;
  Dialog.ShowMessage(Msg, Timeout, bsThinkShape);
end;

procedure TAssistant.ShowTips;
begin
  Dialog := DefaultDialog;
  if Dialog.TipsDlgOptions.Tips.Count > 0 then
    Dialog.ShowTipsDlg;
end;

procedure TAssistant.ShowHint;
begin
  Dialog := DefaultDialog;
  Dialog.ShowHintDlg;
end;

procedure TAssistant.ShowGuide;
begin
  Dialog := DefaultDialog;
  Dialog.ShowGuideDlg;
end;

procedure TAssistant.ShowCustom;
begin
  Dialog := CustomDialog;
  Dialog.ShowCustomDlg;
end;

function TAssistant.ShowHintModal: Word;
var
  Popup: TPopupMenu;
begin
  Dialog := DefaultDialog;
  Popup := Stage.PopupMenu;
  Stage.PopupMenu := nil;
  try
    Result := Dialog.ShowHintDlgModal;
  finally
    Stage.PopupMenu := Popup;
  end;
end;

function TAssistant.ShowGuideModal: Word;
var
  Popup: TPopupMenu;
begin
  Dialog := DefaultDialog;
  Popup := Stage.PopupMenu;
  Stage.PopupMenu := nil;
  try
    Result := Dialog.ShowGuideDlgModal;
  finally
    Stage.PopupMenu := Popup;
  end;
end;

function TAssistant.ShowCustomModal: Word;
var
  Popup: TPopupMenu;
begin
  Dialog := CustomDialog;
  Popup := Stage.PopupMenu;
  Stage.PopupMenu := nil;
  try
    Result := Dialog.ShowCustomDlgModal;
  finally
    Stage.PopupMenu := Popup;
  end;
end;

procedure TAssistant.MoveBalloonTo(Point: TPoint);
begin
  Dialog.MoveTo(Point);
end;

procedure TAssistant.MoveBalloonToCursor;
var
  MousePos: TPoint;
begin
  GetCursorPos(MousePos);
  Dialog.MoveTo(MousePos);
end;

procedure TAssistant.MoveBalloonToControl(Control: TControl);
var
  P: TPoint;
begin
  if Assigned(Control) then
  begin
    P.X := 0;
    P.Y := 0;
    {$IFDEF DELPHI4_UP}
    if Stage.UseRightToLeftAlignment then
      P.X := Control.Width;
    {$ENDIF}
    P := Control.ClientToScreen(P);
    Dialog.MoveTo(P);
  end;
end;

procedure TAssistant.BalloonShow(Sender: TObject);
begin
  Director.IdleLevel := Low(TIdleLevel);
  if Dialog.BalloonKind in [bkHint, bkTips] then
    Stage.HintBulb.Visible := False;
  if Dialog.IsModal then
    Stage.HintBulb.Enabled := False;
  if Assigned(fOnBalloonShow) then
    fOnBalloonShow(Self);
end;

procedure TAssistant.BalloonHide(Sender: TObject);
begin
  Director.IdleLevel := Low(TIdleLevel);
  Stage.HintBulb.Enabled := Assigned(fOnHintBulbClick);
  Stage.HintBulb.Visible := HintBulb and not HiddingPhase;
  if Assigned(fOnBalloonHide) and not (csDestroying in ComponentState) then
    fOnBalloonHide(Self);
end;

procedure TAssistant.AdjustCustomLabel(Sender: TObject; TheLabel: TControl);
begin
  if Assigned(fOnAdjustCustomLabel) then
    fOnAdjustCustomLabel(Self, TheLabel);
end;

procedure TAssistant.TipsTipChanged(Sender: TObject);
begin
  Director.IdleLevel := Low(TIdleLevel);
  if Assigned(fOnTipsTipChange) then
    fOnTipsTipChange(Self);
end;

procedure TAssistant.GuideRequestChanged(Sender: TObject);
begin
  Director.IdleLevel := Low(TIdleLevel);
  if Assigned(fOnGuideRequestChange) then
    fOnGuideRequestChange(Self);
end;

procedure TAssistant.GuideTopicClicked(Sender: TObject; TopicIndex: Integer);
begin
  Director.IdleLevel := Low(TIdleLevel);
  if Assigned(fOnGuideTopicClick) then
    fOnGuideTopicClick(Self, TopicIndex);
end;

procedure TAssistant.GuideSearchClicked(Sender: TObject);
begin
  Director.IdleLevel := Low(TIdleLevel);
  if Assigned(FOnGuideSearchClick) then
    fOnGuideSearchClick(Self);
end;

procedure TAssistant.GuideOptionsClicked(Sender: TObject);
begin
  Director.IdleLevel := Low(TIdleLevel);
  if Assigned(fOnGuideOptionsClick) then
  begin
    Dialog.Close;
    fOnGuideOptionsClick(Self);
  end;
end;

procedure TAssistant.GuideGetBtnStates(Sender: TObject;
  var OptionsBtn, SearchBtn: Boolean);
begin
  OptionsBtn := OptionsBtn and Assigned(fOnGuideOptionsClick);
  SearchBtn := SearchBtn and Assigned(fOnGuideSearchClick);
end;

procedure TAssistant.HintTopicClicked(Sender: TObject; TopicIndex: Integer);
begin
  Director.IdleLevel := Low(TIdleLevel);
  if Assigned(fOnHintTopicClick) then
    fOnHintTopicClick(Self, TopicIndex);
end;

end.
