{------------------------------------------------------------------------------}
{                                                                              }
{  Office Assistant                                                            }
{  by Kambiz R. Khojasteh                                                      }
{                                                                              }
{  kambiz@delphiarea.com                                                       }
{  http://www.delphiarea.com                                                   }
{                                                                              }
{------------------------------------------------------------------------------}

{$I DELPHIAREA.INC}

unit oaStage;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, oaDirect;

type

  THintBulb = class;

  TAssistantStage = class(TCustomForm)
  private
    Director: TAssistantDirector;
    fHintBulb: THintBulb;
    fDraggable: Boolean;
    fOnMoving: TNotifyEvent;
    procedure WMWindowPosChanging(var Message: TWMWindowPosChanging); message WM_WINDOWPOSCHANGING;
    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure WMMoving(var Message: TMessage); message WM_MOVING;
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure WMNCHitTest(var Message: TWMNCHitTest); message WM_NCHITTEST;
    procedure WMNCLButtonDblClk(var Message: TWMMouse); message WM_NCLBUTTONDBLCLK;
    procedure WMNCLButtonDown(var Message: TWMMouse); message WM_NCLBUTTONDOWN;
    procedure WMNCLButtonUp(var Message: TWMMouse); message WM_NCLBUTTONUP;
    procedure WMNCRButtonUp(var Message: TWMMouse); message WM_NCRBUTTONUP;
    procedure WMMouseActivate(var Message: TWMMouseActivate); message WM_MOUSEACTIVATE;
    procedure WMEnable(var Message: TWMEnable); message WM_ENABLE;
    {$IFDEF DELPHI4_UP}
    procedure CMBiDiModeChanged(var Message: TMessage); message CM_BIDIMODECHANGED;
    {$ENDIF}
    function GetVisible: Boolean;
    procedure SetVisible(Value: Boolean);
  protected
    function GetPalette: HPALETTE; override;
    procedure Paint; override;
  public
    constructor Create(AssistantDirector: TAssistantDirector); {$IFDEF DELPHI4_UP} reintroduce; {$ENDIF}
    function CloseQuery: Boolean; {$IFDEF DELPHI4_UP} override; {$ENDIF}
    procedure RefreshRegion;
    property HintBulb: THintBulb read fHintBulb;
    property Draggable: Boolean read fDraggable write fDraggable;
    property Visible: Boolean read GetVisible write SetVisible;
    property OnMoving: TNotifyEvent read fOnMoving write fOnMoving;
    property PopupMenu;
    property OnClick;
    property OnDblClick;
    property OnShow;
    property OnHide;
    property OnMouseUp;
    property OnMouseDown;
    property OnActivate;
    property OnDeactivate;
    property OnResize;
  end;

  THintBulbState = (bsUp, bsDisabled, bsDown, bsHot);

  THintBulb = class(TCustomControl)
  private
    Glyphs: TBitmap;
    fRegion: HRGN;
    fState: THintBulbState;
    fMouseIsDown: Boolean;
    procedure SetState(Value: THintBulbState);
  protected
    procedure Paint; override;
    procedure CreateWnd; override;
    function GetPalette: HPALETTE; override;
    procedure CMVisibleChanged(var Message: TMessage); message CM_VISIBLECHANGED;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    property State: THintBulbState read fState write SetState;
    property MouseIsDown: Boolean read fMouseIsDown;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Region: HRGN read fRegion;
    property OnClick;
  end;

implementation

{$R *.RES}

uses
  oaBitmap, oaEngine;

{ THintBulb }

constructor THintBulb.Create(AOwner: TComponent);
var
  TmpBmp: TBitmap;
begin
  inherited Create(AOwner);
  Glyphs := TBitmap.Create;
  Glyphs.LoadFromResourceName(hInstance, 'BULBGLYPHS256');
  SetBounds(0, 0, Glyphs.Width div 4, Glyphs.Height);
  TmpBmp := TBitmap.Create;
  try
    TmpBmp.Width := Width;
    TmpBmp.Height := Height;
    BitBlt(TmpBmp.Canvas.Handle, 0, 0, Width, Height,
           Glyphs.Canvas.Handle, 0, 0, SRCCOPY);
    fRegion := CreateBitmapRgn(TmpBmp, TmpBmp.TransparentColor);
  finally
    TmpBmp.Free;
  end;
  fState := bsUp;
end;

destructor THintBulb.Destroy;
begin
  DeleteObject(Region);
  Glyphs.Free;
  inherited Destroy;
end;

procedure THintBulb.SetState(Value: THintBulbState);
begin
  if State <> Value then
  begin
    fState := Value;
    if Visible then
    begin
      InvalidateRect(WindowHandle, nil, False);
      UpdateWindow(WindowHandle);
    end;
  end;
end;

procedure THintBulb.Paint;
begin
  BitBlt(Canvas.Handle, 0, 0, Width, Height,
         Glyphs.Canvas.Handle, Ord(State) * Width, 0, SRCCOPY);
end;

procedure THintBulb.CreateWnd;
var
  TmpRgn: HRGN;
begin
  inherited CreateWnd;
  TmpRgn := CreateRectRgn(0, 0, 0, 0);
  CombineRgn(TmpRgn, Region, 0, RGN_COPY);
  SetWindowRgn(WindowHandle, TmpRgn, False);
end;

function THintBulb.GetPalette: HPALETTE;
begin
  Result := Glyphs.Palette;
end;

procedure THintBulb.CMVisibleChanged(var Message: TMessage);
begin
  inherited;
  if (Parent is TAssistantStage) and Parent.Visible then
    TAssistantStage(Parent).RefreshRegion;
end;

procedure THintBulb.CMEnabledChanged(var Message: TMessage);
var
  MousePos: TPoint;
begin
  inherited;
  fMouseIsDown := False;
  if not Enabled then
    State := bsDisabled
  else
  begin
    GetCursorPos(MousePos);
    if WindowFromPoint(MousePos) = WindowHandle then
      State := bsHot
    else
      State := bsUp;
  end;
end;

procedure THintBulb.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if Enabled then
  begin
    fMouseIsDown := (GetCapture = WindowHandle);
    if MouseIsDown then
      State := bsDown
    else
      State := bsHot;
  end;
end;

procedure THintBulb.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  if Enabled then State := bsUp;
end;

procedure THintBulb.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Enabled and (Button = mbLeft) then
  begin
    fMouseIsDown := True;
    State := bsDown;
  end;
end;

procedure THintBulb.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if Enabled and (Button = mbLeft) and fMouseIsDown then
  begin
    fMouseIsDown := False;
    State := bsUp;
  end;
end;

{ TAssistantStage }

constructor TAssistantStage.Create(AssistantDirector: TAssistantDirector);
begin
  inherited CreateNew(nil);
  inherited Visible := False;
  Director := AssistantDirector;
  fHintBulb := THintBulb.Create(Self);
  fHintBulb.Parent := Self;
  fHintBulb.Visible := False;
  fDraggable := True;
  BorderStyle := bsNone;
  FormStyle := fsStayOnTop;
  {$IFDEF DELPHI4_UP}
  DefaultMonitor := dmDesktop;
  {$ENDIF}
end;

function TAssistantStage.GetPalette: HPALETTE;
begin
  Result := Director.Actor.Palette;
end;

function TAssistantStage.CloseQuery: Boolean;
begin
  Result := Director.Actor.Empty;
end;

procedure TAssistantStage.WMSize(var Message: TWMSize);
begin
  if HintBulb <> nil then
    HintBulb.Left := Width - HintBulb.Width;
  if Visible then
    RefreshRegion;
  inherited;
end;

procedure TAssistantStage.WMMoving(var Message: TMessage);
begin
  inherited;
  if Assigned(fOnMoving) then
    fOnMoving(Self);
end;

procedure TAssistantStage.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
  Message.Result := 1;
end;

procedure TAssistantStage.WMNCHitTest(var Message: TWMNCHitTest);
begin
  inherited;
  if not Enabled then
    Message.Result := htNoWhere
  else if (Message.Result = htClient) and Draggable then
    Message.Result := htCaption;
end;

procedure TAssistantStage.WMNCLButtonDblClk(var Message: TWMMouse);
var
  MousePos: TPoint;
begin
  inherited;
  MousePos := ScreenToClient(SmallPointToPoint(Message.Pos));
  if PtInRect(ClientRect, MousePos) then DblClick;
end;

procedure TAssistantStage.WMNCLButtonDown(var Message: TWMMouse);
var
  MousePos: TPoint;
begin
  inherited;
  MousePos := ScreenToClient(SmallPointToPoint(Message.Pos));
  if Assigned(OnMouseUp) then
    MouseUp(mbLeft, [ssLeft], Message.Pos.X, Message.Pos.Y);
  if PtInRect(ClientRect, MousePos) then Click;
end;

procedure TAssistantStage.WMNCLButtonUp(var Message: TWMMouse);
begin
  inherited;
  if Assigned(OnMouseUp) then
    MouseUp(mbLeft, [ssLeft], Message.Pos.X, Message.Pos.Y);
end;

procedure TAssistantStage.WMNCRButtonUp(var Message: TWMMouse);
begin
  inherited;
  if Assigned(OnMouseUp) then
    MouseUp(mbRight, [ssRight], Message.Pos.X, Message.Pos.Y);
  if Assigned(PopupMenu) then
    PopupMenu.Popup(Message.XPos, Message.YPos);
end;

procedure TAssistantStage.WMMouseActivate(var Message: TWMMouseActivate);
begin
  Message.Result := MA_NOACTIVATE;
end;

procedure TAssistantStage.WMEnable(var Message: TWMEnable);
begin
  inherited;
  if not Message.Enabled and Visible and (ParentWindow = 0) then
  begin
    SetWindowPos(WindowHandle, HWND_NOTOPMOST, 0, 0, 0, 0,
      SWP_NOSIZE or SWP_NOMOVE or SWP_NOACTIVATE or SWP_NOSENDCHANGING);
    EnableWindow(WindowHandle, True);
    Application.NormalizeAllTopMosts;
    SetWindowPos(WindowHandle, HWND_TOPMOST, 0, 0, 0, 0,
      SWP_NOSIZE or SWP_NOMOVE or SWP_NOACTIVATE or SWP_NOSENDCHANGING);
  end;
end;

procedure TAssistantStage.WMWindowPosChanging(var Message: TWMWindowPosMsg);
begin
  inherited;
  if Visible and (ParentWindow = 0) then
  begin
    with Message.WindowPos^ do
    begin
      if (flags and SWP_NOZORDER) = 0 then
        hwndInsertAfter := HWND_TOP;
      flags := flags or SWP_NOACTIVATE;
    end;
  end;
end;

{$IFDEF DELPHI4_UP}
procedure TAssistantStage.CMBiDiModeChanged(var Message: TMessage);
begin
  inherited;
  if Assigned(PopupMenu) then PopupMenu.BiDiMode := BiDiMode;
end;
{$ENDIF}

function TAssistantStage.GetVisible: Boolean;
begin
  Result := inherited Visible;
end;

procedure TAssistantStage.SetVisible(Value: Boolean);
begin
  if Visible <> Value then
  begin
    if Value then
    begin
      HandleNeeded;
      RefreshRegion;
      SetWindowPos(WindowHandle, HWND_TOPMOST, Left, Top, Width, Height,
        SWP_SHOWWINDOW or SWP_NOACTIVATE);
      UpdateWindow(WindowHandle);
    end;
    inherited Visible := Value;
  end;
end;

procedure TAssistantStage.Paint;
begin
  with Director.Engine do
  begin
    Bitmap.Canvas.Lock;
    try
      if not Bitmap.Empty then
        BitBlt(Canvas.Handle, 0, 0, Width, Height, Bitmap.Canvas.Handle, 0, 0, SRCCOPY);
    finally
      Bitmap.Canvas.Unlock;
    end;
  end;
end;

procedure TAssistantStage.RefreshRegion;
var
  TmpRgn: HRGN;
begin
  TmpRgn := CreateRectRgn(0, 0, 0, 0);
  if HintBulb.Visible then
  begin
    CombineRgn(TmpRgn, HintBulb.Region, 0, RGN_COPY);
    OffsetRgn(TmpRgn, HintBulb.Left, HintBulb.Top);
    if Director.Engine.Region <> 0 then
      CombineRgn(TmpRgn, TmpRgn, Director.Engine.Region, RGN_OR);
  end
  else if Director.Engine.Region <> 0 then
    CombineRgn(TmpRgn, Director.Engine.Region, 0, RGN_COPY);
  SetWindowRgn(WindowHandle, TmpRgn, True);
  InvalidateRect(WindowHandle, nil, True);
  UpdateWindow(WindowHandle);
end;

end.
