{*******************************************************}
{                     PHP4Delphi                        }
{               PHP - Delphi interface                  }
{                       v.2.2                           }
{ Author:                                               }
{ Serhiy Perevoznyk                                     }
{ serge_perevoznyk@hotmail.com                          }
{ http://users.chello.be/ws36637                        }
{*******************************************************}

{
You can download the latest version of PHP from
http://www.php.net/downloads.php
You have to download and install PHP separately.
It is not included in the package.

For more information on the PHP Group and the PHP project,
please see <http://www.php.net>.
The PHP Licence is available at through the world-wide-web at
http://www.php.net/license/2_02.txt.

This unit was tested with PHP 4.2.3
}

unit psvPHP;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  php4Delphi;
type

  TPHPVariable = class(TCollectionItem)
  private
    FName : string;
    FValue : string;
  published
    property Name  : string read FName write FName;
    property Value : string read FValue write FValue;
  end;

  TPHPVariables = class(TCollection)
  private
    FOwner : TComponent;
    procedure SetItem(Index: Integer; const Value: TPHPVariable);
    function  GetItem(Index: Integer): TPHPVariable;
  protected
    function  GetOwner : TPersistent; override;
  public
    function Add: TPHPVariable;
    constructor Create(AOwner: TComponent);
    function GetVariables : string;
    function IndexOf(AName : string) : integer;
    procedure AddRawString(AString : string);
    property Items[Index: Integer]: TPHPVariable read GetItem write SetItem; default;
  end;

  TpsvPHP = class(TComponent)
  private
    FVariables : TPHPVariables;
    FBuffer : string;
    FFileName : string;
    procedure SetVariables(Value : TPHPVariables);
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    function Execute : string;
    function RunCode(ACode : string) : string;
  published
    property Variables : TPHPVariables read FVariables write SetVariables;
    property FileName : string read FFileName write FFileName;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Additional', [TpsvPHP]);
end;

{ TPHPVariables }

function TPHPVariables.Add: TPHPVariable;
begin
  result := TPHPVariable(inherited Add);
end;

constructor TPHPVariables.Create(AOwner: TComponent);
begin
 inherited create(TPHPVariable);
 FOwner := AOwner;
end;

function TPHPVariables.GetItem(Index: Integer): TPHPVariable;
begin
  Result := TPHPVariable(inherited GetItem(Index));
end;

procedure TPHPVariables.SetItem(Index: Integer; const Value: TPHPVariable);
begin
  inherited SetItem(Index, Value)
end;

function TPHPVariables.GetOwner : TPersistent;
begin
  Result := FOwner;
end;

function TPHPVariables.GetVariables: string;
var i : integer;
begin
  for i := 0 to Count - 1 do
    begin
      Result := Result + Items[i].FName + '=' + Items[i].FValue;
      if i < Count - 1 then
        Result := Result + '&';
    end;
end;

function TPHPVariables.IndexOf(AName: string): integer;
var
 i : integer;
begin
 Result := -1;
 for i := 0 to Count - 1 do
  begin
    if SameText(Items[i].Name, AName) then
     begin
       Result := i;
       break;
     end;
  end;
end;

procedure TPHPVariables.AddRawString(AString : string);
var
 SL : TStringList;
 i  : integer;
 j  : integer;
 V  :  TPHPVariable;
begin
  if AString[Length(AString)] = ';' then
   SetLength(AString, Length(AString)-1);
  SL := TStringList.Create;
  ExtractStrings([';'], [], PChar(AString), SL);
  for i := 0 to SL.Count - 1 do
   begin
     j := IndexOf(SL.Names[i]);
     if  j= -1 then
      begin
        V := Add;
        V.Name := SL.Names[i];
        V.Value := Copy(SL[I], Length(SL.Names[i]) + 2, MaxInt);
      end
       else
        begin
          Items[j].Value := Copy(SL[I], Length(SL.Names[i]) + 2, MaxInt);
        end;
   end;
  SL.Free;
end;

{ TpsvPHP }

constructor TpsvPHP.Create(AOwner: TComponent);
begin
  inherited;
  FVariables := TPHPVariables.Create(Self);
end;

destructor TpsvPHP.Destroy;
begin
  FVariables.Free;
  inherited;
end;

function TpsvPHP.Execute : string;
begin
  try
    if PHPLoaded then
     begin
       RegisterVariables(PChar(FVariables.GetVariables));
       FBuffer := RunPHPScript(FFileName);
       Result := FBuffer;
       FVariables.AddRawString(GetArray);
     end;
  except
    Result := '';
  end;
end;

function TpsvPHP.RunCode(ACode : string) : string;
begin
  try
    if PHPLoaded then
     begin
       RegisterVariables(PChar(FVariables.GetVariables));
       FBuffer := RunPHPCode(ACode);
       Result := FBuffer;
       FVariables.AddRawString(GetArray);
     end;
  except
    Result := '';
  end;
end;

procedure TpsvPHP.SetVariables(Value: TPHPVariables);
begin
  FVariables.Assign(Value);
end;

end.
