{@abstract(Some functions from this unit used by TARWordReport component)}
unit ARWordReportPerfom;

interface  

uses
  Windows, SysUtils, Classes;

  { Checking the SQL statement on forbidden clauses (not used - check it as example)}
  function CheckQuery(s: string): boolean;
  { Returns text enclosed by prefix and postfix in s}
  function GetTextBetween(s,prefix,postfix: string): string;
  { case insensitive string replace }
  function ReplaceUpStr(S, Srch, Replace: string): string;
  { calls 'ReplaceUpStr' for each string in ss}
  procedure ReplaceUpStrings(ss: TStrings; Srch, Replace: string);
  { Getting Date string in SQL format 'dd mmmm yyyy'}
  function DateToSQLA(d: TDateTime; Quoted: boolean): string;
  { Getting Windows temporary folder}
  function GetTempDir: string;
  { Searching for string enclosed by tag_beg and tag_end in Pool}
  function FindTag(Pool: string; tag_beg, tag_end: string; var start_pos, tag_length: integer): string;
  //function IncludeTrailingBackslash(const S: string): string;


implementation

{Service procedures}

function CheckQuery(s: string): boolean;
// Check SQL-query for forbidden clauses
// Example. Not used in ARWordReport.
  function IsGoodClauses(clause: string; postfix: array of string): boolean;
  // check every clause in s by compliance to template "clause postfix1|postfix2|..|postfixN"
  var
    i,j, l, pl: integer;
    txt: string;
  begin
    result:= True;
    clause:= AnsiUpperCase(clause);
    for j:= low(postfix) to high(postfix) do postfix[j]:= AnsiUpperCase(postfix[j]);
    txt:= AnsiUpperCase(s);
    l:= length(clause);
    i:= pos(clause,txt);
    While i>0 do begin
      Delete(txt,i,l);
      for j:= low(postfix) to high(postfix) do begin
        pl:= length(postfix[j]);
        result:= result or (pos(postfix[j],copy(txt,i,pl+2))>0);
        if result then break;
      end;
      if not result then break; // string is invalid
      i:= pos(clause,txt);
    end;
  end;
begin
  result:= True;
  result:= result and (pos('SELECT',AnsiUpperCase(s))>0);
  result:= result and IsGoodClauses('INTO',['#t']);  // #t- temporary table in MS SQL
  result:= result and IsGoodClauses('UPDATE',['#t']);
  result:= result and IsGoodClauses('DELETE',['#t']);
  result:= result and IsGoodClauses('DROP',['TABLE #t']);
  result:= result and IsGoodClauses('INSERT',['#t','INTO #t']);
  result:= result and not(pos('TRUNCATE',AnsiUpperCase(s))>0);
  result:= result and not(pos('ALTER',AnsiUpperCase(s))>0);
  result:= result and not(pos('CREATE',AnsiUpperCase(s))>0);
  result:= result and not(pos('EXEC',AnsiUpperCase(s))>0);
end;

function DateToSQLA(d: TDateTime; Quoted: boolean): string;
// Getting Date string in SQL format.
const
  Month4Sql: array [1..12] of String = ('Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec');
var
  y1,m1,d1: Word;
begin
  DecodeDate(d,y1,m1,d1);
  Result:=Month4Sql[m1]+' '+IntToStr(d1)+' '+IntToStr(y1);
  if Quoted then
    result:= '"'+result+'"';
end;

function FindTag(Pool: string; tag_beg, tag_end: string; var start_pos, tag_length: integer): string;
var
  lbeg, lend, end_pos: integer;
begin
  lbeg:= Length(tag_beg);
  lend:= Length(tag_end);
  start_pos:= pos(tag_beg,pool);
  tag_length:= 0;
  result:= '';
  if start_pos=0 then exit;
  Delete(Pool,1,start_pos-1);
  if (lbeg+lbeg<=Length(pool)) and (Copy(pool,lbeg+1,lbeg)=tag_beg) then begin
    Delete(pool,lbeg,lbeg+lbeg);
  end;
  end_pos:= pos(tag_end,pool);
  if end_pos>0 then begin
    result:= copy(pool,lbeg+1,end_pos-lbeg-1);
    tag_length:= end_pos+lend-1;
  end;
end;

function GetTextBetween(s,prefix,postfix: string): string;
var
  i: integer;
begin
  result:= '';
  i:= Pos(prefix,s);
  if i>0 then begin
    Delete(s,1,length(prefix)+i-1);
    i:= Pos(postfix,s);
    if i>0 then begin
      result:= Copy(s,1,i-1);
    end;
  end;
end;

function ReplaceUpStr(S, Srch, Replace: string): string;
var
  I,l: Integer;
  Source, upSource: string;
begin
  Result := '';
  if s='' then exit;
  Source := S;
  upSource:= AnsiUpperCase(Source);
  Srch:= AnsiUpperCase(Srch);
  l:= Length(Srch);
  repeat
    I := Pos(Srch, upSource);
    if I > 0 then begin
      Result := Result + Copy(Source, 1, I - 1) + Replace;
      Source := Copy(Source, I + l, MaxInt);
      upSource := Copy(upSource, I + l, MaxInt);
    end
    else Result := Result + Source;
  until I <= 0;
end;

procedure ReplaceUpStrings(ss: TStrings; Srch, Replace: string);
var
  i: integer;
begin
  for i:= 0 to ss.Count-1 do ss[i]:= ReplaceUpStr(ss[i],Srch,Replace);
end;

function GetTempDir: string;
var
  Buffer: array[0..1023] of Char;
begin
  SetString(Result, Buffer, GetTempPath(SizeOf(Buffer), Buffer));
end;
{
function IncludeTrailingBackslash(const S: string): string;
begin
  Result := S;
  if not IsPathDelimiter(Result, Length(Result)) then Result := Result + '\';
end;
}
end.



