/*
   +----------------------------------------------------------------------+
   | PHP Version 4                                                        |
   +----------------------------------------------------------------------+
   | Copyright (c) 1997-2002 The PHP Group                                |
   +----------------------------------------------------------------------+
   | This source file is subject to version 2.02 of the PHP license,      |
   | that is bundled with this package in the file LICENSE, and is        |
   | available at through the world-wide-web at                           |
   | http://www.php.net/license/2_02.txt.                                 |
   | If you did not receive a copy of the PHP license and are unable to   |
   | obtain it through the world-wide-web, please send a note to          |
   | license@php.net so we can mail you a copy immediately.               |
   +----------------------------------------------------------------------+
   | Author: Serhiy Perevoznyk <serge_perevoznyk@hotmail.com>             |
   +----------------------------------------------------------------------+
*/

#ifdef PHP_WIN32
# include <windows.h>
# include <process.h>
#else
# define __try
# define __except(val)
# define __declspec(foo)
#endif

#include <httpext.h>
#include <httpfilt.h>
#include <httpext.h>
#include "php.h"
#include "php_main.h"
#include "SAPI.h"
#include "php_globals.h"
#include "ext/standard/info.h"
#include "php_variables.h"
#include "php_ini.h"

#ifdef WITH_ZEUS
# include "httpext.h"
# include <errno.h>
# define GetLastError() errno
#endif

#ifdef PHP_WIN32
#define PHP_ENABLE_SEH
#endif


static char *Variables = " ";
static char *VarsStr;
static char *ArrayStr;

typedef void (WINAPI * writeclient) (char c);

static writeclient  WRI_FUNC; 



/* delphi_strcat: like strcat, but automatically expands dest string */
void delphi_strcat(char** dst, int* curSize, const char* src)
{
	int dstLen;
	int srcLen;

	srcLen = strlen(src);

	if (*dst == 0) {
		*curSize = srcLen+1;
		*dst = (char*) malloc(*curSize);
		strcpy(*dst, src);
		return;
	}

	dstLen = strlen(*dst);
	if (dstLen + srcLen + 1 > *curSize) {
		while (dstLen + srcLen + 1 > *curSize) {
			if(*curSize == 0) {
				*curSize = 1;
			}
			else {
				*curSize *= 2;
			}
		}
		*dst = realloc(*dst, *curSize);
	}

	strcat(*dst, src);
}

static void php_info_delphi(ZEND_MODULE_INFO_FUNC_ARGS)
{
  php_info_print_table_start();
  php_info_print_table_row(2, "SAPI module version", "PHP4Delphi 2.2 Nov 2002");
  php_info_print_table_row(2, "Supported method", "GET");
  php_info_print_table_row(2,"Variables support", "add 'register_globals = On' to php.ini");
  php_info_print_table_end();

}

static zend_module_entry php_delphi_module = {
    STANDARD_MODULE_HEADER,
	"DELPHI",
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	php_info_delphi,
    NULL,
	STANDARD_MODULE_PROPERTIES
};


static int sapi_delphi_ub_write(const char *str, uint str_length TSRMLS_DC)
{
uint i;
for (i=0; i < str_length; i++ )
{  WRI_FUNC(str[i]);}
 return str_length;
}


static int sapi_delphi_header_handler(sapi_header_struct *sapi_header, sapi_headers_struct *sapi_headers TSRMLS_DC)
{
	return SAPI_HEADER_ADD;
}



static int sapi_delphi_send_headers(sapi_headers_struct *sapi_headers TSRMLS_DC)
{
	return SAPI_HEADER_SENT_SUCCESSFULLY;
}


static int php_delphi_startup(sapi_module_struct *sapi_module)
{
	if (php_module_startup(sapi_module)==FAILURE
		|| zend_startup_module(&php_delphi_module)==FAILURE) {
		return FAILURE;
	} else {
		return SUCCESS;
	}
}



static int sapi_delphi_read_post(char *buffer, uint count_bytes TSRMLS_DC)
{
	return 0;
}


static char *sapi_delphi_read_cookies(TSRMLS_D)
{
	return NULL;
}



static void init_request_info(TSRMLS_D)
{
	SG(request_info).request_method = "GET";
	SG(request_info).query_string = Variables;
	SG(sapi_headers).http_response_code = 200;  
}

static void sapi_delphi_register_server_variables(zval *track_vars_array TSRMLS_DC)
{


    php_register_variable("GATEWAY_INTERFACE", "CGI/1.1",
			track_vars_array TSRMLS_CC);
    php_register_variable("REQUEST_METHOD", "GET",
			track_vars_array TSRMLS_CC);
    php_register_variable("QUERY_STRING", Variables,
			track_vars_array TSRMLS_CC);
    php_register_variable("SERVER_SOFTWARE","Delphi",
			track_vars_array TSRMLS_CC);
    php_register_variable("SERVER_NAME","Delphi",
			track_vars_array TSRMLS_CC);

}


static void sapi_delphi_dump_array(char *name, uint name_length TSRMLS_DC)
{
	zval **data, **tmp, tmp2;
	char *string_key;
	ulong num_key;
	int Size;

	Size = 0;

	if (zend_hash_find(&EG(symbol_table), name, name_length+1, (void **) &data)!=FAILURE
		&& (Z_TYPE_PP(data)==IS_ARRAY)) {
		zend_hash_internal_pointer_reset(Z_ARRVAL_PP(data));
		while (zend_hash_get_current_data(Z_ARRVAL_PP(data), (void **) &tmp) == SUCCESS) {
			switch (zend_hash_get_current_key(Z_ARRVAL_PP(data), &string_key, &num_key, 0)) {
				case HASH_KEY_IS_STRING:
					delphi_strcat(&ArrayStr,&Size, string_key);
					delphi_strcat(&ArrayStr, &Size, "=");
					break;
				case HASH_KEY_IS_LONG:
					
					break;
			}
			
			if (Z_TYPE_PP(tmp) == IS_ARRAY) {
			} else if (Z_TYPE_PP(tmp) != IS_STRING) {
				tmp2 = **tmp;
				zval_copy_ctor(&tmp2);
				convert_to_string(&tmp2);
				delphi_strcat(&ArrayStr, &Size,Z_STRVAL(tmp2));
    			delphi_strcat(&ArrayStr, &Size,";");
				zval_dtor(&tmp2);
			} else {
				delphi_strcat(&ArrayStr,&Size,Z_STRVAL_PP(tmp));
    			delphi_strcat(&ArrayStr, &Size,";");
			}
			zend_hash_move_forward(Z_ARRVAL_PP(data));
		}
	}
}


static sapi_module_struct delphi_sapi_module = {
	"delphi",						/* name */
	"PHP 4 DELPHI",					/* pretty name */
									
	php_delphi_startup,				/* startup */
	php_module_shutdown_wrapper,	/* shutdown */

	NULL,							/* activate */
	NULL,							/* deactivate */

	sapi_delphi_ub_write,			/* unbuffered write */
	NULL,							/* flush */
	NULL,							/* get uid */
	NULL,							/* getenv */

	php_error,						/* error handler */

	sapi_delphi_header_handler,		/* header handler */
	sapi_delphi_send_headers,		/* send headers handler */
	NULL,							/* send header handler */

	sapi_delphi_read_post,			/* read POST data */
	sapi_delphi_read_cookies,		/* read Cookies */

	sapi_delphi_register_server_variables,	/* register server variables */
	NULL,									/* Log message */

	NULL,									/* Block interruptions */
	NULL,									/* Unblock interruptions */

	STANDARD_SAPI_MODULE_PROPERTIES
};

void WINAPI RegisterVariables(char *var)
{
  Variables = var;
}




char *WINAPI GetArray()
{
	return ArrayStr;
}


void  WINAPI ExecutePHP(char *FileName, writeclient wri_callback)
{
	zend_file_handle file_handle;
	zend_bool stack_overflown=0;
	TSRMLS_FETCH();
	init_request_info(TSRMLS_C);
	WRI_FUNC = wri_callback;
	file_handle.filename = FileName;
	file_handle.free_filename = 0;
	file_handle.type = ZEND_HANDLE_FILENAME;
	file_handle.opened_path = NULL;
	php_request_startup(TSRMLS_C);
	ArrayStr = NULL;
	php_execute_script(&file_handle TSRMLS_CC);
	sapi_delphi_dump_array("_GET", sizeof("_GET")-1 TSRMLS_CC);
	php_request_shutdown(NULL);
}


void WINAPI ExecuteCode(char *PHPCode, writeclient wri_callback)
{

	TSRMLS_FETCH();
	init_request_info(TSRMLS_C);
	WRI_FUNC = wri_callback;
	php_request_startup(TSRMLS_C);
	ArrayStr = NULL;
	zend_eval_string(PHPCode, NULL, "Code from Delphi" TSRMLS_CC);
	sapi_delphi_dump_array("_GET", sizeof("_GET")-1 TSRMLS_CC);
	php_request_shutdown(NULL);
	 
}

__declspec(dllexport) BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
	switch (fdwReason) {
		case DLL_PROCESS_ATTACH:
#ifdef WITH_ZEUS
			tsrm_startup(1, 1, TSRM_ERROR_LEVEL_CORE, "TSRM.log");
#else
			tsrm_startup(1, 1, TSRM_ERROR_LEVEL_CORE, "C:\\TSRM.log");
#endif
			sapi_startup(&delphi_sapi_module);
			if (delphi_sapi_module.startup) {
				delphi_sapi_module.startup(&sapi_module);
			}
			break;
		case DLL_THREAD_ATTACH:
			break;
		case DLL_THREAD_DETACH:
			ts_free_thread();
			break;
		case DLL_PROCESS_DETACH:
			if (delphi_sapi_module.shutdown) {
				delphi_sapi_module.shutdown(&sapi_module);
			}
			tsrm_shutdown();
			break;
	}
	return TRUE;
}

